@@ -5,7 +5,7 @@
 #include <algorithm>
 #include <vector>
 
-std::map<wxString, int> CDirectoryListingParser::m_MonthNamesMap;
+std::map<std::wstring, int> CDirectoryListingParser::m_MonthNamesMap;
 
 //#define LISTDEBUG_MVS
 //#define LISTDEBUG
@@ -19,7 +19,7 @@ static char const data[][150]={
 namespace {
 struct ObjectCache
 {
-	CRefcountObject<fzstring> const& get(fzstring const& v)
+	fz::shared_value<std::wstring> const& get(std::wstring const& v)
 	{
 		auto it = std::lower_bound(cache.begin(), cache.end(), v);
 
@@ -32,7 +32,7 @@ struct ObjectCache
 	// Vector coupled with binary search and sorted insertion is fastest
 	// alternative as we expect a relatively low amount of inserts.
 	// Note that we cannot use set, as it it cannot search based on a different type.
-	std::vector<CRefcountObject<fzstring>> cache;
+	std::vector<fz::shared_value<std::wstring>> cache;
 };
 
 
@@ -58,12 +58,12 @@ public:
 		hex
 	};
 
-	CToken(wxChar const* p, unsigned int len)
+	CToken(wchar_t const* p, unsigned int len)
 		: m_pToken(p)
 		, m_len(len)
 	{}
 
-	wxChar const* GetToken() const
+	wchar_t const* GetToken() const
 	{
 		return m_pToken;
 	}
@@ -73,13 +73,13 @@ public:
 		return m_len;
 	}
 
-	fzstring GetString() const
+	std::wstring GetString() const
 	{
 		if (!m_pToken || !m_len) {
-			return fzstring();
+			return std::wstring();
 		}
 		else {
-			return fzstring(m_pToken, m_len);
+			return std::wstring(m_pToken, m_len);
 		}
 	}
 
@@ -113,7 +113,7 @@ public:
 
 	bool IsNumeric(unsigned int start, unsigned int len)
 	{
-		for (unsigned int i = start; i < wxMin(start + len, m_len); ++i)
+		for (unsigned int i = start; i < std::min(start + len, m_len); ++i)
 			if (m_pToken[i] < '0' || m_pToken[i] > '9')
 				return false;
 		return true;
@@ -145,7 +145,7 @@ public:
 		return m_rightNumeric == Yes;
 	}
 
-	int Find(const wxChar* chr, int start = 0) const
+	int Find(const wchar_t* chr, int start = 0) const
 	{
 		if (!chr)
 			return -1;
@@ -159,7 +159,7 @@ public:
 		return -1;
 	}
 
-	int Find(wxChar chr, int start = 0) const
+	int Find(wchar_t chr, int start = 0) const
 	{
 		if (!m_pToken)
 			return -1;
@@ -227,7 +227,7 @@ public:
 			{
 				int64_t number = 0;
 				for (unsigned int i = 0; i < m_len; ++i) {
-					const wxChar& c = m_pToken[i];
+					const wchar_t& c = m_pToken[i];
 					if (c >= '0' && c <= '9') {
 						number *= 16;
 						number += c - '0';
@@ -248,7 +248,7 @@ public:
 		}
 	}
 
-	wxChar operator[](unsigned int n) const
+	wchar_t operator[](unsigned int n) const
 	{
 		if (n >= m_len)
 			return 0;
@@ -257,7 +257,7 @@ public:
 	}
 
 protected:
-	wxChar const* m_pToken{};
+	wchar_t const* m_pToken{};
 	unsigned int m_len{};
 
 	TokenInformation m_numeric{Unknown};
@@ -269,13 +269,13 @@ protected:
 class CLine
 {
 public:
-	CLine(wxChar* p, int len = -1, int trailing_whitespace = 0)
+	CLine(wchar_t* p, int len = -1, int trailing_whitespace = 0)
 	{
 		m_pLine = p;
 		if (len >= 0)
 			m_len = len;
 		else
-			m_len = wxStrlen(p);
+			m_len = fz::strlen(p);
 
 		m_parsePos = 0;
 
@@ -343,7 +343,7 @@ public:
 				CToken ref;
 				if (!GetToken(prev, ref))
 					return false;
-				wxChar const* p = ref.GetToken() + ref.GetLength() + 1;
+				wchar_t const* p = ref.GetToken() + ref.GetLength() + 1;
 
 				auto const newLen = m_len - (p - m_pLine);
 				if (newLen <= 0) {
@@ -364,7 +364,7 @@ public:
 
 			for (unsigned int i = static_cast<unsigned int>(m_LineEndTokens.size()); i <= n; ++i) {
 				const CToken *refToken = m_Tokens[i];
-				const wxChar* p = refToken->GetToken();
+				const wchar_t* p = refToken->GetToken();
 				auto const newLen = m_len - (p - m_pLine) - m_trailing_whitespace;
 				if (newLen <= 0) {
 					return false;
@@ -380,10 +380,10 @@ public:
 	CLine *Concat(const CLine *pLine) const
 	{
 		int newLen = m_len + pLine->m_len + 1;
-		wxChar* p = new wxChar[newLen];
-		memcpy(p, m_pLine, m_len * sizeof(wxChar));
+		wchar_t* p = new wchar_t[newLen];
+		memcpy(p, m_pLine, m_len * sizeof(wchar_t));
 		p[m_len] = ' ';
-		memcpy(p + m_len + 1, pLine->m_pLine, pLine->m_len * sizeof(wxChar));
+		memcpy(p + m_len + 1, pLine->m_pLine, pLine->m_len * sizeof(wchar_t));
 
 		return new CLine(p, m_len + pLine->m_len + 1, pLine->m_trailing_whitespace);
 	}
@@ -399,7 +399,7 @@ protected:
 	int m_parsePos;
 	int m_len;
 	int m_trailing_whitespace;
-	wxChar* m_pLine;
+	wchar_t* m_pLine;
 	unsigned int offset_{};
 };
 
@@ -599,20 +599,19 @@ CDirectoryListingParser::CDirectoryListingParser(CControlSocket* pControlSocket,
 
 		// Some servers send a combination of month name and number,
 		// Add corresponding numbers to the month names.
-		std::map<wxString, int> combo;
-		for (auto iter = m_MonthNamesMap.begin(); iter != m_MonthNamesMap.end(); ++iter)
-		{
+		std::map<std::wstring, int> combo;
+		for (auto iter = m_MonthNamesMap.begin(); iter != m_MonthNamesMap.end(); ++iter) {
 			// January could be 1 or 0, depends how the server counts
-			combo[wxString::Format(_T("%s%02d"), iter->first, iter->second)] = iter->second;
-			combo[wxString::Format(_T("%s%02d"), iter->first, iter->second - 1)] = iter->second;
+			combo[wxString::Format(_T("%s%02d"), iter->first, iter->second).ToStdWstring()] = iter->second;
+			combo[wxString::Format(_T("%s%02d"), iter->first, iter->second - 1).ToStdWstring()] = iter->second;
 			if (iter->second < 10)
-				combo[wxString::Format(_T("%s%d"), iter->first, iter->second)] = iter->second;
+				combo[wxString::Format(_T("%s%d"), iter->first, iter->second).ToStdWstring()] = iter->second;
 			else
-				combo[wxString::Format(_T("%s%d"), iter->first, iter->second % 10)] = iter->second;
+				combo[wxString::Format(_T("%s%d"), iter->first, iter->second % 10).ToStdWstring()] = iter->second;
 			if (iter->second <= 10)
-				combo[wxString::Format(_T("%s%d"), iter->first, iter->second - 1)] = iter->second;
+				combo[wxString::Format(_T("%s%d"), iter->first, iter->second - 1).ToStdWstring()] = iter->second;
 			else
-				combo[wxString::Format(_T("%s%d"), iter->first, (iter->second - 1) % 10)] = iter->second;
+				combo[wxString::Format(_T("%s%d"), iter->first, (iter->second - 1) % 10).ToStdWstring()] = iter->second;
 		}
 		m_MonthNamesMap.insert(combo.begin(), combo.end());
 
@@ -694,7 +693,7 @@ CDirectoryListing CDirectoryListingParser::Parse(const CServerPath &path)
 {
 	CDirectoryListing listing;
 	listing.path = path;
-	listing.m_firstListTime = CMonotonicClock::now();
+	listing.m_firstListTime = fz::monotonic_clock::now();
 
 	if (!ParseData(false)){
 		listing.m_flags |= CDirectoryListing::listing_failed;
@@ -723,8 +722,8 @@ CDirectoryListing CDirectoryListingParser::Parse(const CServerPath &path)
 
 bool CDirectoryListingParser::ParseLine(CLine &line, const enum ServerType serverType, bool concatenated)
 {
-	CRefcountObject<CDirentry> refEntry;
-	CDirentry & entry = refEntry.Get();
+	fz::shared_value<CDirentry> refEntry;
+	CDirentry & entry = refEntry.get();
 
 	bool res;
 	int ires;
@@ -833,7 +832,7 @@ done:
 	if (serverType == VMS && entry.is_dir()) {
 		// Trim version information from directories
 		auto pos = entry.name.rfind(';');
-		if (pos != fzstring::npos && pos > 0)
+		if (pos != std::wstring::npos && pos > 0)
 			entry.name = entry.name.substr(0, pos);
 	}
 
@@ -844,8 +843,8 @@ done:
 		if (line.GetToken(0, t)) {
 			int64_t seconds = t.GetNumber();
 			if (seconds > 0 && seconds <= 0xffffffffll) {
-				CDateTime time(static_cast<time_t>(seconds), CDateTime::seconds);
-				if (time.IsValid()) {
+				fz::datetime time(static_cast<time_t>(seconds), fz::datetime::seconds);
+				if (time.empty()) {
 					entry.time = time;
 				}
 			}
@@ -855,7 +854,7 @@ done:
 	{
 		auto const timezoneOffset = m_server.GetTimezoneOffset();
 		if (timezoneOffset) {
-			entry.time += duration::from_minutes(timezoneOffset);
+			entry.time += fz::duration::from_minutes(timezoneOffset);
 		}
 	}
 
@@ -876,7 +875,7 @@ bool CDirectoryListingParser::ParseAsUnix(CLine &line, CDirentry &entry, bool ex
 	if (!line.GetToken(index, token))
 		return false;
 
-	wxChar chr = token[0];
+	wchar_t chr = token[0];
 	if (chr != 'b' &&
 		chr != 'c' &&
 		chr != 'd' &&
@@ -886,7 +885,7 @@ bool CDirectoryListingParser::ParseAsUnix(CLine &line, CDirentry &entry, bool ex
 		chr != '-')
 		return false;
 
-	fzstring permissions = token.GetString();
+	std::wstring permissions = token.GetString();
 
 	entry.flags = 0;
 
@@ -921,7 +920,7 @@ bool CDirectoryListingParser::ParseAsUnix(CLine &line, CDirentry &entry, bool ex
 		// Reset index
 		index = startindex;
 
-		fzstring ownerGroup;
+		std::wstring ownerGroup;
 		for (int i = 0; i < numOwnerGroup; ++i) {
 			if (!line.GetToken(++index, token))
 				return false;
@@ -945,17 +944,17 @@ bool CDirectoryListingParser::ParseAsUnix(CLine &line, CDirentry &entry, bool ex
 			if (!ownerGroup.empty())
 				ownerGroup += _T(" ");
 
-			wxString const group = token.GetString();
+			std::wstring const group = token.GetString();
 			int i;
-			for( i = group.size() - 1;
+			for (i = group.size() - 1;
 				 i >= 0 && group[i] >= '0' && group[i] <= '9';
-				 --i ) {}
+				 --i) {}
 
-			ownerGroup += group.Left(i + 1);
+			ownerGroup += group.substr(0, i + 1);
 		}
 
 		if (expect_date) {
-			entry.time = CDateTime();
+			entry.time = fz::datetime();
 			if (!ParseUnixDateTime(line, index, entry))
 				continue;
 		}
@@ -975,8 +974,8 @@ bool CDirectoryListingParser::ParseAsUnix(CLine &line, CDirentry &entry, bool ex
 
 		if (entry.is_link()) {
 			size_t pos;
-			if ((pos = entry.name.find(_T(" -> "))) != fzstring::npos) {
-				entry.target = CSparseOptional<fzstring>(entry.name.substr(pos + 4));
+			if ((pos = entry.name.find(_T(" -> "))) != std::wstring::npos) {
+				entry.target = fz::sparse_optional<std::wstring>(entry.name.substr(pos + 4));
 				entry.name = entry.name.substr(0, pos);
 			}
 		}
@@ -1036,7 +1035,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 			if (token[pos] == '.')
 				return true;
 
-			tm t = entry.time.GetTm(CDateTime::utc);
+			tm t = entry.time.get_tm(fz::datetime::utc);
 			year = t.tm_year + 1900;
 			month = t.tm_mon + 1;
 			day = t.tm_mday;
@@ -1106,20 +1105,20 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 	}
 
 	if (month < 1) {
-		wxString strMonth = dateMonth.GetString();
-		if (dateMonth.IsLeftNumeric() && (unsigned int)strMonth[strMonth.Length() - 1] > 127) {
+		std::wstring strMonth = dateMonth.GetString();
+		if (dateMonth.IsLeftNumeric() && (unsigned int)strMonth[strMonth.size() - 1] > 127) {
 			// Most likely an Asian server sending some unknown language specific
 			// suffix at the end of the monthname. Filter it out.
 			int i;
-			for (i = strMonth.Length() - 1; i > 0; --i) {
+			for (i = strMonth.size() - 1; i > 0; --i) {
 				if (strMonth[i] >= '0' && strMonth[i] <= '9')
 					break;
 			}
-			strMonth = strMonth.Left(i + 1);
+			strMonth = strMonth.substr(0, i + 1);
 		}
 		// Check month name
-		while (strMonth.Right(1) == _T(",") || strMonth.Right(1) == _T("."))
-			strMonth.RemoveLast();
+		while (!strMonth.empty() && (strMonth.back() == ',' || strMonth.back() == '.'))
+			strMonth.pop_back();
 		if (!GetMonthFromName(strMonth, month))
 			return false;
 	}
@@ -1148,7 +1147,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 		// Some servers use times only for files newer than 6 months
 		if( year <= 0 ) {
 			wxASSERT( month != -1 && day != -1 );
-			tm const t = CDateTime::Now().GetTm(CDateTime::utc);
+			tm const t = fz::datetime::now().get_tm(fz::datetime::utc);
 			year = t.tm_year + 1900;
 			int const currentDayOfYear = t.tm_mday + 31 * t.tm_mon;
 			int const fileDayOfYear = day + 31 * (month - 1);
@@ -1204,7 +1203,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 	else
 		--index;
 
-	if (!entry.time.Set(CDateTime::utc, year, month, day, hour, minute)) {
+	if (!entry.time.set(fz::datetime::utc, year, month, day, hour, minute)) {
 		return false;
 	}
 
@@ -1233,7 +1232,7 @@ bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bo
 		// Seems to be monthname-dd-yy
 
 		// Check month name
-		wxString dateMonth = token.GetString().substr(0, pos);
+		std::wstring const dateMonth = token.GetString().substr(0, pos);
 		if (!GetMonthFromName(dateMonth, month))
 			return false;
 		gotMonth = true;
@@ -1309,7 +1308,7 @@ bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bo
 	if (gotYear || gotDay) {
 		// Month field in yyyy-mm-dd or dd-mm-yyyy
 		// Check month name
-		wxString dateMonth = token.GetString().substr(pos + 1, pos2 - pos - 1);
+		std::wstring dateMonth = token.GetString().substr(pos + 1, pos2 - pos - 1);
 		if (!GetMonthFromName(dateMonth, month))
 			return false;
 		gotMonth = true;
@@ -1348,7 +1347,7 @@ bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bo
 	wxASSERT(gotMonth);
 	wxASSERT(gotDay);
 
-	if (!entry.time.Set(CDateTime::utc, year, month, day)) {
+	if (!entry.time.set(fz::datetime::utc, year, month, day)) {
 		return false;
 	}
 
@@ -1410,7 +1409,7 @@ bool CDirectoryListingParser::ParseAsDos(CLine &line, CDirentry &entry)
 	entry.name = token.GetString();
 
 	entry.target.clear();
-	entry.ownerGroup = objcache.get(fzstring());
+	entry.ownerGroup = objcache.get(std::wstring());
 	entry.permissions = entry.ownerGroup;
 	entry.time += m_timezoneOffset;
 
@@ -1464,7 +1463,7 @@ bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 				hour = 0;
 	}
 
-	return entry.time.ImbueTime(hour, minute, seconds);
+	return entry.time.imbue_time(hour, minute, seconds);
 }
 
 bool CDirectoryListingParser::ParseAsEplf(CLine &line, CDirentry &entry)
@@ -1485,7 +1484,7 @@ bool CDirectoryListingParser::ParseAsEplf(CLine &line, CDirentry &entry)
 	entry.flags = 0;
 	entry.size = -1;
 
-	fzstring permissions;
+	std::wstring permissions;
 
 	int fact = 1;
 	while (fact < pos) {
@@ -1511,7 +1510,7 @@ bool CDirectoryListingParser::ParseAsEplf(CLine &line, CDirentry &entry)
 			int64_t number = token.GetNumber(fact + 1, len - 1);
 			if (number < 0)
 				return false;
-			entry.time = CDateTime(static_cast<time_t>(number), CDateTime::seconds);
+			entry.time = fz::datetime(static_cast<time_t>(number), fz::datetime::seconds);
 		}
 		else if (type == 'u' && len > 2 && token[fact + 1] == 'p')
 			permissions = token.GetString().substr(fact + 2, len - 2);
@@ -1520,16 +1519,16 @@ bool CDirectoryListingParser::ParseAsEplf(CLine &line, CDirentry &entry)
 	}
 
 	entry.permissions = objcache.get(permissions);
-	entry.ownerGroup = objcache.get(fzstring());
+	entry.ownerGroup = objcache.get(std::wstring());
 	return true;
 }
 
 namespace {
-wxString Unescape(const wxString& str, wxChar escape)
+std::wstring Unescape(const std::wstring& str, wchar_t escape)
 {
-	wxString res;
+	std::wstring res;
 	for (unsigned int i = 0; i < str.size(); ++i) {
-		wxChar c = str[i];
+		wchar_t c = str[i];
 		if (c == escape) {
 			++i;
 			if (i == str.size() || !str[i]) {
@@ -1574,8 +1573,8 @@ bool CDirectoryListingParser::ParseAsVms(CLine &line, CDirentry &entry)
 	if (!line.GetToken(++index, token))
 		return false;
 
-	fzstring ownerGroup;
-	fzstring permissions;
+	std::wstring ownerGroup;
+	std::wstring permissions;
 
 	// This field can either be the filesize, a username (at least that's what I think) enclosed in [] or a date.
 	if (!token.IsNumeric() && !token.IsLeftNumeric()) {
@@ -1737,7 +1736,7 @@ bool CDirectoryListingParser::ParseAsIbm(CLine &line, CDirentry &entry)
 	}
 
 	entry.ownerGroup = objcache.get(ownerGroupToken.GetString());
-	entry.permissions = objcache.get(fzstring());
+	entry.permissions = objcache.get(std::wstring());
 
 	entry.time += m_timezoneOffset;
 
@@ -1769,7 +1768,7 @@ bool CDirectoryListingParser::ParseOther(CLine &line, CDirentry &entry)
 		if (firstToken.GetLength() >= 2 && firstToken[1] == '4')
 			entry.flags |= CDirentry::flag_dir;
 
-		fzstring ownerGroup = token.GetString();
+		std::wstring ownerGroup = token.GetString();
 
 		if (!line.GetToken(++index, token))
 			return false;
@@ -1792,7 +1791,7 @@ bool CDirectoryListingParser::ParseOther(CLine &line, CDirentry &entry)
 		int64_t number = token.GetNumber();
 		if (number < 0)
 			return false;
-		entry.time = CDateTime(static_cast<time_t>(number), CDateTime::seconds);
+		entry.time = fz::datetime(static_cast<time_t>(number), fz::datetime::seconds);
 
 		// Get filename
 		if (!line.GetToken(++index, token, true))
@@ -1813,7 +1812,7 @@ bool CDirectoryListingParser::ParseOther(CLine &line, CDirentry &entry)
 		entry.size = firstToken.GetNumber();
 
 		// Get date
-		wxString dateMonth = token.GetString();
+		std::wstring dateMonth = token.GetString();
 		int month = 0;
 		if (!GetMonthFromName(dateMonth, month)) {
 			// OS/2 or nortel.VxWorks
@@ -1846,8 +1845,7 @@ bool CDirectoryListingParser::ParseOther(CLine &line, CDirentry &entry)
 
 			entry.name = token.GetString();
 			if (entry.name.size() >= 5) {
-				wxString type = entry.name.substr(entry.name.size() - 5);
-				MakeLowerAscii(type);
+				std::wstring type = fz::str_tolower_ascii(entry.name.substr(entry.name.size() - 5));
 				if (!skippedCount && type == _T("<dir>")) {
 					entry.flags |= CDirentry::flag_dir;
 					entry.name = entry.name.substr(0, entry.name.size() - 5);
@@ -1881,7 +1879,7 @@ bool CDirectoryListingParser::ParseOther(CLine &line, CDirentry &entry)
 			else if (year < 1000)
 				year += 1900;
 
-			if( !entry.time.Set(CDateTime::utc, year, month, day) ) {
+			if (!entry.time.set(fz::datetime::utc, year, month, day)) {
 				return false;
 			}
 
@@ -1904,7 +1902,7 @@ bool CDirectoryListingParser::ParseOther(CLine &line, CDirentry &entry)
 			}
 		}
 		entry.target.clear();
-		entry.ownerGroup = objcache.get(fzstring());
+		entry.ownerGroup = objcache.get(std::wstring());
 		entry.permissions = entry.ownerGroup;
 		entry.time += m_timezoneOffset;
 	}
@@ -1925,7 +1923,7 @@ bool CDirectoryListingParser::AddData(char *pData, int len)
 	return ParseData(true);
 }
 
-bool CDirectoryListingParser::AddLine(wxChar const* pLine)
+bool CDirectoryListingParser::AddLine(wchar_t const* pLine)
 {
 	if (m_pControlSocket)
 		m_pControlSocket->LogMessageRaw(MessageType::RawList, pLine);
@@ -1936,11 +1934,11 @@ bool CDirectoryListingParser::AddLine(wxChar const* pLine)
 	if (!*pLine)
 		return false;
 
-	const int len = wxStrlen(pLine);
+	const int len = fz::strlen(pLine);
 
-	wxChar* p = new wxChar[len + 1];
+	wchar_t* p = new wchar_t[len + 1];
 
-	wxStrcpy(p, pLine);
+	wcscpy(p, pLine);
 
 	CLine line(p, len);
 
@@ -1990,7 +1988,9 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/, bool &error)
 				++iter;
 				if (iter == m_DataList.end()) {
 					if (reslen > 10000) {
-						m_pControlSocket->LogMessage(MessageType::Error, _("Received a line exceeding 10000 characters, aborting."));
+						if (m_pControlSocket) {
+							m_pControlSocket->LogMessage(MessageType::Error, _("Received a line exceeding 10000 characters, aborting."));
+						}
 						error = true;
 						return 0;
 					}
@@ -2004,7 +2004,9 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/, bool &error)
 		}
 
 		if (reslen > 10000) {
-			m_pControlSocket->LogMessage(MessageType::Error, _("Received a line exceeding 10000 characters, aborting."));
+			if (m_pControlSocket) {
+				m_pControlSocket->LogMessage(MessageType::Error, _("Received a line exceeding 10000 characters, aborting."));
+			}
 			error = true;
 			return 0;
 		}
@@ -2049,22 +2051,21 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/, bool &error)
 			m_DataList.erase(m_DataList.begin(), iter);
 
 		size_t lineLength{};
-		wxChar* buffer;
+		wchar_t* buffer;
 		if (m_pControlSocket) {
 			buffer = m_pControlSocket->ConvToLocalBuffer(res, buflen, lineLength);
 			m_pControlSocket->LogMessageRaw(MessageType::RawList, buffer);
 		}
 		else {
 			wxString str(res, wxConvUTF8);
-			if (str.empty())
-			{
+			if (str.empty()) {
 				str = wxString(res, wxConvLocal);
 				if (str.empty())
 					str = wxString(res, wxConvISO8859_1);
 			}
 			lineLength = str.Len() + 1;
-			buffer = new wxChar[str.Len() + 1];
-			wxStrcpy(buffer, str.c_str());
+			buffer = new wchar_t[str.Len() + 1];
+			wcscpy(buffer, str.c_str());
 		}
 		delete [] res;
 
@@ -2122,7 +2123,7 @@ bool CDirectoryListingParser::ParseAsWfFtp(CLine &line, CDirentry &entry)
 	if (!ParseTime(token, entry))
 		return false;
 
-	entry.ownerGroup = objcache.get(fzstring());
+	entry.ownerGroup = objcache.get(std::wstring());
 	entry.permissions = entry.ownerGroup;
 	entry.time += m_timezoneOffset;
 
@@ -2157,11 +2158,11 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine &line, CDirentry &entry)
 			return false;
 
 		entry.name = token.GetString();
-		if (entry.name.find(' ') != fzstring::npos)
+		if (entry.name.find(' ') != std::wstring::npos)
 			return false;
 
 		entry.size = -1;
-		entry.ownerGroup = objcache.get(fzstring());
+		entry.ownerGroup = objcache.get(std::wstring());
 		entry.permissions = entry.ownerGroup;
 
 		return true;
@@ -2220,7 +2221,7 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine &line, CDirentry &entry)
 
 	entry.name = token.GetString();
 
-	entry.ownerGroup = objcache.get(fzstring());
+	entry.ownerGroup = objcache.get(std::wstring());
 	entry.permissions = entry.ownerGroup;
 
 	return true;
@@ -2283,7 +2284,7 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_PDS(CLine &line, CDirentry &entry)
 	if (!line.GetToken(index++, token, true))
 		return false;
 
-	entry.ownerGroup = objcache.get(fzstring());
+	entry.ownerGroup = objcache.get(std::wstring());
 	entry.permissions = entry.ownerGroup;
 	entry.time += m_timezoneOffset;
 
@@ -2300,8 +2301,7 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Migrated(CLine &line, CDirentry &en
 	if (!line.GetToken(index, token))
 		return false;
 
-	wxString s = token.GetString();
-	MakeLowerAscii(s);
+	std::wstring s = fz::str_tolower_ascii(token.GetString());
 	if (s != _T("migrated"))
 		return false;
 
@@ -2315,7 +2315,7 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Migrated(CLine &line, CDirentry &en
 
 	entry.flags = 0;
 	entry.size = -1;
-	entry.ownerGroup = objcache.get(fzstring());
+	entry.ownerGroup = objcache.get(std::wstring());
 	entry.permissions = entry.ownerGroup;
 
 	return true;
@@ -2331,7 +2331,7 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_PDS2(CLine &line, CDirentry &entry)
 	entry.name = token.GetString();
 
 	entry.flags = 0;
-	entry.ownerGroup = objcache.get(fzstring());
+	entry.ownerGroup = objcache.get(std::wstring());
 	entry.permissions = entry.ownerGroup;
 	entry.size = -1;
 
@@ -2400,8 +2400,7 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Tape(CLine &line, CDirentry &entry)
 	if (!line.GetToken(index++, token))
 		return false;
 
-	wxString s = token.GetString();
-	MakeLowerAscii(s);
+	std::wstring s = fz::str_tolower_ascii(token.GetString());
 	if (s != _T("tape"))
 		return false;
 
@@ -2411,8 +2410,8 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Tape(CLine &line, CDirentry &entry)
 
 	entry.name = token.GetString();
 	entry.flags = 0;
-	entry.ownerGroup = objcache.get(fzstring());
-	entry.permissions = objcache.get(fzstring());
+	entry.ownerGroup = objcache.get(std::wstring());
+	entry.permissions = objcache.get(std::wstring());
 	entry.size = -1;
 
 	if (line.GetToken(index++, token))
@@ -2522,50 +2521,49 @@ int CDirectoryListingParser::ParseAsMlsd(CLine &line, CDirentry &entry)
 	if (!line.GetToken(0, token))
 		return 0;
 
-	fzstring facts = token.GetString();
+	std::wstring const facts = token.GetString();
 	if (facts.empty())
 		return 0;
 
 	entry.flags = 0;
 	entry.size = -1;
 
-	fzstring owner, group, uid, gid;
-	fzstring ownerGroup;
-	fzstring permissions;
+	std::wstring owner, group, uid, gid;
+	std::wstring ownerGroup;
+	std::wstring permissions;
 
-	while (!facts.empty()) {
-		auto delim = facts.find(';');
-		if (delim == fzstring::npos) {
+	size_t start = 0;
+	while (start < facts.size()) {
+		auto delim = facts.find(';', start);
+		if (delim == std::wstring::npos) {
 			delim = facts.size();
 		}
-		else if (delim < 3) {
+		else if (delim < start + 3) {
 			return 0;
 		}
 
-		auto const pos = facts.find('=');
-		if (pos == fzstring::npos || pos < 1 || pos > delim)
+		auto const pos = facts.find('=', start);
+		if (pos == std::wstring::npos || pos < start + 1 || pos > delim)
 			return 0;
 
-		wxString factname = facts.substr(0, pos);
-		MakeLowerAscii(factname);
-		fzstring value = facts.substr(pos + 1, delim - pos - 1);
+		std::wstring factname = fz::str_tolower_ascii(facts.substr(start, pos - start));
+		std::wstring value = facts.substr(pos + 1, delim - pos - 1);
 		if (factname == _T("type")) {
 			auto colonPos = value.find(':');
-			wxString valuePrefix;
-			if (colonPos == fzstring::npos)
-				valuePrefix = value;
+			std::wstring valuePrefix;
+			if (colonPos == std::wstring::npos)
+				valuePrefix = fz::str_tolower_ascii(value);
 			else
-				valuePrefix = value.substr(0, colonPos);
-			MakeLowerAscii(valuePrefix);
+				valuePrefix = fz::str_tolower_ascii(value.substr(0, colonPos));
 
-			if (valuePrefix == _T("dir") && colonPos == fzstring::npos)
+			if (valuePrefix == _T("dir") && colonPos == std::wstring::npos)
 				entry.flags |= CDirentry::flag_dir;
 			else if (valuePrefix == _T("os.unix=slink") || valuePrefix == _T("os.unix=symlink")) {
 				entry.flags |= CDirentry::flag_dir | CDirentry::flag_link;
-				if (colonPos != fzstring::npos)
-					entry.target = CSparseOptional<fzstring>(value.substr(colonPos));
+				if (colonPos != std::wstring::npos)
+					entry.target = fz::sparse_optional<std::wstring>(value.substr(colonPos));
 			}
-			else if ((valuePrefix == _T("cdir") || valuePrefix == _T("pdir")) && colonPos == fzstring::npos) {
+			else if ((valuePrefix == _T("cdir") || valuePrefix == _T("pdir")) && colonPos == std::wstring::npos) {
 				// Current and parent directory, don't parse it
 				return 2;
 			}
@@ -2583,8 +2581,8 @@ int CDirectoryListingParser::ParseAsMlsd(CLine &line, CDirentry &entry)
 		else if (factname == _T("modify") ||
 			(!entry.has_date() && factname == _T("create")))
 		{
-			entry.time = CDateTime(value, CDateTime::utc);
-			if (!entry.time.IsValid()) {
+			entry.time = fz::datetime(value, fz::datetime::utc);
+			if (!entry.time.empty()) {
 				return 0;
 			}
 		}
@@ -2611,12 +2609,7 @@ int CDirectoryListingParser::ParseAsMlsd(CLine &line, CDirentry &entry)
 		else if (factname == _T("unix.gid"))
 			gid = value;
 
-		if (delim >= facts.size()) {
-			facts.clear();
-		}
-		else {
-			facts = facts.substr(delim + 1);
-		}
+		start = delim + 1;
 	}
 
 	// The order of the facts is undefined, so assemble ownerGroup in correct
@@ -2741,7 +2734,7 @@ bool CDirectoryListingParser::ParseAsZVM(CLine &line, CDirentry &entry)
 	// File format. Unused
 	if (!line.GetToken(++index, token))
 		return false;
-	wxString format = token.GetString();
+	std::wstring format = token.GetString();
 	if (format != _T("V") && format != _T("F"))
 		return false;
 
@@ -2796,7 +2789,7 @@ bool CDirectoryListingParser::ParseAsZVM(CLine &line, CDirentry &entry)
 		return false;
 
 	entry.ownerGroup = objcache.get(ownerGroupToken.GetString());
-	entry.permissions = objcache.get(fzstring());
+	entry.permissions = objcache.get(std::wstring());
 	entry.target.clear();
 	entry.time += m_timezoneOffset;
 
@@ -2845,7 +2838,7 @@ bool CDirectoryListingParser::ParseAsHPNonstop(CLine &line, CDirentry &entry)
 	// Owner
 	if (!line.GetToken(++index, token))
 		return false;
-	fzstring ownerGroup = token.GetString();
+	std::wstring ownerGroup = token.GetString();
 
 	if (token[token.GetLength() - 1] == ',') {
 		// Owner, part 2
@@ -2869,17 +2862,12 @@ bool CDirectoryListingParser::ParseAsHPNonstop(CLine &line, CDirentry &entry)
 	return true;
 }
 
-bool CDirectoryListingParser::GetMonthFromName(const wxString& name, int &month)
+bool CDirectoryListingParser::GetMonthFromName(const std::wstring& name, int &month)
 {
-	auto iter = m_MonthNamesMap.find(name.Lower());
+	std::wstring lower = fz::str_tolower_ascii(name);
+	auto iter = m_MonthNamesMap.find(lower);
 	if (iter == m_MonthNamesMap.end())
-	{
-		wxString lower(name);
-		MakeLowerAscii(lower);
-		iter = m_MonthNamesMap.find(lower);
-		if (iter == m_MonthNamesMap.end())
-			return false;
-	}
+		return false;
 
 	month = iter->second;
 
@@ -2931,42 +2919,43 @@ void CDirectoryListingParser::DeduceEncoding()
 
 	int count_normal = 0;
 	int count_ebcdic = 0;
-	for (int i = '0'; i <= '9'; ++i ) {
+	for (int i = '0'; i <= '9'; ++i) {
 		count_normal += count[i];
 	}
-	for (int i = 'a'; i <= 'z'; ++i ) {
+	for (int i = 'a'; i <= 'z'; ++i) {
 		count_normal += count[i];
 	}
-	for (int i = 'A'; i <= 'Z'; ++i ) {
+	for (int i = 'A'; i <= 'Z'; ++i) {
 		count_normal += count[i];
 	}
 
-	for (int i = 0x81; i <= 0x89; ++i ) {
+	for (int i = 0x81; i <= 0x89; ++i) {
 		count_ebcdic += count[i];
 	}
-	for (int i = 0x91; i <= 0x99; ++i ) {
+	for (int i = 0x91; i <= 0x99; ++i) {
 		count_ebcdic += count[i];
 	}
-	for (int i = 0xa2; i <= 0xa9; ++i ) {
+	for (int i = 0xa2; i <= 0xa9; ++i) {
 		count_ebcdic += count[i];
 	}
-	for (int i = 0xc1; i <= 0xc9; ++i ) {
+	for (int i = 0xc1; i <= 0xc9; ++i) {
 		count_ebcdic += count[i];
 	}
-	for (int i = 0xd1; i <= 0xd9; ++i ) {
+	for (int i = 0xd1; i <= 0xd9; ++i) {
 		count_ebcdic += count[i];
 	}
-	for (int i = 0xe2; i <= 0xe9; ++i ) {
+	for (int i = 0xe2; i <= 0xe9; ++i) {
 		count_ebcdic += count[i];
 	}
-	for (int i = 0xf0; i <= 0xf9; ++i ) {
+	for (int i = 0xf0; i <= 0xf9; ++i) {
 		count_ebcdic += count[i];
 	}
 
 
-	if ((count[0x1f] || count[0x15] || count[0x25]) && !count[0x0a] && count[static_cast<unsigned char>('@')] && count[static_cast<unsigned char>('@')] > count[static_cast<unsigned char>(' ')] && count_ebcdic > count_normal)
-	{
-		m_pControlSocket->LogMessage(MessageType::Status, _("Received a directory listing which appears to be encoded in EBCDIC."));
+	if ((count[0x1f] || count[0x15] || count[0x25]) && !count[0x0a] && count[static_cast<unsigned char>('@')] && count[static_cast<unsigned char>('@')] > count[static_cast<unsigned char>(' ')] && count_ebcdic > count_normal) {
+		if (m_pControlSocket) {
+			m_pControlSocket->LogMessage(MessageType::Status, _("Received a directory listing which appears to be encoded in EBCDIC."));
+		}
 		m_listingEncoding = listingEncoding::ebcdic;
 		for (auto it = m_DataList.begin(); it != m_DataList.end(); ++it)
 			ConvertEncoding(it->p, it->len);
