@@ -2,7 +2,6 @@
 #include "ControlSocket.h"
 #include "directorycache.h"
 #include "engineprivate.h"
-#include "event_loop.h"
 #include "ftpcontrolsocket.h"
 #include "httpcontrolsocket.h"
 #include "logging_private.h"
@@ -10,15 +9,17 @@
 #include "ratelimiter.h"
 #include "sftpcontrolsocket.h"
 
+#include <libfilezilla/event_loop.hpp>
+
 #include <algorithm>
 
-mutex CFileZillaEnginePrivate::mutex_;
+fz::mutex CFileZillaEnginePrivate::mutex_;
 std::vector<CFileZillaEnginePrivate*> CFileZillaEnginePrivate::m_engineList;
 std::atomic_int CFileZillaEnginePrivate::m_activeStatus[2] = {{0}, {0}};
 std::list<CFileZillaEnginePrivate::t_failedLogins> CFileZillaEnginePrivate::m_failedLogins;
 
 CFileZillaEnginePrivate::CFileZillaEnginePrivate(CFileZillaEngineContext& context, CFileZillaEngine& parent)
-	: CEventHandler(context.GetEventLoop())
+	: event_handler(context.GetEventLoop())
 	, transfer_status_(*this)
 	, m_options(context.GetOptions())
 	, m_rateLimiter(context.GetRateLimiter())
@@ -29,7 +30,7 @@ CFileZillaEnginePrivate::CFileZillaEnginePrivate(CFileZillaEngineContext& contex
 	m_engineList.push_back(this);
 
 	{
-		scoped_lock lock(mutex_);
+		fz::scoped_lock lock(mutex_);
 		static int id = 0;
 		m_engine_id = ++id;
 	}
@@ -38,7 +39,7 @@ CFileZillaEnginePrivate::CFileZillaEnginePrivate(CFileZillaEngineContext& contex
 
 	{
 		bool queue_logs = ShouldQueueLogsFromOptions();
-		scoped_lock lock(notification_mutex_);
+		fz::scoped_lock lock(notification_mutex_);
 		queue_logs_ = queue_logs;
 	}
 
@@ -57,7 +58,7 @@ bool CFileZillaEnginePrivate::ShouldQueueLogsFromOptions() const
 
 CFileZillaEnginePrivate::~CFileZillaEnginePrivate()
 {
-	RemoveHandler();
+	remove_handler();
 	m_maySendNotificationEvent = false;
 
 	m_pControlSocket.reset();
@@ -100,13 +101,13 @@ void CFileZillaEnginePrivate::OnEngineEvent(EngineNotificationType type)
 
 bool CFileZillaEnginePrivate::IsBusy() const
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	return m_pCurrentCommand != 0;
 }
 
 bool CFileZillaEnginePrivate::IsConnected() const
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	if (!m_pControlSocket)
 		return false;
 
@@ -115,13 +116,13 @@ bool CFileZillaEnginePrivate::IsConnected() const
 
 const CCommand *CFileZillaEnginePrivate::GetCurrentCommand() const
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	return m_pCurrentCommand.get();
 }
 
 Command CFileZillaEnginePrivate::GetCurrentCommandId() const
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	if (!m_pCurrentCommand)
 		return Command::none;
 	else
@@ -131,7 +132,7 @@ Command CFileZillaEnginePrivate::GetCurrentCommandId() const
 void CFileZillaEnginePrivate::AddNotification(CNotification *pNotification)
 {
 	{
-		scoped_lock lock(notification_mutex_);
+		fz::scoped_lock lock(notification_mutex_);
 		m_NotificationList.push_back(pNotification);
 
 		if (!m_maySendNotificationEvent || !m_pEventHandler) {
@@ -145,7 +146,7 @@ void CFileZillaEnginePrivate::AddNotification(CNotification *pNotification)
 
 void CFileZillaEnginePrivate::AddLogNotification(CLogmsgNotification *pNotification)
 {
-	scoped_lock lock(notification_mutex_);
+	fz::scoped_lock lock(notification_mutex_);
 
 	if (pNotification->msgType == MessageType::Error) {
 		queue_logs_ = false;
@@ -167,7 +168,7 @@ void CFileZillaEnginePrivate::AddLogNotification(CLogmsgNotification *pNotificat
 void CFileZillaEnginePrivate::SendQueuedLogs(bool reset_flag)
 {
 	{
-		scoped_lock lock(notification_mutex_);
+		fz::scoped_lock lock(notification_mutex_);
 		m_NotificationList.insert(m_NotificationList.end(), queued_logs_.begin(), queued_logs_.end());
 		queued_logs_.clear();
 
@@ -186,7 +187,7 @@ void CFileZillaEnginePrivate::SendQueuedLogs(bool reset_flag)
 
 void CFileZillaEnginePrivate::ClearQueuedLogs(bool reset_flag)
 {
-	scoped_lock lock(notification_mutex_);
+	fz::scoped_lock lock(notification_mutex_);
 
 	for (auto msg : queued_logs_) {
 		delete msg;
@@ -200,7 +201,7 @@ void CFileZillaEnginePrivate::ClearQueuedLogs(bool reset_flag)
 
 int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	m_pLogging->LogMessage(MessageType::Debug_Debug, _T("CFileZillaEnginePrivate::ResetOperation(%d)"), nErrorCode);
 
 	if (nErrorCode & FZ_REPLY_DISCONNECTED)
@@ -208,7 +209,6 @@ int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 
 	if (m_pCurrentCommand) {
 		if ((nErrorCode & FZ_REPLY_NOTSUPPORTED) == FZ_REPLY_NOTSUPPORTED) {
-			wxASSERT(m_bIsInCommand);
 			m_pLogging->LogMessage(MessageType::Error, _("Command not supported by this protocol"));
 		}
 
@@ -227,8 +227,8 @@ int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 						if (!delay)
 							delay = 1;
 						m_pLogging->LogMessage(MessageType::Status, _("Waiting to retry..."));
-						StopTimer(m_retryTimer);
-						m_retryTimer = AddTimer(duration::from_milliseconds(delay), true);
+						stop_timer(m_retryTimer);
+						m_retryTimer = add_timer(fz::duration::from_milliseconds(delay), true);
 						return FZ_REPLY_WOULDBLOCK;
 					}
 				}
@@ -267,7 +267,7 @@ int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 
 unsigned int CFileZillaEnginePrivate::GetNextAsyncRequestNumber()
 {
-	scoped_lock lock(notification_mutex_);
+	fz::scoped_lock lock(notification_mutex_);
 	return ++m_asyncRequestCounter;
 }
 
@@ -327,7 +327,7 @@ int CFileZillaEnginePrivate::List(const CListCommand &command)
 					else {
 						if (!avoid) {
 							m_lastListDir = pListing->path;
-							m_lastListTime = CMonotonicClock::now();
+							m_lastListTime = fz::monotonic_clock::now();
 							CDirectoryListingNotification *pNotification = new CDirectoryListingNotification(pListing->path);
 							AddNotification(pNotification);
 						}
@@ -353,7 +353,7 @@ int CFileZillaEnginePrivate::FileTransfer(const CFileTransferCommand &command)
 int CFileZillaEnginePrivate::RawCommand(const CRawCommand& command)
 {
 	{
-		scoped_lock lock(notification_mutex_);
+		fz::scoped_lock lock(notification_mutex_);
 		queue_logs_ = false;
 	}
 	return m_pControlSocket->RawCommand(command.GetCommand());
@@ -392,7 +392,7 @@ int CFileZillaEnginePrivate::Chmod(const CChmodCommand& command)
 
 void CFileZillaEnginePrivate::SendDirectoryListingNotification(const CServerPath& path, bool onList, bool modified, bool failed)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	wxASSERT(m_pControlSocket);
 
@@ -403,13 +403,13 @@ void CFileZillaEnginePrivate::SendDirectoryListingNotification(const CServerPath
 
 	if (failed) {
 		AddNotification(new CDirectoryListingNotification(path, false, true));
-		m_lastListTime = CMonotonicClock::now();
+		m_lastListTime = fz::monotonic_clock::now();
 
 		// On failed messages, we don't notify other engines
 		return;
 	}
 
-	CMonotonicClock changeTime;
+	fz::monotonic_clock changeTime;
 	if (!directory_cache_.GetChangeTime(changeTime, *pOwnServer, path))
 		return;
 
@@ -443,10 +443,10 @@ void CFileZillaEnginePrivate::SendDirectoryListingNotification(const CServerPath
 
 void CFileZillaEnginePrivate::RegisterFailedLoginAttempt(const CServer& server, bool critical)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	std::list<t_failedLogins>::iterator iter = m_failedLogins.begin();
 	while (iter != m_failedLogins.end()) {
-		duration const span = CDateTime::Now() - iter->time;
+		fz::duration const span = fz::datetime::now() - iter->time;
 		if (span.get_seconds() >= m_options.GetOptionVal(OPTION_RECONNECTDELAY) ||
 			iter->server == server || (!critical && (iter->server.GetHost() == server.GetHost() && iter->server.GetPort() == server.GetPort())))
 		{
@@ -460,17 +460,17 @@ void CFileZillaEnginePrivate::RegisterFailedLoginAttempt(const CServer& server,
 
 	t_failedLogins failure;
 	failure.server = server;
-	failure.time = CDateTime::Now();
+	failure.time = fz::datetime::now();
 	failure.critical = critical;
 	m_failedLogins.push_back(failure);
 }
 
 unsigned int CFileZillaEnginePrivate::GetRemainingReconnectDelay(const CServer& server)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	std::list<t_failedLogins>::iterator iter = m_failedLogins.begin();
 	while (iter != m_failedLogins.end()) {
-		duration const span = CDateTime::Now() - iter->time;
+		fz::duration const span = fz::datetime::now() - iter->time;
 		const int delay = m_options.GetOptionVal(OPTION_RECONNECTDELAY);
 		if (span.get_seconds() >= delay) {
 			std::list<t_failedLogins>::iterator prev = iter;
@@ -508,7 +508,7 @@ void CFileZillaEnginePrivate::OnTimer(int)
 
 int CFileZillaEnginePrivate::ContinueConnect()
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	if (!m_pCurrentCommand || m_pCurrentCommand->GetId() != Command::connect) {
 		m_pLogging->LogMessage(MessageType::Debug_Warning, _T("CFileZillaEnginePrivate::ContinueConnect called without pending Command::connect"));
@@ -520,8 +520,8 @@ int CFileZillaEnginePrivate::ContinueConnect()
 	unsigned int delay = GetRemainingReconnectDelay(server);
 	if (delay) {
 		m_pLogging->LogMessage(MessageType::Status, wxPLURAL("Delaying connection for %d second due to previously failed connection attempt...", "Delaying connection for %d seconds due to previously failed connection attempt...", (delay + 999) / 1000), (delay + 999) / 1000);
-		StopTimer(m_retryTimer);
-		m_retryTimer = AddTimer(duration::from_milliseconds(delay), true);
+		stop_timer(m_retryTimer);
+		m_retryTimer = add_timer(fz::duration::from_milliseconds(delay), true);
 		return FZ_REPLY_WOULDBLOCK;
 	}
 
@@ -554,14 +554,14 @@ int CFileZillaEnginePrivate::ContinueConnect()
 
 void CFileZillaEnginePrivate::InvalidateCurrentWorkingDirs(const CServerPath& path)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	wxASSERT(m_pControlSocket);
 	const CServer* const pOwnServer = m_pControlSocket->GetCurrentServer();
 	wxASSERT(pOwnServer);
 
 	for (auto & engine : m_engineList) {
-		if (engine == this)
+		if (!engine || engine == this)
 			continue;
 
 		if (!engine->m_pControlSocket)
@@ -575,11 +575,11 @@ void CFileZillaEnginePrivate::InvalidateCurrentWorkingDirs(const CServerPath& pa
 	}
 }
 
-void CFileZillaEnginePrivate::operator()(CEventBase const& ev)
+void CFileZillaEnginePrivate::operator()(fz::event_base const& ev)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
-	Dispatch<CFileZillaEngineEvent, CCommandEvent, CAsyncRequestReplyEvent, CTimerEvent>(ev, this,
+	fz::dispatch<CFileZillaEngineEvent, CCommandEvent, CAsyncRequestReplyEvent, fz::timer_event>(ev, this,
 		&CFileZillaEnginePrivate::OnEngineEvent,
 		&CFileZillaEnginePrivate::OnCommandEvent,
 		&CFileZillaEnginePrivate::OnSetAsyncRequestReplyEvent,
@@ -606,7 +606,7 @@ int CFileZillaEnginePrivate::CheckCommandPreconditions(CCommand const& command,
 
 void CFileZillaEnginePrivate::OnCommandEvent()
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	if (m_pCurrentCommand) {
 		CCommand & command = *m_pCurrentCommand;
@@ -664,7 +664,7 @@ void CFileZillaEnginePrivate::OnCommandEvent()
 
 void CFileZillaEnginePrivate::DoCancel()
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	if (!IsBusy())
 		return;
 
@@ -675,7 +675,7 @@ void CFileZillaEnginePrivate::DoCancel()
 
 		m_pCurrentCommand.reset();
 
-		StopTimer(m_retryTimer);
+		stop_timer(m_retryTimer);
 		m_retryTimer = 0;
 
 		m_pLogging->LogMessage(MessageType::Error, _("Connection attempt interrupted by user"));
@@ -703,7 +703,7 @@ bool CFileZillaEnginePrivate::CheckAsyncRequestReplyPreconditions(std::unique_pt
 
 	bool match;
 	{
-		scoped_lock l(notification_mutex_);
+		fz::scoped_lock l(notification_mutex_);
 		match = reply->requestNumber == m_asyncRequestCounter;
 	}
 
@@ -712,7 +712,7 @@ bool CFileZillaEnginePrivate::CheckAsyncRequestReplyPreconditions(std::unique_pt
 
 void CFileZillaEnginePrivate::OnSetAsyncRequestReplyEvent(std::unique_ptr<CAsyncRequestNotification> const& reply)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	if (!CheckAsyncRequestReplyPreconditions(reply)) {
 		return;
 	}
@@ -723,14 +723,14 @@ void CFileZillaEnginePrivate::OnSetAsyncRequestReplyEvent(std::unique_ptr<CAsync
 
 int CFileZillaEnginePrivate::Init(wxEvtHandler *pEventHandler)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	m_pEventHandler = pEventHandler;
 	return FZ_REPLY_OK;
 }
 
 int CFileZillaEnginePrivate::Execute(const CCommand &command)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	int res = CheckCommandPreconditions(command, true);
 	if (res != FZ_REPLY_OK) {
@@ -738,14 +738,14 @@ int CFileZillaEnginePrivate::Execute(const CCommand &command)
 	}
 
 	m_pCurrentCommand.reset(command.Clone());
-	SendEvent<CCommandEvent>();
+	send_event<CCommandEvent>();
 
 	return FZ_REPLY_WOULDBLOCK;
 }
 
 std::unique_ptr<CNotification> CFileZillaEnginePrivate::GetNextNotification()
 {
-	scoped_lock lock(notification_mutex_);
+	fz::scoped_lock lock(notification_mutex_);
 
 	if (m_NotificationList.empty()) {
 		m_maySendNotificationEvent = true;
@@ -759,12 +759,12 @@ std::unique_ptr<CNotification> CFileZillaEnginePrivate::GetNextNotification()
 
 bool CFileZillaEnginePrivate::SetAsyncRequestReply(std::unique_ptr<CAsyncRequestNotification> && pNotification)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	if (!CheckAsyncRequestReplyPreconditions(pNotification)) {
 		return false;
 	}
 
-	SendEvent<CAsyncRequestReplyEvent>(std::move(pNotification));
+	send_event<CAsyncRequestReplyEvent>(std::move(pNotification));
 
 	return true;
 }
@@ -777,7 +777,7 @@ bool CFileZillaEnginePrivate::IsPendingAsyncRequestReply(std::unique_ptr<CAsyncR
 	if (!IsBusy())
 		return false;
 
-	scoped_lock lock(notification_mutex_);
+	fz::scoped_lock lock(notification_mutex_);
 	return pNotification->requestNumber == m_asyncRequestCounter;
 }
 
@@ -809,7 +809,7 @@ CTransferStatus CFileZillaEnginePrivate::GetTransferStatus(bool &changed)
 int CFileZillaEnginePrivate::CacheLookup(const CServerPath& path, CDirectoryListing& listing)
 {
 	// TODO: Possible optimization: Atomically get current server. The cache has its own mutex.
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	if (!IsConnected())
 		return FZ_REPLY_ERROR;
@@ -825,18 +825,18 @@ int CFileZillaEnginePrivate::CacheLookup(const CServerPath& path, CDirectoryList
 
 int CFileZillaEnginePrivate::Cancel()
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	if (!IsBusy())
 		return FZ_REPLY_OK;
 
-	SendEvent<CFileZillaEngineEvent>(engineCancel);
+	send_event<CFileZillaEngineEvent>(engineCancel);
 	return FZ_REPLY_WOULDBLOCK;
 }
 
 void CFileZillaEnginePrivate::OnOptionsChanged(changed_options_t const&)
 {
 	bool queue_logs = ShouldQueueLogsFromOptions();
-	scoped_lock lock(notification_mutex_);
+	fz::scoped_lock lock(notification_mutex_);
 	queue_logs_ = queue_logs;
 
 	if (!queue_logs_) {
@@ -853,7 +853,7 @@ CTransferStatusManager::CTransferStatusManager(CFileZillaEnginePrivate& engine)
 void CTransferStatusManager::Reset()
 {
 	{
-		scoped_lock lock(mutex_);
+		fz::scoped_lock lock(mutex_);
 		status_.clear();
 		send_state_ = 0;
 	}
@@ -863,7 +863,7 @@ void CTransferStatusManager::Reset()
 
 void CTransferStatusManager::Init(wxFileOffset totalSize, wxFileOffset startOffset, bool list)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	if (startOffset < 0)
 		startOffset = 0;
 
@@ -873,16 +873,16 @@ void CTransferStatusManager::Init(wxFileOffset totalSize, wxFileOffset startOffs
 
 void CTransferStatusManager::SetStartTime()
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	if (!status_)
 		return;
 
-	status_.started = CDateTime::Now();
+	status_.started = fz::datetime::now();
 }
 
 void CTransferStatusManager::SetMadeProgress()
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	if (!status_)
 		return;
 
@@ -896,7 +896,7 @@ void CTransferStatusManager::Update(wxFileOffset transferredBytes)
 	{
 		int64_t oldOffset = currentOffset_.fetch_add(transferredBytes);
 		if (!oldOffset) {
-			scoped_lock lock(mutex_);
+			fz::scoped_lock lock(mutex_);
 			if (!status_) {
 				return;
 			}
@@ -916,7 +916,7 @@ void CTransferStatusManager::Update(wxFileOffset transferredBytes)
 
 CTransferStatus CTransferStatusManager::Get(bool &changed)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	if (!status_) {
 		changed = false;
 		send_state_ = 0;
@@ -937,6 +937,6 @@ CTransferStatus CTransferStatusManager::Get(bool &changed)
 
 bool CTransferStatusManager::empty()
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 	return status_.empty();
 }
