@@ -1,17 +1,24 @@
-#include "FileZilla.h"
+#include <filezilla.h>
+
 #include "ControlSocket.h"
+#include "engineprivate.h"
 #include "httpcontrolsocket.h"
-#include <errno.h>
+#include "tlssocket.h"
+
+#include <libfilezilla/iputils.hpp>
+#include <libfilezilla/local_filesys.hpp>
+
+#include <wx/file.h>
 
 #define FZ_REPLY_REDIRECTED FZ_REPLY_ALREADYCONNECTED
 
 // Connect is special for HTTP: It is done on a per-command basis, so we need
 // to establish a connection before each command.
-class CHttpConnectOpData : public COpData
+class CHttpConnectOpData : public CConnectOpData
 {
 public:
 	CHttpConnectOpData()
-		: COpData(cmd_connect)
+		: tls(false)
 	{
 	}
 
@@ -19,8 +26,7 @@ public:
 	{
 	}
 
-	wxString host;
-	unsigned int port;
+	bool tls;
 };
 
 class CHttpOpData
@@ -29,28 +35,18 @@ public:
 	CHttpOpData(COpData* pOpData)
 		: m_pOpData(pOpData)
 	{
-		m_gotHeader = false;
-		m_responseCode = -1;
-		m_redirectionCount = 0;
-		m_transferEncoding = unknown;
-
-		m_chunkData.getTrailer = false;
-		m_chunkData.size = 0;
-		m_chunkData.terminateChunk = false;
-
-		m_totalSize = -1;
-		m_receivedData = 0;
 	}
 
-	bool m_gotHeader;
-	int m_responseCode;
-	wxString m_responseString;
-	wxString m_newLocation;
-	wxString m_newHostWithPort;
-	int m_redirectionCount;
+	virtual ~CHttpOpData() = default;
 
-	wxLongLong m_totalSize;
-	wxLongLong m_receivedData;
+	bool m_gotHeader{};
+	int m_responseCode{-1};
+	wxString m_responseString;
+	wxURI m_newLocation;
+	int m_redirectionCount{};
+
+	int64_t m_totalSize{-1};
+	int64_t m_receivedData{};
 
 	COpData* m_pOpData;
 
@@ -60,21 +56,21 @@ public:
 		chunked,
 		unknown
 	};
-	enum transferEncodings m_transferEncoding;
+	transferEncodings m_transferEncoding{unknown};
 
 	struct t_chunkData
 	{
-		bool getTrailer;
-		bool terminateChunk;
-		wxLongLong size;
+		bool getTrailer{};
+		bool terminateChunk{};
+		int64_t size{};
 	} m_chunkData;
 };
 
 class CHttpFileTransferOpData : public CFileTransferOpData, public CHttpOpData
 {
 public:
-	CHttpFileTransferOpData(const wxString& local_file, const wxString& remote_file, const CServerPath& remote_path)
-		: CFileTransferOpData(local_file, remote_file, remote_path), CHttpOpData(this)
+	CHttpFileTransferOpData(bool is_download, const wxString& local_file, const wxString& remote_file, const CServerPath& remote_path)
+		: CFileTransferOpData(is_download, local_file, remote_file, remote_path), CHttpOpData(this)
 	{
 		pFile = 0;
 	}
@@ -87,39 +83,44 @@ public:
 	wxFile* pFile;
 };
 
-CHttpControlSocket::CHttpControlSocket(CFileZillaEnginePrivate *pEngine)
-	: CRealControlSocket(pEngine)
+CHttpControlSocket::CHttpControlSocket(CFileZillaEnginePrivate & engine)
+	: CRealControlSocket(engine)
 {
 	m_pRecvBuffer = 0;
+	m_recvBufferPos = 0;
+	m_pTlsSocket = 0;
+	m_pHttpOpData = 0;
 }
 
 CHttpControlSocket::~CHttpControlSocket()
 {
+	remove_handler();
+	DoClose();
 	delete [] m_pRecvBuffer;
 }
 
 int CHttpControlSocket::SendNextCommand()
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::SendNextCommand()"));
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::SendNextCommand()"));
 	if (!m_pCurOpData)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("SendNextCommand called without active operation"));
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("SendNextCommand called without active operation"));
 		ResetOperation(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
 	}
 
 	if (m_pCurOpData->waitForAsyncRequest)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Waiting for async request, ignoring SendNextCommand"));
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Waiting for async request, ignoring SendNextCommand"));
 		return FZ_REPLY_WOULDBLOCK;
 	}
 
 	switch (m_pCurOpData->opId)
 	{
-	case cmd_transfer:
+	case Command::transfer:
 		return FileTransferSend();
 	default:
-		LogMessage(__TFILE__, __LINE__, this, ::Debug_Warning, _T("Unknown opID (%d) in SendNextCommand"), m_pCurOpData->opId);
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("Unknown opID (%d) in SendNextCommand"), m_pCurOpData->opId);
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		break;
 	}
@@ -130,11 +131,11 @@ int CHttpControlSocket::SendNextCommand()
 
 int CHttpControlSocket::ContinueConnect()
 {
-	LogMessage(__TFILE__, __LINE__, this, Debug_Verbose, _T("CHttpControlSocket::ContinueConnect() m_pEngine=%p"), m_pEngine);
-	if (GetCurrentCommandId() != cmd_connect ||
+	LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Verbose, _T("CHttpControlSocket::ContinueConnect() &engine_=%p"), &engine_);
+	if (GetCurrentCommandId() != Command::connect ||
 		!m_pCurrentServer)
 	{
-		LogMessage(Debug_Warning, _T("Invalid context for call to ContinueConnect(), cmd=%d, m_pCurrentServer=%p"), GetCurrentCommandId(), m_pCurrentServer);
+		LogMessage(MessageType::Debug_Warning, _T("Invalid context for call to ContinueConnect(), cmd=%d, m_pCurrentServer=%p"), GetCurrentCommandId(), m_pCurrentServer);
 		return DoClose(FZ_REPLY_INTERNALERROR);
 	}
 
@@ -148,7 +149,7 @@ bool CHttpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 	{
 		if (!m_pCurOpData->waitForAsyncRequest)
 		{
-			LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Not waiting for request reply, ignoring request reply %d"), pNotification->GetRequestID());
+			LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Not waiting for request reply, ignoring request reply %d"), pNotification->GetRequestID());
 			return false;
 		}
 		m_pCurOpData->waitForAsyncRequest = false;
@@ -158,25 +159,30 @@ bool CHttpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 	{
 	case reqId_fileexists:
 		{
-			if (!m_pCurOpData || m_pCurOpData->opId != cmd_transfer)
+			if (!m_pCurOpData || m_pCurOpData->opId != Command::transfer)
 			{
-				LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("No or invalid operation in progress, ignoring request reply %f"), pNotification->GetRequestID());
+				LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("No or invalid operation in progress, ignoring request reply %f"), pNotification->GetRequestID());
 				return false;
 			}
 
-			CFileExistsNotification *pFileExistsNotification = reinterpret_cast<CFileExistsNotification *>(pNotification);
-			switch (pFileExistsNotification->overwriteAction)
+			CFileExistsNotification *pFileExistsNotification = static_cast<CFileExistsNotification *>(pNotification);
+			return SetFileExistsAction(pFileExistsNotification);
+		}
+		break;
+	case reqId_certificate:
+		{
+			if (!m_pTlsSocket || m_pTlsSocket->GetState() != CTlsSocket::TlsState::verifycert)
 			{
-			case CFileExistsNotification::resume:
-				ResetOperation(FZ_REPLY_CRITICALERROR | FZ_REPLY_NOTSUPPORTED);
-				break;
-			default:
-				return SetFileExistsAction(pFileExistsNotification);
+				LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("No or invalid operation in progress, ignoring request reply %d"), pNotification->GetRequestID());
+				return false;
 			}
+
+			CCertificateNotification* pCertificateNotification = static_cast<CCertificateNotification *>(pNotification);
+			m_pTlsSocket->TrustCurrentCert(pCertificateNotification->m_trusted);
 		}
 		break;
 	default:
-		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("Unknown request %d"), pNotification->GetRequestID());
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("Unknown request %d"), pNotification->GetRequestID());
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return false;
 	}
@@ -205,7 +211,7 @@ int CHttpControlSocket::DoReceive()
 
 		unsigned int len = m_recvBufferLen - m_recvBufferPos;
 		int error;
-		int read = m_pSocket->Read(m_pRecvBuffer + m_recvBufferPos, len, error);
+		int read = m_pBackend->Read(m_pRecvBuffer + m_recvBufferPos, len, error);
 		if (read == -1)
 		{
 			if (error != EAGAIN)
@@ -215,10 +221,9 @@ int CHttpControlSocket::DoReceive()
 			return 0;
 		}
 
-		m_pEngine->SetActive(CFileZillaEngine::recv);
+		SetActive(CFileZillaEngine::recv);
 
-		if (!m_pCurOpData || m_pCurOpData->opId == cmd_connect)
-		{
+		if (!m_pCurOpData || m_pCurOpData->opId == Command::connect) {
 			// Just ignore all further data
 			m_recvBufferPos = 0;
 			return 0;
@@ -226,8 +231,7 @@ int CHttpControlSocket::DoReceive()
 
 		m_recvBufferPos += read;
 
-		if (!m_pHttpOpData->m_gotHeader)
-		{
+		if (!m_pHttpOpData->m_gotHeader) {
 			if (!read)
 			{
 				ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
@@ -272,8 +276,40 @@ int CHttpControlSocket::DoReceive()
 
 void CHttpControlSocket::OnConnect()
 {
-	LogMessage(Status, _("Connection established, sending HTTP request"));
-	ResetOperation(FZ_REPLY_OK);
+	wxASSERT(GetCurrentCommandId() == Command::connect);
+
+	CHttpConnectOpData *pData = static_cast<CHttpConnectOpData *>(m_pCurOpData);
+
+	if (pData->tls) {
+		if (!m_pTlsSocket) {
+			LogMessage(MessageType::Status, _("Connection established, initializing TLS..."));
+
+			delete m_pBackend;
+			m_pTlsSocket = new CTlsSocket(this, *m_pSocket, this);
+			m_pBackend = m_pTlsSocket;
+
+			if (!m_pTlsSocket->Init()) {
+				LogMessage(MessageType::Error, _("Failed to initialize TLS."));
+				DoClose();
+				return;
+			}
+
+			int res = m_pTlsSocket->Handshake();
+			if (res == FZ_REPLY_ERROR)
+				DoClose();
+		}
+		else {
+			LogMessage(MessageType::Status, _("TLS connection established, sending HTTP request"));
+			ResetOperation(FZ_REPLY_OK);
+		}
+
+		return;
+	}
+	else
+	{
+		LogMessage(MessageType::Status, _("Connection established, sending HTTP request"));
+		ResetOperation(FZ_REPLY_OK);
+	}
 }
 
 enum filetransferStates
@@ -285,11 +321,11 @@ enum filetransferStates
 
 int CHttpControlSocket::FileTransfer(const wxString localFile, const CServerPath &remotePath,
 							  const wxString &remoteFile, bool download,
-							  const CFileTransferCommand::t_transferSettings& transferSettings)
+							  const CFileTransferCommand::t_transferSettings&)
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::FileTransfer()"));
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::FileTransfer()"));
 
-	LogMessage(Status, _("Downloading %s"), remotePath.FormatFilename(remoteFile).c_str());
+	LogMessage(MessageType::Status, _("Downloading %s"), remotePath.FormatFilename(remoteFile));
 
 	if (!download)
 	{
@@ -299,18 +335,19 @@ int CHttpControlSocket::FileTransfer(const wxString localFile, const CServerPath
 
 	if (m_pCurOpData)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("deleting nonzero pData"));
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("deleting nonzero pData"));
 		delete m_pCurOpData;
 	}
 
-	CHttpFileTransferOpData *pData = new CHttpFileTransferOpData(localFile, remoteFile, remotePath);
+	CHttpFileTransferOpData *pData = new CHttpFileTransferOpData(download, localFile, remoteFile, remotePath);
 	m_pCurOpData = pData;
 	m_pHttpOpData = pData;
 
-	pData->download = download;
+	m_current_uri = wxURI(m_pCurrentServer->FormatServer() + pData->remotePath.FormatFilename(pData->remoteFile));
+
+	if (!localFile.empty()) {
+		pData->localFileSize = fz::local_filesys::get_size(fz::to_native(pData->localFile));
 
-	if (localFile != _T(""))
-	{
 		pData->opState = filetransfer_waitfileexists;
 		int res = CheckOverwriteFile();
 		if (res != FZ_REPLY_OK)
@@ -318,21 +355,14 @@ int CHttpControlSocket::FileTransfer(const wxString localFile, const CServerPath
 
 		pData->opState = filetransfer_transfer;
 
-		pData->pFile = new wxFile();
-
-		CreateLocalDir(pData->localFile);
-
-		if (!pData->pFile->Open(pData->localFile, wxFile::write))
-		{
-			LogMessage(::Error, _("Failed to open \"%s\" for writing"), pData->localFile.c_str());
-			ResetOperation(FZ_REPLY_ERROR);
-			return FZ_REPLY_ERROR;
-		}
+		res = OpenFile(pData);
+		if( res != FZ_REPLY_OK )
+			return res;
 	}
 	else
 		pData->opState = filetransfer_transfer;
 
-	int res = InternalConnect(m_pCurrentServer->GetHost(), m_pCurrentServer->GetPort());
+	int res = InternalConnect(m_pCurrentServer->GetHost(), m_pCurrentServer->GetPort(), m_pCurrentServer->GetProtocol() == HTTPS);
 	if (res != FZ_REPLY_OK)
 		return res;
 
@@ -341,11 +371,11 @@ int CHttpControlSocket::FileTransfer(const wxString localFile, const CServerPath
 
 int CHttpControlSocket::FileTransferSubcommandResult(int prevResult)
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::FileTransferSubcommandResult(%d)"), prevResult);
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::FileTransferSubcommandResult(%d)"), prevResult);
 
 	if (!m_pCurOpData)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty m_pCurOpData"));
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
@@ -361,11 +391,17 @@ int CHttpControlSocket::FileTransferSubcommandResult(int prevResult)
 
 int CHttpControlSocket::FileTransferSend()
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::FileTransferSend()"));
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::FileTransferSend()"));
 
 	if (!m_pCurOpData)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty m_pCurOpData"));
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
+		ResetOperation(FZ_REPLY_INTERNALERROR);
+		return FZ_REPLY_ERROR;
+	}
+
+	if( !m_current_uri.HasScheme() || !m_current_uri.HasServer() || !m_current_uri.HasPath() ) {
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("Invalid URI: %s"), m_current_uri.BuildURI());
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
@@ -375,39 +411,32 @@ int CHttpControlSocket::FileTransferSend()
 	if (pData->opState == filetransfer_waitfileexists)
 	{
 		pData->opState = filetransfer_transfer;
-		pData->pFile = new wxFile();
 
-		CreateLocalDir(pData->localFile);
+		int res = OpenFile(pData);
+		if( res != FZ_REPLY_OK)
+			return res;
 
-		if (!pData->pFile->Open(pData->localFile, wxFile::write))
-		{
-			LogMessage(::Error, _("Failed to open \"%s\" for writing"), pData->localFile.c_str());
-			ResetOperation(FZ_REPLY_ERROR);
-			return FZ_REPLY_ERROR;
-		}
-
-		int res = InternalConnect(m_pCurrentServer->GetHost(), m_pCurrentServer->GetPort());
+		res = InternalConnect(m_pCurrentServer->GetHost(), m_pCurrentServer->GetPort(), m_pCurrentServer->GetProtocol() == HTTPS);
 		if (res != FZ_REPLY_OK)
 			return res;
 	}
 
-	wxString location;
-	wxString hostWithPort;
-	if (pData->m_newLocation == _T(""))
-	{
-		location = _T("http://") + m_pCurrentServer->FormatHost() + pData->remotePath.FormatFilename(pData->remoteFile).c_str();
-		hostWithPort = wxString::Format(_T("%s:%d"), m_pCurrentServer->FormatHost(true).c_str(), m_pCurrentServer->GetPort());
+	wxString location = m_current_uri.GetPath();
+	if( m_current_uri.HasQuery() ) {
+		location += _T("?") + m_current_uri.GetQuery();
 	}
-	else
-	{
-		location = pData->m_newLocation;
-		hostWithPort = pData->m_newHostWithPort;
+	wxString action = wxString::Format(_T("GET %s HTTP/1.1"), location );
+	LogMessageRaw(MessageType::Command, action);
+
+	wxString hostWithPort = m_current_uri.GetServer();
+	if( m_current_uri.HasPort() ) {
+		hostWithPort += _T(":") + m_current_uri.GetPort();
 	}
-
-	wxString action = wxString::Format(_T("GET %s HTTP/1.1"), location.c_str());
-	LogMessageRaw(Command, action);
-
-	wxString command = wxString::Format(_T("%s\r\nHost: %s\r\nUser-Agent: %s\r\nConnection: close\r\n\r\n"), action.c_str(), hostWithPort.c_str(), wxString(PACKAGE_STRING, wxConvLocal).c_str());
+	wxString command = wxString::Format(_T("%s\r\nHost: %s\r\nUser-Agent: %s\r\nConnection: close\r\n"), action, hostWithPort, wxString(PACKAGE_STRING, wxConvLocal));
+	if( pData->resume ) {
+		command += wxString::Format(_T("Range: bytes=%") + wxString(wxFileOffsetFmtSpec) + _T("d-\r\n"), pData->localFileSize);
+	}
+	command += _T("\r\n");
 
 	const wxWX2MBbuf str = command.mb_str();
 	if (!Send(str, strlen(str)))
@@ -416,17 +445,19 @@ int CHttpControlSocket::FileTransferSend()
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-int CHttpControlSocket::InternalConnect(wxString host, unsigned short port)
+int CHttpControlSocket::InternalConnect(wxString host, unsigned short port, bool tls)
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::InternalConnect()"));
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::InternalConnect()"));
 
 	CHttpConnectOpData* pData = new CHttpConnectOpData;
 	pData->pNextOpData = m_pCurOpData;
 	m_pCurOpData = pData;
 	pData->port = port;
+	pData->tls = tls;
 
-	if (!IsIpAddress(host))
-		LogMessage(Status, _("Resolving address of %s"), host.c_str());
+	if (fz::get_address_type(host.ToStdWstring()) == fz::address_type::unknown) {
+		LogMessage(MessageType::Status, _("Resolving address of %s"), host);
+	}
 
 	pData->host = host;
 	return DoInternalConnect();
@@ -434,76 +465,68 @@ int CHttpControlSocket::InternalConnect(wxString host, unsigned short port)
 
 int CHttpControlSocket::DoInternalConnect()
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::DoInternalConnect()"));
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::DoInternalConnect()"));
 
 	if (!m_pCurOpData)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty m_pCurOpData"));
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
 
 	CHttpConnectOpData *pData = static_cast<CHttpConnectOpData *>(m_pCurOpData);
-	LogMessage(Status, _("Connecting to %s:%d..."), pData->host.c_str(), pData->port);
 
-	if (m_pBackend)
-		delete m_pBackend;
-	m_pBackend = new CSocketBackend(this, m_pSocket);
+	delete m_pBackend;
+	m_pBackend = new CSocketBackend(this, *m_pSocket, engine_.GetRateLimiter());
 
-	int res = m_pSocket->Connect(pData->host, pData->port);
+	int res = m_pSocket->Connect(fz::to_native(pData->host), pData->port);
 	if (!res)
 		return FZ_REPLY_OK;
 
 	if (res && res != EINPROGRESS)
 		return ResetOperation(FZ_REPLY_ERROR);
-	
+
 	return FZ_REPLY_WOULDBLOCK;
 }
 
 int CHttpControlSocket::FileTransferParseResponse(char* p, unsigned int len)
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::FileTransferParseResponse(%p, %d)"), p, len);
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::FileTransferParseResponse(%p, %d)"), p, len);
 
-	if (!m_pCurOpData)
-	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty m_pCurOpData"));
+	if (!m_pCurOpData) {
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
 
 	CHttpFileTransferOpData *pData = static_cast<CHttpFileTransferOpData *>(m_pCurOpData);
 
-	if (!p)
-	{
+	if (!p) {
 		ResetOperation(FZ_REPLY_OK);
 		return FZ_REPLY_OK;
 	}
 
-	if (!m_pTransferStatus)
-	{
-		InitTransferStatus(pData->m_totalSize.GetLo() + ((wxFileOffset)pData->m_totalSize.GetHi() << 32), 0, false);
-		SetTransferStatusStartTime();
+	if (engine_.transfer_status_.empty()) {
+		engine_.transfer_status_.Init(pData->m_totalSize, 0, false);
+		engine_.transfer_status_.SetStartTime();
 	}
 
-	if (pData->localFile == _T(""))
-	{
+	if (pData->localFile.empty()) {
 		char* q = new char[len];
 		memcpy(q, p, len);
-		m_pEngine->AddNotification(new CDataNotification(q, len));
+		engine_.AddNotification(new CDataNotification(q, len));
 	}
-	else
-	{
+	else {
 		wxASSERT(pData->pFile);
 
-		if (pData->pFile->Write(p, len) != len)
-		{
-			LogMessage(::Error, _("Failed to write to file %s"), pData->localFile.c_str());
+		if (pData->pFile->Write(p, len) != len) {
+			LogMessage(MessageType::Error, _("Failed to write to file %s"), pData->localFile);
 			ResetOperation(FZ_REPLY_ERROR);
 			return FZ_REPLY_ERROR;
 		}
 	}
 
-	UpdateTransferStatus(len);
+	engine_.transfer_status_.Update(len);
 
 	return FZ_REPLY_WOULDBLOCK;
 }
@@ -514,8 +537,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 	// We do just the neccessary parsing and silently ignore most header fields
 	// Redirects are supported though if the server sends the Location field.
 
-	while (true)
-	{
+	for (;;) {
 		// Find line ending
 		unsigned int i = 0;
 		for (i = 0; (i + 1) < m_recvBufferPos; i++)
@@ -524,7 +546,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 			{
 				if (m_pRecvBuffer[i + 1] != '\n')
 				{
-					LogMessage(::Error, _("Malformed reply, server not sending proper line endings"));
+					LogMessage(MessageType::Error, _("Malformed reply, server not sending proper line endings"));
 					ResetOperation(FZ_REPLY_ERROR);
 					return FZ_REPLY_ERROR;
 				}
@@ -536,7 +558,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 			if (m_recvBufferPos == m_recvBufferLen)
 			{
 				// We don't support header lines larger than 4096
-				LogMessage(::Error, _("Too long header line"));
+				LogMessage(MessageType::Error, _("Too long header line"));
 				ResetOperation(FZ_REPLY_ERROR);
 				return FZ_REPLY_ERROR;
 			}
@@ -544,9 +566,9 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 		}
 
 		m_pRecvBuffer[i] = 0;
-		const wxString& line = wxString(m_pRecvBuffer, wxConvLocal);
-		if (line != _T(""))
-			LogMessageRaw(Response, line);
+		wxString const line = wxString(m_pRecvBuffer, wxConvLocal);
+		if (!line.empty())
+			LogMessageRaw(MessageType::Response, line);
 
 		if (pData->m_responseCode == -1)
 		{
@@ -554,7 +576,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 			if (m_recvBufferPos < 16 || memcmp(m_pRecvBuffer, "HTTP/1.", 7))
 			{
 				// Invalid HTTP Status-Line
-				LogMessage(::Error, _("Invalid HTTP Response"));
+				LogMessage(MessageType::Error, _("Invalid HTTP Response"));
 				ResetOperation(FZ_REPLY_ERROR);
 				return FZ_REPLY_ERROR;
 			}
@@ -564,13 +586,28 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 				m_pRecvBuffer[11] < '0' || m_pRecvBuffer[11] > '9')
 			{
 				// Invalid response code
-				LogMessage(::Error, _("Invalid response code"));
+				LogMessage(MessageType::Error, _("Invalid response code"));
 				ResetOperation(FZ_REPLY_ERROR);
 				return FZ_REPLY_ERROR;
 			}
 
 			pData->m_responseCode = (m_pRecvBuffer[9] - '0') * 100 + (m_pRecvBuffer[10] - '0') * 10 + m_pRecvBuffer[11] - '0';
 
+			if( pData->m_responseCode == 416 ) {
+				CHttpFileTransferOpData* pTransfer = static_cast<CHttpFileTransferOpData*>(pData->m_pOpData);
+				if( pTransfer->resume ) {
+					// Sad, the server does not like our attempt to resume.
+					// Get full file instead.
+					pTransfer->resume = false;
+					int res = OpenFile(pTransfer);
+					if( res != FZ_REPLY_OK ) {
+						return res;
+					}
+					pData->m_newLocation = m_current_uri;
+					pData->m_responseCode = 300;
+				}
+			}
+
 			if (pData->m_responseCode >= 400)
 			{
 				// Failed request
@@ -581,7 +618,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 			if (pData->m_responseCode == 305)
 			{
 				// Unsupported redirect
-				LogMessage(::Error, _("Unsupported redirect"));
+				LogMessage(MessageType::Error, _("Unsupported redirect"));
 				ResetOperation(FZ_REPLY_ERROR);
 				return FZ_REPLY_ERROR;
 			}
@@ -595,9 +632,8 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 				// Redirect if neccessary
 				if (pData->m_responseCode >= 300)
 				{
-					if (pData->m_redirectionCount++ == 5)
-					{
-						LogMessage(::Error, _("Too many redirects"));
+					if (pData->m_redirectionCount++ == 6) {
+						LogMessage(MessageType::Error, _("Too many redirects"));
 						ResetOperation(FZ_REPLY_ERROR);
 						return FZ_REPLY_ERROR;
 					}
@@ -605,45 +641,48 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 					ResetSocket();
 					ResetHttpData(pData);
 
-					wxString host;
-					int pos;
-					if ((pos = pData->m_newLocation.Find(_T("://"))) != -1)
-						host = pData->m_newLocation.Mid(pos + 3);
-					else
-						host = pData->m_newLocation;
-
-					if ((pos = host.Find(_T("/"))) != -1)
-						host = host.Left(pos);
-
-					unsigned long port;
-					if ((pos = host.Find(':', true)) != -1)
-					{
-						wxString strport = host.Mid(pos + 1);
-						if (!strport.ToULong(&port) || port < 1 || port > 65535)
-							port = 80;
-						host = host.Left(pos);
-					}
-					else
-						port = 80;
-
-					if (host == _T(""))
-					{
-						// Unsupported redirect
-						LogMessage(::Error, _("Redirection to invalid address"));
+					if( !pData->m_newLocation.HasScheme() || !pData->m_newLocation.HasServer() || !pData->m_newLocation.HasPath() ) {
+						LogMessage(MessageType::Error, _("Redirection to invalid or unsupported URI: %s"), m_current_uri.BuildURI());
 						ResetOperation(FZ_REPLY_ERROR);
 						return FZ_REPLY_ERROR;
 					}
-					pData->m_newHostWithPort = wxString::Format(_T("%s:%d"), host.c_str(), (int)port);
+
+					enum ServerProtocol protocol = CServer::GetProtocolFromPrefix(pData->m_newLocation.GetScheme());
+					if( protocol != HTTP && protocol != HTTPS ) {
+						LogMessage(MessageType::Error, _("Redirection to invalid or unsupported address: %s"), pData->m_newLocation.BuildURI());
+						ResetOperation(FZ_REPLY_ERROR);
+						return FZ_REPLY_ERROR;
+					}
+
+					long port = CServer::GetDefaultPort(protocol);
+					if( pData->m_newLocation.HasPort() && (!pData->m_newLocation.GetPort().ToLong(&port) || port < 1 || port > 65535) ) {
+						LogMessage(MessageType::Error, _("Redirection to invalid or unsupported address: %s"), pData->m_newLocation.BuildURI());
+						ResetOperation(FZ_REPLY_ERROR);
+						return FZ_REPLY_ERROR;
+					}
+
+					m_current_uri = pData->m_newLocation;
 
 					// International domain names
-					host = ConvertDomainName(host);
+					wxString host = ConvertDomainName(m_current_uri.GetServer());
 
-					int res = InternalConnect(host, port);
+					int res = InternalConnect(host, static_cast<unsigned short>(port), protocol == HTTPS);
 					if (res == FZ_REPLY_WOULDBLOCK)
 						res |= FZ_REPLY_REDIRECTED;
 					return res;
 				}
 
+				if( pData->m_pOpData && pData->m_pOpData->opId == Command::transfer) {
+					CHttpFileTransferOpData* pTransfer = static_cast<CHttpFileTransferOpData*>(pData->m_pOpData);
+					if( pTransfer->resume && pData->m_responseCode != 206 ) {
+						pTransfer->resume = false;
+						int res = OpenFile(pTransfer);
+						if( res != FZ_REPLY_OK ) {
+							return res;
+						}
+					}
+				}
+
 				pData->m_gotHeader = true;
 
 				memmove(m_pRecvBuffer, m_pRecvBuffer + 2, m_recvBufferPos - 2);
@@ -667,7 +706,8 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 			}
 			if (m_recvBufferPos > 12 && !memcmp(m_pRecvBuffer, "Location: ", 10))
 			{
-				pData->m_newLocation = wxString(m_pRecvBuffer + 10, wxConvLocal);
+				pData->m_newLocation = wxURI(wxString(m_pRecvBuffer + 10, wxConvLocal));
+				pData->m_newLocation.Resolve(m_current_uri);
 			}
 			else if (m_recvBufferPos > 21 && !memcmp(m_pRecvBuffer, "Transfer-Encoding: ", 19))
 			{
@@ -686,7 +726,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 				{
 					if (*p < '0' || *p > '9')
 					{
-						LogMessage(::Error, _("Malformed header: %s"), _("Invalid Content-Length"));
+						LogMessage(MessageType::Error, _("Malformed header: %s"), _("Invalid Content-Length"));
 						ResetOperation(FZ_REPLY_ERROR);
 						return FZ_REPLY_ERROR;
 					}
@@ -710,19 +750,19 @@ int CHttpControlSocket::OnChunkedData(CHttpOpData* pData)
 	char* p = m_pRecvBuffer;
 	unsigned int len = m_recvBufferPos;
 
-	while (true)
+	for (;;)
 	{
 		if (pData->m_chunkData.size != 0)
 		{
 			unsigned int dataLen = len;
 			if (pData->m_chunkData.size < len)
-				dataLen = pData->m_chunkData.size.GetLo();
+				dataLen = static_cast<unsigned int>(pData->m_chunkData.size);
 			pData->m_receivedData += dataLen;
 			int res = ProcessData(p, dataLen);
 			if (res != FZ_REPLY_WOULDBLOCK)
 				return res;
 
-            pData->m_chunkData.size -= dataLen;
+			pData->m_chunkData.size -= dataLen;
 			p += dataLen;
 			len -= dataLen;
 
@@ -741,7 +781,7 @@ int CHttpControlSocket::OnChunkedData(CHttpOpData* pData)
 			{
 				if (p[i + 1] != '\n')
 				{
-					LogMessage(::Error, _("Malformed chunk data: %s"), _("Wrong line endings"));
+					LogMessage(MessageType::Error, _("Malformed chunk data: %s"), _("Wrong line endings"));
 					ResetOperation(FZ_REPLY_ERROR);
 					return FZ_REPLY_ERROR;
 				}
@@ -753,7 +793,7 @@ int CHttpControlSocket::OnChunkedData(CHttpOpData* pData)
 			if (len == m_recvBufferLen)
 			{
 				// We don't support lines larger than 4096
-				LogMessage(::Error, _("Malformed chunk data: %s"), _("Line length exceeded"));
+				LogMessage(MessageType::Error, _("Malformed chunk data: %s"), _("Line length exceeded"));
 				ResetOperation(FZ_REPLY_ERROR);
 				return FZ_REPLY_ERROR;
 			}
@@ -768,7 +808,7 @@ int CHttpControlSocket::OnChunkedData(CHttpOpData* pData)
 			{
 				// The chunk data has to end with CRLF. If i is nonzero,
 				// it didn't end with just CRLF.
-				LogMessage(::Error, _("Malformed chunk data: %s"), _("Chunk data improperly terminated"));
+				LogMessage(MessageType::Error, _("Malformed chunk data: %s"), _("Chunk data improperly terminated"));
 				ResetOperation(FZ_REPLY_ERROR);
 				return FZ_REPLY_ERROR;
 			}
@@ -787,34 +827,23 @@ int CHttpControlSocket::OnChunkedData(CHttpOpData* pData)
 		else
 		{
 			// Read chunk size
-			char* q = p;
-			while (*q)
-			{
-				if (*q >= '0' && *q <= '9')
-				{
-					pData->m_chunkData.size *= 16;
+			for( char* q = p; *q && *q != ';' && *q != ' '; ++q ) {
+				pData->m_chunkData.size *= 16;
+				if (*q >= '0' && *q <= '9') {
 					pData->m_chunkData.size += *q - '0';
 				}
-				else if (*q >= 'A' && *q <= 'F')
-				{
-					pData->m_chunkData.size *= 16;
+				else if (*q >= 'A' && *q <= 'F') {
 					pData->m_chunkData.size += *q - 'A' + 10;
 				}
-				else if (*q >= 'a' && *q <= 'f')
-				{
-					pData->m_chunkData.size *= 16;
+				else if (*q >= 'a' && *q <= 'f') {
 					pData->m_chunkData.size += *q - 'a' + 10;
 				}
-				else if (*q == ';' || *q == ' ')
-					break;
-				else
-				{
+				else {
 					// Invalid size
-					LogMessage(::Error, _("Malformed chunk data: %s"), _("Invalid chunk size"));
+					LogMessage(MessageType::Error, _("Malformed chunk data: %s"), _("Invalid chunk size"));
 					ResetOperation(FZ_REPLY_ERROR);
 					return FZ_REPLY_ERROR;
 				}
-				q++;
 			}
 			if (pData->m_chunkData.size == 0)
 				pData->m_chunkData.getTrailer = true;
@@ -838,7 +867,7 @@ int CHttpControlSocket::OnChunkedData(CHttpOpData* pData)
 
 int CHttpControlSocket::ResetOperation(int nErrorCode)
 {
-	if (m_pCurOpData && m_pCurOpData->opId == cmd_transfer)
+	if (m_pCurOpData && m_pCurOpData->opId == Command::transfer)
 	{
 		CHttpFileTransferOpData *pData = static_cast<CHttpFileTransferOpData *>(m_pCurOpData);
 		delete pData->pFile;
@@ -847,10 +876,13 @@ int CHttpControlSocket::ResetOperation(int nErrorCode)
 
 	if (!m_pCurOpData || !m_pCurOpData->pNextOpData)
 	{
-		if (nErrorCode == FZ_REPLY_OK)
-			LogMessage(Status, _("Disconnected from server"));
-		else
-			LogMessage(::Error, _("Disconnected from server"));
+		if (m_pBackend)
+		{
+			if (nErrorCode == FZ_REPLY_OK)
+				LogMessage(MessageType::Status, _("Disconnected from server"));
+			else
+				LogMessage(MessageType::Error, _("Disconnected from server"));
+		}
 		ResetSocket();
 		m_pHttpOpData = 0;
 	}
@@ -860,46 +892,36 @@ int CHttpControlSocket::ResetOperation(int nErrorCode)
 
 void CHttpControlSocket::OnClose(int error)
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::OnClose(%d)"), error);
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::OnClose(%d)"), error);
 
-	if (error)
-	{
-		LogMessage(::Error, _("Disconnected from server: %s"), CSocket::GetErrorDescription(error).c_str());
+	if (error) {
+		LogMessage(MessageType::Error, _("Disconnected from server: %s"), CSocket::GetErrorDescription(error));
 		ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
 		return;
 	}
-	
-	if (DoReceive() == FZ_REPLY_REDIRECTED)
-		return; // Socket got closed already
 
 	// HTTP socket isn't connected outside operations
 	if (!m_pCurOpData)
 		return;
 
-	if (m_pCurOpData->pNextOpData)
-	{
+	if (m_pCurOpData->pNextOpData) {
 		ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
 		return;
 	}
 
-	if (!m_pHttpOpData->m_gotHeader)
-	{
+	if (!m_pHttpOpData->m_gotHeader) {
 		ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
 		return;
 	}
 
-	if (m_pHttpOpData->m_transferEncoding == CHttpOpData::chunked)
-	{
-		if (!m_pHttpOpData->m_chunkData.getTrailer)
-		{
+	if (m_pHttpOpData->m_transferEncoding == CHttpOpData::chunked) {
+		if (!m_pHttpOpData->m_chunkData.getTrailer) {
 			ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
 			return;
 		}
 	}
-	else
-	{
-		if (m_pHttpOpData->m_totalSize != -1 && m_pHttpOpData->m_receivedData != m_pHttpOpData->m_totalSize)
-		{
+	else {
+		if (m_pHttpOpData->m_totalSize != -1 && m_pHttpOpData->m_receivedData != m_pHttpOpData->m_totalSize) {
 			ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
 			return;
 		}
@@ -930,14 +952,14 @@ void CHttpControlSocket::ResetHttpData(CHttpOpData* pData)
 int CHttpControlSocket::ProcessData(char* p, int len)
 {
 	int res;
-	enum Command commandId = GetCurrentCommandId();
+	Command commandId = GetCurrentCommandId();
 	switch (commandId)
 	{
-	case cmd_transfer:
+	case Command::transfer:
 		res = FileTransferParseResponse(p, len);
 		break;
 	default:
-		LogMessage(Debug_Warning, _T("No action for parsing data for command %d"), (int)commandId);
+		LogMessage(MessageType::Debug_Warning, _T("No action for parsing data for command %d"), (int)commandId);
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		res = FZ_REPLY_ERROR;
 		break;
@@ -950,23 +972,47 @@ int CHttpControlSocket::ProcessData(char* p, int len)
 
 int CHttpControlSocket::ParseSubcommandResult(int prevResult)
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::SendNextCommand(%d)"), prevResult);
-	if (!m_pCurOpData)
-	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("SendNextCommand called without active operation"));
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::SendNextCommand(%d)"), prevResult);
+	if (!m_pCurOpData) {
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("SendNextCommand called without active operation"));
 		ResetOperation(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
 	}
 
 	switch (m_pCurOpData->opId)
 	{
-	case cmd_transfer:
+	case Command::transfer:
 		return FileTransferSubcommandResult(prevResult);
 	default:
-		LogMessage(__TFILE__, __LINE__, this, ::Debug_Warning, _T("Unknown opID (%d) in SendNextCommand"), m_pCurOpData->opId);
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("Unknown opID (%d) in SendNextCommand"), m_pCurOpData->opId);
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		break;
 	}
 
 	return FZ_REPLY_ERROR;
 }
+
+int CHttpControlSocket::Disconnect()
+{
+	DoClose();
+	return FZ_REPLY_OK;
+}
+
+int CHttpControlSocket::OpenFile(CHttpFileTransferOpData* pData)
+{
+	delete pData->pFile;
+	pData->pFile = new wxFile();
+	CreateLocalDir(pData->localFile);
+
+	if (!pData->pFile->Open(pData->localFile, pData->resume ? wxFile::write_append : wxFile::write)) {
+		LogMessage(MessageType::Error, _("Failed to open \"%s\" for writing"), pData->localFile);
+		ResetOperation(FZ_REPLY_ERROR);
+		return FZ_REPLY_ERROR;
+	}
+	wxFileOffset end = pData->pFile->SeekEnd();
+	if (!end) {
+		pData->resume = false;
+	}
+	pData->localFileSize = fz::local_filesys::get_size(fz::to_native(pData->localFile));
+	return FZ_REPLY_OK;
+}
