@@ -21,21 +21,21 @@ int CLogging::m_max_size;
 wxString CLogging::m_file;
 
 int CLogging::m_refcount = 0;
-mutex CLogging::mutex_(false);
+fz::mutex CLogging::mutex_(false);
 
-thread_local int CLogging::debug_level_{0};
-thread_local int CLogging::raw_listing_{0};
+thread_local int CLogging::debug_level_{};
+thread_local int CLogging::raw_listing_{};
 
 CLogging::CLogging(CFileZillaEnginePrivate & engine)
 	: engine_(engine)
 {
-	scoped_lock l(mutex_);
+	fz::scoped_lock l(mutex_);
 	m_refcount++;
 }
 
 CLogging::~CLogging()
 {
-	scoped_lock l(mutex_);
+	fz::scoped_lock l(mutex_);
 	m_refcount--;
 
 	if (!m_refcount) {
@@ -83,27 +83,28 @@ bool CLogging::ShouldLog(MessageType nMessageType) const
 	return true;
 }
 
-void CLogging::InitLogFile() const
+bool CLogging::InitLogFile(fz::scoped_lock& l) const
 {
 	if (m_logfile_initialized)
-		return;
+		return true;
 
 	m_logfile_initialized = true;
 
 	m_file = engine_.GetOptions().GetOption(OPTION_LOGGING_FILE);
 	if (m_file.empty())
-		return;
+		return false;
 
 #ifdef __WXMSW__
-	m_log_fd = CreateFile(m_file, FILE_APPEND_DATA, FILE_SHARE_DELETE | FILE_SHARE_WRITE | FILE_SHARE_READ, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
+	m_log_fd = CreateFile(m_file.wc_str(), FILE_APPEND_DATA, FILE_SHARE_DELETE | FILE_SHARE_WRITE | FILE_SHARE_READ, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
 	if (m_log_fd == INVALID_HANDLE_VALUE)
 #else
 	m_log_fd = open(m_file.fn_str(), O_WRONLY | O_APPEND | O_CREAT | O_CLOEXEC, 0644);
 	if (m_log_fd == -1)
 #endif
 	{
+		l.unlock(); //Avoid recursion
 		LogMessage(MessageType::Error, _("Could not open log file: %s"), wxSysErrorMsg());
-		return;
+		return false;
 	}
 
 	m_prefixes[static_cast<int>(MessageType::Status)] = _("Status:");
@@ -124,13 +125,19 @@ void CLogging::InitLogFile() const
 	else if (m_max_size > 2000)
 		m_max_size = 2000;
 	m_max_size *= 1024 * 1024;
+
+	return true;
 }
 
 void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 {
-	scoped_lock l(mutex_);
+	fz::scoped_lock l(mutex_);
 
-	InitLogFile();
+	if (!m_logfile_initialized) {
+		if (!InitLogFile(l)) {
+			return;
+		}
+	}
 #ifdef __WXMSW__
 	if (m_log_fd == INVALID_HANDLE_VALUE)
 		return;
@@ -139,14 +146,14 @@ void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 		return;
 #endif
 
-	wxDateTime now = wxDateTime::Now();
+	fz::datetime now = fz::datetime::now();
 	wxString out(wxString::Format(_T("%s %u %d %s %s")
 #ifdef __WXMSW__
 		_T("\r\n"),
 #else
 		_T("\n"),
 #endif
-		now.Format(_T("%Y-%m-%d %H:%M:%S")), m_pid, engine_.GetEngineId(), m_prefixes[static_cast<int>(nMessageType)], msg));
+		now.format(_T("%Y-%m-%d %H:%M:%S"), fz::datetime::local), m_pid, engine_.GetEngineId(), m_prefixes[static_cast<int>(nMessageType)], msg));
 
 	const wxWX2MBbuf utf8 = out.mb_str(wxConvUTF8);
 	if (utf8) {
@@ -155,12 +162,19 @@ void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 			LARGE_INTEGER size;
 			if (!GetFileSizeEx(m_log_fd, &size) || size.QuadPart > m_max_size) {
 				CloseHandle(m_log_fd);
+				m_log_fd = INVALID_HANDLE_VALUE;
 
 				// m_log_fd might no longer be the original file.
 				// Recheck on a new handle. Proteced with a mutex against other processes
 				HANDLE hMutex = ::CreateMutex(0, true, _T("FileZilla 3 Logrotate Mutex"));
+				if (!hMutex) {
+					wxString error = wxSysErrorMsg();
+					l.unlock();
+					LogMessage(MessageType::Error, _("Could not create logging mutex: %s"), error);
+					return;
+				}
 
-				HANDLE hFile = CreateFile(m_file, FILE_APPEND_DATA, FILE_SHARE_DELETE | FILE_SHARE_WRITE | FILE_SHARE_READ, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
+				HANDLE hFile = CreateFile(m_file.wc_str(), FILE_APPEND_DATA, FILE_SHARE_DELETE | FILE_SHARE_WRITE | FILE_SHARE_READ, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
 				if (hFile == INVALID_HANDLE_VALUE) {
 					wxString error = wxSysErrorMsg();
 
@@ -168,7 +182,7 @@ void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 					ReleaseMutex(hMutex);
 					CloseHandle(hMutex);
 
-					m_log_fd = INVALID_HANDLE_VALUE;
+					l.unlock(); // Avoid recursion
 					LogMessage(MessageType::Error, _("Could not open log file: %s"), error);
 					return;
 				}
@@ -182,12 +196,11 @@ void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 					// Todo: Handle the case in which logdir and tmpdir are on different volumes.
 					// (Why is everthing so needlessly complex on MSW?)
 					wxString tmp = wxFileName::CreateTempFileName(_T("fz3"));
-					MoveFileEx(m_file + _T(".1"), tmp, MOVEFILE_REPLACE_EXISTING);
-					DeleteFile(tmp);
-					MoveFileEx(m_file, m_file + _T(".1"), MOVEFILE_REPLACE_EXISTING);
-					m_log_fd = CreateFile(m_file, FILE_APPEND_DATA, FILE_SHARE_DELETE | FILE_SHARE_WRITE | FILE_SHARE_READ, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
-					if (m_log_fd == INVALID_HANDLE_VALUE)
-					{
+					MoveFileEx((m_file + _T(".1")).wc_str(), tmp.wc_str(), MOVEFILE_REPLACE_EXISTING);
+					DeleteFile(tmp.wc_str());
+					MoveFileEx(m_file.wc_str(), (m_file + _T(".1")).wc_str(), MOVEFILE_REPLACE_EXISTING);
+					m_log_fd = CreateFile(m_file.wc_str(), FILE_APPEND_DATA, FILE_SHARE_DELETE | FILE_SHARE_WRITE | FILE_SHARE_READ, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
+					if (m_log_fd == INVALID_HANDLE_VALUE) {
 						// If this function would return bool, I'd return FILE_NOT_FOUND here.
 						error = wxSysErrorMsg();
 					}
@@ -200,25 +213,28 @@ void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 					CloseHandle(hMutex);
 				}
 
-				if (!error.empty())
+				if (!error.empty()) {
+					l.unlock(); // Avoid recursion
 					LogMessage(MessageType::Error, _("Could not open log file: %s"), error);
+					return;
+				}
 			}
 		}
 		DWORD len = (DWORD)strlen((const char*)utf8);
 		DWORD written;
 		BOOL res = WriteFile(m_log_fd, (const char*)utf8, len, &written, 0);
-		if (!res || written != len)
-		{
-			LogMessage(MessageType::Error, _("Could not write to log file: %s"), wxSysErrorMsg());
+		if (!res || written != len) {
 			CloseHandle(m_log_fd);
 			m_log_fd = INVALID_HANDLE_VALUE;
+			l.unlock(); // Avoid recursion
+			LogMessage(MessageType::Error, _("Could not write to log file: %s"), wxSysErrorMsg());
 		}
 #else
 		if (m_max_size) {
 			struct stat buf;
 			int rc = fstat(m_log_fd, &buf);
 			while (!rc && buf.st_size > m_max_size) {
-				struct flock lock = {0};
+				struct flock lock = {};
 				lock.l_type = F_WRLCK;
 				lock.l_whence = SEEK_SET;
 				lock.l_start = 0;
@@ -237,6 +253,7 @@ void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 					close(m_log_fd);
 					m_log_fd = -1;
 
+					l.unlock(); // Avoid recursion
 					LogMessage(MessageType::Error, error);
 					return;
 				}
@@ -261,6 +278,7 @@ void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 				// Get the new file
 				m_log_fd = open(m_file.fn_str(), O_WRONLY | O_APPEND | O_CREAT | O_CLOEXEC, 0644);
 				if (m_log_fd == -1) {
+					l.unlock(); // Avoid recursion
 					LogMessage(MessageType::Error, wxSysErrorMsg());
 					return;
 				}
@@ -274,6 +292,8 @@ void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 		if (written != len) {
 			close(m_log_fd);
 			m_log_fd = -1;
+
+			l.unlock(); // Avoid recursion
 			LogMessage(MessageType::Error, _("Could not write to log file: %s"), wxSysErrorMsg());
 		}
 #endif
