@@ -1,24 +1,15 @@
 #include <wx/defs.h>
-#ifdef __WXMSW__
-  #ifndef NOMINMAX
-  #define NOMINMAX
-  #endif
-  // MinGW needs this for getaddrinfo
-  #if defined(_WIN32_WINNT)
-    #if _WIN32_WINNT < 0x0600
-      #undef _WIN32_WINNT
-      #define _WIN32_WINNT 0x0600
-    #endif
-  #else
-    #define _WIN32_WINNT 0x0600
-  #endif
+#include <libfilezilla/libfilezilla.hpp>
+#ifdef FZ_WINDOWS
+  #include <libfilezilla/private/windows.hpp>
   #include <winsock2.h>
   #include <ws2tcpip.h>
 #endif
 #include <filezilla.h>
-#include "mutex.h"
+#include <libfilezilla/mutex.hpp>
 #include "socket.h"
-#ifndef __WXMSW__
+#ifndef FZ_WINDOWS
+  #define mutex mutex_override // Sadly on some platforms system headers include conflicting names
   #include <sys/types.h>
   #include <sys/socket.h>
   #include <netdb.h>
@@ -29,6 +20,7 @@
   #if !defined(MSG_NOSIGNAL) && !defined(SO_NOSIGPIPE)
     #include <signal.h>
   #endif
+  #undef mutex
 #endif
 
 // Fixups needed on FreeBSD
@@ -58,17 +50,15 @@ union sockaddr_u
 #define WAIT_EVENTCOUNT 5
 
 class CSocketThread;
+
 namespace {
 static std::vector<CSocketThread*> waiting_socket_threads;
-static mutex waiting_socket_threads_mutex{false};
-};
+static fz::mutex waiting_socket_threads_mutex{ false };
+}
 
-struct socket_event_type;
-typedef CEvent<socket_event_type> CInternalSocketEvent;
-
-void RemoveSocketEvents(CEventHandler * handler, CSocketEventSource const* const source)
+void RemoveSocketEvents(fz::event_handler * handler, CSocketEventSource const* const source)
 {
-	auto socketEventFilter = [&](CEventLoop::Events::value_type const& ev) -> bool {
+	auto socketEventFilter = [&](fz::event_loop::Events::value_type const& ev) -> bool {
 		if (ev.first != handler) {
 			return false;
 		}
@@ -81,10 +71,10 @@ void RemoveSocketEvents(CEventHandler * handler, CSocketEventSource const* const
 		return false;
 	};
 
-	handler->event_loop_.FilterEvents(socketEventFilter);
+	handler->event_loop_.filter_events(socketEventFilter);
 }
 
-void ChangeSocketEventHandler(CEventHandler * oldHandler, CEventHandler * newHandler, CSocketEventSource const* const source)
+void ChangeSocketEventHandler(fz::event_handler * oldHandler, fz::event_handler * newHandler, CSocketEventSource const* const source)
 {
 	if (!oldHandler)
 		return;
@@ -96,7 +86,7 @@ void ChangeSocketEventHandler(CEventHandler * oldHandler, CEventHandler * newHan
 		RemoveSocketEvents(oldHandler, source);
 	}
 	else {
-		auto socketEventFilter = [&](CEventLoop::Events::value_type & ev) -> bool {
+		auto socketEventFilter = [&](fz::event_loop::Events::value_type & ev) -> bool {
 			if (ev.first == oldHandler) {
 				if (ev.second->derived_type() == CSocketEvent::type()) {
 					if (std::get<0>(static_cast<CSocketEvent const&>(*ev.second).v_) == source) {
@@ -112,12 +102,12 @@ void ChangeSocketEventHandler(CEventHandler * oldHandler, CEventHandler * newHan
 			return false;
 		};
 
-		oldHandler->event_loop_.FilterEvents(socketEventFilter);
+		oldHandler->event_loop_.filter_events(socketEventFilter);
 	}
 }
 
 namespace {
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 static int ConvertMSWErrorCode(int error)
 {
 	switch (error)
@@ -160,6 +150,8 @@ static int ConvertMSWErrorCode(int error)
 		return EHOSTDOWN;
 	case WSAENETUNREACH:
 		return ENETUNREACH;
+	case WSAEADDRINUSE:
+		return EADDRINUSE;
 	default:
 		return error;
 	}
@@ -183,7 +175,7 @@ public:
 			0
 		};
 		for (int i = 0; errors[i]; ++i) {
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 			int code = ConvertMSWErrorCode(errors[i]);
 #else
 			int code = errors[i];
@@ -197,14 +189,14 @@ public:
 #endif
 }
 
-class CSocketThread final : protected wxThread
+class CSocketThread final : protected fz::thread
 {
 	friend class CSocket;
 public:
 	CSocketThread()
-		: wxThread(wxTHREAD_JOINABLE), m_sync(false)
+		: m_sync(false)
 	{
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 		m_sync_event = WSA_INVALID_EVENT;
 #else
 		m_pipe[0] = -1;
@@ -217,7 +209,8 @@ public:
 
 	virtual ~CSocketThread()
 	{
-#ifdef __WXMSW__
+		join();
+#ifdef FZ_WINDOWS
 		if (m_sync_event != WSA_INVALID_EVENT)
 			WSACloseEvent(m_sync_event);
 #else
@@ -230,11 +223,11 @@ public:
 
 	void SetSocket(CSocket* pSocket)
 	{
-		scoped_lock l(m_sync);
+		fz::scoped_lock l(m_sync);
 		SetSocket(pSocket, l);
 	}
 
-	void SetSocket(CSocket* pSocket, scoped_lock const&)
+	void SetSocket(CSocket* pSocket, fz::scoped_lock const&)
 	{
 		m_pSocket = pSocket;
 
@@ -244,20 +237,19 @@ public:
 		m_waiting = 0;
 	}
 
-	int Connect(wxString const& bind)
+	int Connect(std::string const& bind)
 	{
-		wxASSERT(m_pSocket);
-
-		auto const hostBuf = m_pSocket->m_host.mb_str();
-		auto const bindBuf = bind.mb_str();
-		if (!hostBuf || !bindBuf) {
-			m_bind.clear();
-			m_host.clear();
-			m_port.clear();
+		assert(m_pSocket);
+		if (!m_pSocket) {
 			return EINVAL;
 		}
-		m_host = hostBuf;
-		m_bind = bindBuf;
+
+		m_host = fz::to_utf8(m_pSocket->m_host);
+		if (m_host.empty()) {
+			return EINVAL;
+		}
+
+		m_bind = bind;
 
 		// Connect method of CSocket ensures port is in range
 		char tmp[7];
@@ -273,14 +265,14 @@ public:
 	int Start()
 	{
 		if (m_started) {
-			scoped_lock l(m_sync);
-			wxASSERT(m_threadwait);
+			fz::scoped_lock l(m_sync);
+			assert(m_threadwait);
 			m_waiting = 0;
 			WakeupThread(l);
 			return 0;
 		}
 		m_started = true;
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 		if (m_sync_event == WSA_INVALID_EVENT)
 			m_sync_event = WSACreateEvent();
 		if (m_sync_event == WSA_INVALID_EVENT)
@@ -292,11 +284,9 @@ public:
 		}
 #endif
 
-		int res = Create();
-		if (res != wxTHREAD_NO_ERROR)
+		if (!run()) {
 			return 1;
-
-		Run();
+		}
 
 		return 0;
 	}
@@ -304,11 +294,11 @@ public:
 	// Cancels select or idle wait
 	void WakeupThread()
 	{
-		scoped_lock l(m_sync);
+		fz::scoped_lock l(m_sync);
 		WakeupThread(l);
 	}
 
-	void WakeupThread(scoped_lock & l)
+	void WakeupThread(fz::scoped_lock & l)
 	{
 		if (!m_started || m_finished) {
 			return;
@@ -320,7 +310,7 @@ public:
 			return;
 		}
 
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 		WSASetEvent(m_sync_event);
 #else
 		char tmp = 0;
@@ -336,7 +326,7 @@ protected:
 	static int CreateSocketFd(addrinfo const& addr)
 	{
 		int fd;
-#if defined(SOCK_CLOEXEC) && !defined(__WXMSW__)
+#if defined(SOCK_CLOEXEC) && !defined(FZ_WINDOWS)
 		fd = socket(addr.ai_family, addr.ai_socktype | SOCK_CLOEXEC, addr.ai_protocol);
 		if (fd == -1 && errno == EINVAL)
 #endif
@@ -359,7 +349,7 @@ protected:
 	static void CloseSocketFd(int& fd)
 	{
 		if (fd != -1) {
-	#ifdef __WXMSW__
+	#ifdef FZ_WINDOWS
 			closesocket(fd);
 	#else
 			close(fd);
@@ -368,16 +358,16 @@ protected:
 		}
 	}
 
-	int TryConnectHost(addrinfo & addr, sockaddr_u const& bindAddr, scoped_lock & l)
+	int TryConnectHost(addrinfo & addr, sockaddr_u const& bindAddr, fz::scoped_lock & l)
 	{
 		if (m_pSocket->m_pEvtHandler) {
-			m_pSocket->m_pEvtHandler->SendEvent<CHostAddressEvent>(m_pSocket, CSocket::AddressToString(addr.ai_addr, addr.ai_addrlen));
+			m_pSocket->m_pEvtHandler->send_event<CHostAddressEvent>(m_pSocket, CSocket::AddressToString(addr.ai_addr, addr.ai_addrlen));
 		}
 
 		int fd = CreateSocketFd(addr);
 		if (fd == -1) {
 			if (m_pSocket->m_pEvtHandler) {
-				m_pSocket->m_pEvtHandler->SendEvent<CSocketEvent>(m_pSocket, addr.ai_next ? SocketEventType::connection_next : SocketEventType::connection, GetLastSocketError());
+				m_pSocket->m_pEvtHandler->send_event<CSocketEvent>(m_pSocket, addr.ai_next ? SocketEventType::connection_next : SocketEventType::connection, GetLastSocketError());
 			}
 
 			return 0;
@@ -392,13 +382,13 @@ protected:
 
 		int res = connect(fd, addr.ai_addr, addr.ai_addrlen);
 		if (res == -1) {
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 			// Map to POSIX error codes
 			int error = WSAGetLastError();
 			if (error == WSAEWOULDBLOCK)
 				res = EINPROGRESS;
 			else
-				res = ConvertMSWErrorCode(WSAGetLastError());
+				res = GetLastSocketError();
 #else
 			res = errno;
 #endif
@@ -428,7 +418,7 @@ protected:
 
 		if (res) {
 			if (m_pSocket->m_pEvtHandler) {
-				m_pSocket->m_pEvtHandler->SendEvent<CSocketEvent>(m_pSocket, addr.ai_next ? SocketEventType::connection_next : SocketEventType::connection, res);
+				m_pSocket->m_pEvtHandler->send_event<CSocketEvent>(m_pSocket, addr.ai_next ? SocketEventType::connection_next : SocketEventType::connection, res);
 			}
 
 			CloseSocketFd(fd);
@@ -439,7 +429,7 @@ protected:
 			m_pSocket->m_state = CSocket::connected;
 
 			if (m_pSocket->m_pEvtHandler) {
-				m_pSocket->m_pEvtHandler->SendEvent<CSocketEvent>(m_pSocket, SocketEventType::connection, 0);
+				m_pSocket->m_pEvtHandler->send_event<CSocketEvent>(m_pSocket, SocketEventType::connection, 0);
 			}
 
 			// We're now interested in all the other nice events
@@ -452,7 +442,7 @@ protected:
 	}
 
 	// Only call while locked
-	bool DoConnect(scoped_lock & l)
+	bool DoConnect(fz::scoped_lock & l)
 	{
 		if (m_host.empty() || m_port.empty()) {
 			m_pSocket->m_state = CSocket::closed;
@@ -470,12 +460,13 @@ protected:
 			// Convert bind address
 			addrinfo bind_hints{};
 			bind_hints.ai_flags = AI_NUMERICHOST | AI_NUMERICSERV | AI_PASSIVE;
+			bind_hints.ai_socktype = SOCK_STREAM;
 			addrinfo *bindAddressList{};
 			int res = getaddrinfo(bind.empty() ? 0 : bind.c_str(), "0", &bind_hints, &bindAddressList);
-			if (!res && bindAddressList && bindAddressList->ai_addr) {
-				memcpy(&bindAddr.storage, bindAddressList->ai_addr, bindAddressList->ai_addrlen);
-			}
 			if (!res && bindAddressList) {
+				if (bindAddressList->ai_addr) {
+					memcpy(&bindAddr.storage, bindAddressList->ai_addr, bindAddressList->ai_addrlen);
+				}
 				freeaddrinfo(bindAddressList);
 			}
 		}
@@ -514,12 +505,12 @@ protected:
 		}
 
 		if (res) {
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 			res = ConvertMSWErrorCode(res);
 #endif
 
 			if (m_pSocket->m_pEvtHandler) {
-				m_pSocket->m_pEvtHandler->SendEvent<CSocketEvent>(m_pSocket, SocketEventType::connection, res);
+				m_pSocket->m_pEvtHandler->send_event<CSocketEvent>(m_pSocket, SocketEventType::connection, res);
 			}
 			m_pSocket->m_state = CSocket::closed;
 
@@ -542,7 +533,7 @@ protected:
 		freeaddrinfo(addressList);
 
 		if (m_pSocket->m_pEvtHandler) {
-			m_pSocket->m_pEvtHandler->SendEvent<CSocketEvent>(m_pSocket, SocketEventType::connection, ECONNABORTED);
+			m_pSocket->m_pEvtHandler->send_event<CSocketEvent>(m_pSocket, SocketEventType::connection, ECONNABORTED);
 		}
 		m_pSocket->m_state = CSocket::closed;
 
@@ -555,12 +546,12 @@ protected:
 	}
 
 	// Call only while locked
-	bool DoWait(int wait, scoped_lock & l)
+	bool DoWait(int wait, fz::scoped_lock & l)
 	{
 		m_waiting |= wait;
 
 		for (;;) {
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 			int wait_events = FD_CLOSE;
 			if (m_waiting & WAIT_CONNECT)
 				wait_events |= FD_CONNECT;
@@ -584,7 +575,7 @@ protected:
 			WSANETWORKEVENTS events;
 			int res = WSAEnumNetworkEvents(m_pSocket->m_fd, m_sync_event, &events);
 			if (res) {
-				res = ConvertMSWErrorCode(WSAGetLastError());
+				res = GetLastSocketError();
 				return false;
 			}
 
@@ -639,11 +630,11 @@ protected:
 			if (m_waiting & (WAIT_WRITE | WAIT_CONNECT))
 				FD_SET(m_pSocket->m_fd, &writefds);
 
-			int max = wxMax(m_pipe[0], m_pSocket->m_fd) + 1;
+			int maxfd = std::max(m_pipe[0], m_pSocket->m_fd) + 1;
 
 			l.unlock();
 
-			int res = select(max, &readfds, &writefds, 0, 0);
+			int res = select(maxfd, &readfds, &writefds, 0, 0);
 
 			l.lock();
 
@@ -661,8 +652,6 @@ protected:
 				continue;
 			if (res == -1) {
 				res = errno;
-				//printf("select failed: %s\n", (const char *)CSocket::GetErrorDescription(res).mb_str());
-				//fflush(stdout);
 
 				if (res == EINTR)
 					continue;
@@ -712,17 +701,15 @@ protected:
 		if (!m_pSocket || !m_pSocket->m_pEvtHandler)
 			return;
 		if (m_triggered & WAIT_READ) {
-			if (m_pSocket->m_synchronous_read_cb)
-				m_pSocket->m_synchronous_read_cb->cb();
-			m_pSocket->m_pEvtHandler->SendEvent<CSocketEvent>(m_pSocket, SocketEventType::read, m_triggered_errors[1]);
+			m_pSocket->m_pEvtHandler->send_event<CSocketEvent>(m_pSocket, SocketEventType::read, m_triggered_errors[1]);
 			m_triggered &= ~WAIT_READ;
 		}
 		if (m_triggered & WAIT_WRITE) {
-			m_pSocket->m_pEvtHandler->SendEvent<CSocketEvent>(m_pSocket, SocketEventType::write, m_triggered_errors[2]);
+			m_pSocket->m_pEvtHandler->send_event<CSocketEvent>(m_pSocket, SocketEventType::write, m_triggered_errors[2]);
 			m_triggered &= ~WAIT_WRITE;
 		}
 		if (m_triggered & WAIT_ACCEPT) {
-			m_pSocket->m_pEvtHandler->SendEvent<CSocketEvent>(m_pSocket, SocketEventType::connection, m_triggered_errors[3]);
+			m_pSocket->m_pEvtHandler->send_event<CSocketEvent>(m_pSocket, SocketEventType::connection, m_triggered_errors[3]);
 			m_triggered &= ~WAIT_ACCEPT;
 		}
 		if (m_triggered & WAIT_CLOSE) {
@@ -736,7 +723,7 @@ protected:
 			return;
 		}
 
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 		// MSDN says this:
 		//   FD_CLOSE being posted after all data is read from a socket.
 		//   An application should check for remaining data upon receipt
@@ -747,18 +734,18 @@ protected:
 			if( !(m_waiting & WAIT_READ) ) {
 				return;
 			}
-			m_pSocket->m_pEvtHandler->SendEvent<CSocketEvent>(m_pSocket, SocketEventType::read, 0);
+			m_pSocket->m_pEvtHandler->send_event<CSocketEvent>(m_pSocket, SocketEventType::read, 0);
 		}
 		else
 #endif
 		{
-			m_pSocket->m_pEvtHandler->SendEvent<CSocketEvent>(m_pSocket, SocketEventType::close, m_triggered_errors[4]);
+			m_pSocket->m_pEvtHandler->send_event<CSocketEvent>(m_pSocket, SocketEventType::close, m_triggered_errors[4]);
 			m_triggered &= ~WAIT_CLOSE;
 		}
 	}
 
 	// Call only while locked
-	bool IdleLoop(scoped_lock & l)
+	bool IdleLoop(fz::scoped_lock & l)
 	{
 		if (m_quit)
 			return false;
@@ -773,13 +760,13 @@ protected:
 		return true;
 	}
 
-	virtual ExitCode Entry()
+	virtual void entry()
 	{
-		scoped_lock l(m_sync);
+		fz::scoped_lock l(m_sync);
 		for (;;) {
 			if (!IdleLoop(l)) {
 				m_finished = true;
-				return 0;
+				return;
 			}
 
 			if (m_pSocket->m_state == CSocket::listening) {
@@ -799,7 +786,7 @@ protected:
 						continue;
 				}
 
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 				m_waiting |= WAIT_CLOSE;
 				int wait_close = WAIT_CLOSE;
 #endif
@@ -812,7 +799,7 @@ protected:
 
 					if (m_triggered & WAIT_CLOSE && m_pSocket) {
 						m_pSocket->m_state = CSocket::closing;
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 						wait_close = 0;
 #endif
 					}
@@ -821,7 +808,7 @@ protected:
 						break;
 
 					SendEvents();
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 					m_waiting |= wait_close;
 #endif
 				}
@@ -829,7 +816,7 @@ protected:
 		}
 
 		m_finished = true;
-		return 0;
+		return;
 	}
 
 	CSocket* m_pSocket{};
@@ -838,7 +825,7 @@ protected:
 	std::string m_port;
 	std::string m_bind;
 
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 	// We wait on this using WSAWaitForMultipleEvents
 	WSAEVENT m_sync_event;
 #else
@@ -846,8 +833,8 @@ protected:
 	int m_pipe[2];
 #endif
 
-	mutex m_sync;
-	condition m_condition;
+	fz::mutex m_sync;
+	fz::condition m_condition;
 
 	bool m_started{};
 	bool m_quit{};
@@ -864,18 +851,12 @@ protected:
 	bool m_threadwait{};
 };
 
-CSocket::CSocket(CEventHandler* pEvtHandler)
+CSocket::CSocket(fz::event_handler* pEvtHandler)
 	: m_pEvtHandler(pEvtHandler)
 {
 #ifdef ERRORCODETEST
 	CErrorCodeTest test;
 #endif
-	m_fd = -1;
-	m_state = none;
-	m_pSocketThread = 0;
-	m_flags = 0;
-
-	m_port = 0;
 	m_family = AF_UNSPEC;
 
 	m_buffer_sizes[0] = -1;
@@ -887,20 +868,21 @@ CSocket::~CSocket()
 	if (m_state != none)
 		Close();
 
-	DetachThread();
+	if (m_pSocketThread) {
+		fz::scoped_lock l(m_pSocketThread->m_sync);
+		DetachThread(l);
+	}
 }
 
-void CSocket::DetachThread()
+void CSocket::DetachThread(fz::scoped_lock & l)
 {
 	if (!m_pSocketThread)
 		return;
 
-	scoped_lock l(m_pSocketThread->m_sync);
 	m_pSocketThread->SetSocket(0, l);
 	if (m_pSocketThread->m_finished) {
 		m_pSocketThread->WakeupThread(l);
 		l.unlock();
-		m_pSocketThread->Wait();
 		delete m_pSocketThread;
 	}
 	else {
@@ -913,7 +895,7 @@ void CSocket::DetachThread()
 			m_pSocketThread->WakeupThread(l);
 			l.unlock();
 
-			scoped_lock wl(waiting_socket_threads_mutex);
+			fz::scoped_lock wl(waiting_socket_threads_mutex);
 			waiting_socket_threads.push_back(m_pSocketThread);
 		}
 	}
@@ -922,7 +904,7 @@ void CSocket::DetachThread()
 	Cleanup(false);
 }
 
-int CSocket::Connect(wxString const& host, unsigned int port, address_family family, wxString const& bind)
+int CSocket::Connect(fz::native_string const& host, unsigned int port, address_family family, std::string const& bind)
 {
 	if (m_state != none)
 		return EISCONN;
@@ -930,35 +912,33 @@ int CSocket::Connect(wxString const& host, unsigned int port, address_family fam
 	if (port < 1 || port > 65535)
 		return EINVAL;
 
+	if (host.empty()) {
+		return EINVAL;
+	}
+
+	int af{};
+
 	switch (family)
 	{
 	case unspec:
-		m_family = AF_UNSPEC;
+		af = AF_UNSPEC;
 		break;
 	case ipv4:
-		m_family = AF_INET;
+		af = AF_INET;
 		break;
 	case ipv6:
-		m_family = AF_INET6;
+		af = AF_INET6;
 		break;
 	default:
 		return EINVAL;
 	}
 
 	if (m_pSocketThread && m_pSocketThread->m_started) {
-		scoped_lock l(m_pSocketThread->m_sync);
+		fz::scoped_lock l(m_pSocketThread->m_sync);
 		if (!m_pSocketThread->m_threadwait) {
-			m_pSocketThread->WakeupThread(l);
-			l.unlock();
-			// Wait a small amount of time
-			wxMilliSleep(100);
-
-			l.lock();
-			if (!m_pSocketThread->m_threadwait) {
-				// Inside a blocking call, e.g. getaddrinfo
-				l.unlock();
-				DetachThread();
-			}
+			// Possibly inside a blocking call, e.g. getaddrinfo.
+			// Detach the thread so that we can continue.
+			DetachThread(l);
 		}
 	}
 	if (!m_pSocketThread) {
@@ -966,6 +946,7 @@ int CSocket::Connect(wxString const& host, unsigned int port, address_family fam
 		m_pSocketThread->SetSocket(this);
 	}
 
+	m_family = af;
 	m_state = connecting;
 
 	m_host = host;
@@ -981,10 +962,10 @@ int CSocket::Connect(wxString const& host, unsigned int port, address_family fam
 	return EINPROGRESS;
 }
 
-void CSocket::SetEventHandler(CEventHandler* pEvtHandler)
+void CSocket::SetEventHandler(fz::event_handler* pEvtHandler)
 {
 	if (m_pSocketThread) {
-		scoped_lock l(m_pSocketThread->m_sync);
+		fz::scoped_lock l(m_pSocketThread->m_sync);
 
 		if (m_pEvtHandler == pEvtHandler) {
 			return;
@@ -995,16 +976,16 @@ void CSocket::SetEventHandler(CEventHandler* pEvtHandler)
 		m_pEvtHandler = pEvtHandler;
 
 		if (pEvtHandler && m_state == connected) {
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 			// If a graceful shutdown is going on in background already,
 			// no further events are recorded. Send out events we're not
 			// waiting for (i.e. they got triggered already) manually.
 
 			if (!(m_pSocketThread->m_waiting & WAIT_WRITE)) {
-				pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::write, 0);
+				pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::write, 0);
 			}
 
-			pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::read, 0);
+			pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::read, 0);
 			if (m_pSocketThread->m_waiting & WAIT_READ) {
 				m_pSocketThread->m_waiting &= ~WAIT_READ;
 				m_pSocketThread->WakeupThread(l);
@@ -1027,12 +1008,12 @@ void CSocket::SetEventHandler(CEventHandler* pEvtHandler)
 	}
 }
 
-#define ERRORDECL(c, desc) { c, _T(#c), desc },
+#define ERRORDECL(c, desc) { c, #c, desc },
 
 struct Error_table
 {
 	int code;
-	const wxChar* const name;
+	const char* const name;
 	const wxChar* const description;
 };
 
@@ -1099,7 +1080,7 @@ static Error_table const error_table[] =
 #endif
 
 	// Codes that have no POSIX equivalence
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 	ERRORDECL(WSANOTINITIALISED, TRANSLATE_T("Not initialized, need to call WSAStartup"))
 	ERRORDECL(WSAENETDOWN, TRANSLATE_T("System's network subsystem has failed"))
 	ERRORDECL(WSAEPROTOTYPE, TRANSLATE_T("Protocol not supported on given socket type"))
@@ -1110,43 +1091,41 @@ static Error_table const error_table[] =
 	{ 0, 0, 0 }
 };
 
-wxString CSocket::GetErrorString(int error)
+std::string CSocket::GetErrorString(int error)
 {
-	for (int i = 0; error_table[i].code; ++i)
-	{
+	for (int i = 0; error_table[i].code; ++i) {
 		if (error != error_table[i].code)
 			continue;
 
 		return error_table[i].name;
 	}
 
-	return wxString::Format(_T("%d"), error);
+	return wxString::Format(_T("%d"), error).ToStdString();
 }
 
-wxString CSocket::GetErrorDescription(int error)
+fz::native_string CSocket::GetErrorDescription(int error)
 {
-	for (int i = 0; error_table[i].code; ++i)
-	{
+	for (int i = 0; error_table[i].code; ++i) {
 		if (error != error_table[i].code)
 			continue;
 
-		return wxString(error_table[i].name) + _T(" - ") + wxGetTranslation(error_table[i].description);
+		return fz::to_native(wxString(error_table[i].name) + _T(" - ") + wxGetTranslation(error_table[i].description));
 	}
 
-	return wxString::Format(_T("%d"), error);
+	return fz::to_native(wxString::Format(_T("%d"), error));
 }
 
 int CSocket::Close()
 {
 	if (m_pSocketThread) {
-		scoped_lock l(m_pSocketThread->m_sync);
+		fz::scoped_lock l(m_pSocketThread->m_sync);
 		int fd = m_fd;
 		m_fd = -1;
 
 		m_pSocketThread->m_host.clear();
 		m_pSocketThread->m_port.clear();
 
-			m_pSocketThread->WakeupThread(l);
+		m_pSocketThread->WakeupThread(l);
 
 		CSocketThread::CloseSocketFd(fd);
 		m_state = none;
@@ -1190,19 +1169,18 @@ CSocket::SocketState CSocket::GetState()
 
 void CSocket::Cleanup(bool force)
 {
-	scoped_lock wl(waiting_socket_threads_mutex);
+	fz::scoped_lock wl(waiting_socket_threads_mutex);
 	auto iter = waiting_socket_threads.begin();
 	for (; iter != waiting_socket_threads.end(); ++iter) {
 		CSocketThread *const pThread = *iter;
 
 		if (!force) {
-			scoped_lock l(pThread->m_sync);
+			fz::scoped_lock l(pThread->m_sync);
 			if (!pThread->m_finished) {
 				break;
 			}
 		}
 
-		pThread->Wait(wxTHREAD_WAIT_BLOCK);
 		delete pThread;
 	}
 	waiting_socket_threads.erase(waiting_socket_threads.begin(), iter);
@@ -1216,7 +1194,7 @@ int CSocket::Read(void* buffer, unsigned int size, int& error)
 		error = GetLastSocketError();
 		if (error == EAGAIN) {
 			if (m_pSocketThread) {
-				scoped_lock l(m_pSocketThread->m_sync);
+				fz::scoped_lock l(m_pSocketThread->m_sync);
 				if (!(m_pSocketThread->m_waiting & WAIT_READ)) {
 					m_pSocketThread->m_waiting |= WAIT_READ;
 					m_pSocketThread->WakeupThread(l);
@@ -1250,10 +1228,10 @@ int CSocket::Write(const void* buffer, unsigned int size, int& error)
 #else
 	const int flags = 0;
 
-#if !defined(SO_NOSIGPIPE) && !defined(__WXMSW__)
+#if !defined(SO_NOSIGPIPE) && !defined(FZ_WINDOWS)
 	// Some systems have neither. Need to block signal
 	struct sigaction old_action;
-	struct sigaction action = {0};
+	struct sigaction action = {};
 	action.sa_handler = SIG_IGN;
 	int signal_set = sigaction(SIGPIPE, &action, &old_action);
 #endif
@@ -1262,7 +1240,7 @@ int CSocket::Write(const void* buffer, unsigned int size, int& error)
 
 	int res = send(m_fd, (const char*)buffer, size, flags);
 
-#if !defined(MSG_NOSIGNAL) && !defined(SO_NOSIGPIPE) && !defined(__WXMSW__)
+#if !defined(MSG_NOSIGNAL) && !defined(SO_NOSIGPIPE) && !defined(FZ_WINDOWS)
 	// Restore previous signal handler
 	if (!signal_set)
 		sigaction(SIGPIPE, &old_action, 0);
@@ -1272,7 +1250,7 @@ int CSocket::Write(const void* buffer, unsigned int size, int& error)
 		error = GetLastSocketError();
 		if (error == EAGAIN) {
 			if (m_pSocketThread) {
-				scoped_lock l (m_pSocketThread->m_sync);
+				fz::scoped_lock l (m_pSocketThread->m_sync);
 				if (!(m_pSocketThread->m_waiting & WAIT_WRITE)) {
 					m_pSocketThread->m_waiting |= WAIT_WRITE;
 					m_pSocketThread->WakeupThread(l);
@@ -1286,40 +1264,41 @@ int CSocket::Write(const void* buffer, unsigned int size, int& error)
 	return res;
 }
 
-wxString CSocket::AddressToString(const struct sockaddr* addr, int addr_len, bool with_port /*=true*/, bool strip_zone_index/*=false*/)
+std::string CSocket::AddressToString(const struct sockaddr* addr, int addr_len, bool with_port, bool strip_zone_index)
 {
 	char hostbuf[NI_MAXHOST];
 	char portbuf[NI_MAXSERV];
 
 	int res = getnameinfo(addr, addr_len, hostbuf, NI_MAXHOST, portbuf, NI_MAXSERV, NI_NUMERICHOST | NI_NUMERICSERV);
 	if (res) // Should never fail
-		return wxString();
+		return std::string();
 
-	wxString host = wxString(hostbuf, wxConvLibc);
-	wxString port = wxString(portbuf, wxConvLibc);
+	std::string host = hostbuf;
+	std::string port = portbuf;
 
 	// IPv6 uses colons as separator, need to enclose address
 	// to avoid ambiguity if also showing port
 	if (addr->sa_family == AF_INET6) {
 		if (strip_zone_index) {
-			int pos = host.Find('%');
-			if (pos != -1)
-				host.Truncate(pos);
+			auto pos = host.find('%');
+			if (pos != std::string::npos) {
+				host = host.substr(0, pos);
+			}
 		}
 		if (with_port)
-			host = _T("[") + host + _T("]");
+			host = "[" + host + "]";
 	}
 
 	if (with_port)
-		return host + _T(":") + port;
+		return host + ":" + port;
 	else
 		return host;
 }
 
-wxString CSocket::AddressToString(char const* buf, int buf_len)
+std::string CSocket::AddressToString(char const* buf, int buf_len)
 {
 	if (buf_len != 4 && buf_len != 16) {
-		return wxString();
+		return std::string();
 	}
 
 	sockaddr_u addr;
@@ -1332,28 +1311,27 @@ wxString CSocket::AddressToString(char const* buf, int buf_len)
 		addr.in4.sin_family = AF_INET;
 	}
 
-
 	return AddressToString(&addr.sockaddr, sizeof(addr), false, true);
 }
 
-wxString CSocket::GetLocalIP(bool strip_zone_index /*=false*/) const
+std::string CSocket::GetLocalIP(bool strip_zone_index /*=false*/) const
 {
 	struct sockaddr_storage addr;
 	socklen_t addr_len = sizeof(addr);
 	int res = getsockname(m_fd, (sockaddr*)&addr, &addr_len);
 	if (res)
-		return wxString();
+		return std::string();
 
 	return AddressToString((sockaddr *)&addr, addr_len, false, strip_zone_index);
 }
 
-wxString CSocket::GetPeerIP(bool strip_zone_index /*=false*/) const
+std::string CSocket::GetPeerIP(bool strip_zone_index /*=false*/) const
 {
 	struct sockaddr_storage addr;
 	socklen_t addr_len = sizeof(addr);
 	int res = getpeername(m_fd, (sockaddr*)&addr, &addr_len);
 	if (res)
-		return wxString();
+		return std::string();
 
 	return AddressToString((sockaddr *)&addr, addr_len, false, strip_zone_index);
 }
@@ -1401,7 +1379,7 @@ int CSocket::Listen(address_family family, int port)
 	}
 
 	{
-		struct addrinfo hints = {0};
+		struct addrinfo hints = {};
 		hints.ai_family = m_family;
 		hints.ai_socktype = SOCK_STREAM;
 		hints.ai_flags = AI_PASSIVE;
@@ -1417,7 +1395,7 @@ int CSocket::Listen(address_family family, int port)
 		int res = getaddrinfo(0, portstring, &hints, &addressList);
 
 		if (res) {
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 			return ConvertMSWErrorCode(res);
 #else
 			return res;
@@ -1469,7 +1447,7 @@ int CSocket::GetLocalPort(int& error)
 	socklen_t addr_len = sizeof(addr);
 	error = getsockname(m_fd, &addr.sockaddr, &addr_len);
 	if (error) {
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 		error = ConvertMSWErrorCode(error);
 #endif
 		return -1;
@@ -1491,7 +1469,7 @@ int CSocket::GetRemotePort(int& error)
 	error = getpeername(m_fd, &addr.sockaddr, &addr_len);
 	if (error)
 	{
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 		error = ConvertMSWErrorCode(error);
 #endif
 		return -1;
@@ -1509,7 +1487,7 @@ int CSocket::GetRemotePort(int& error)
 CSocket* CSocket::Accept(int &error)
 {
 	if (m_pSocketThread) {
-		scoped_lock l(m_pSocketThread->m_sync);
+		fz::scoped_lock l(m_pSocketThread->m_sync);
 		m_pSocketThread->m_waiting |= WAIT_ACCEPT;
 		m_pSocketThread->WakeupThread(l);
 	}
@@ -1543,7 +1521,7 @@ CSocket* CSocket::Accept(int &error)
 int CSocket::SetNonblocking(int fd)
 {
 	// Set socket to non-blocking.
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 	unsigned long nonblock = 1;
 	int res = ioctlsocket(fd, FIONBIO, &nonblock);
 	if (!res)
@@ -1634,18 +1612,7 @@ int CSocket::DoSetBufferSizes(int fd, int size_read, int size_write)
 	return ret;
 }
 
-void CSocket::SetSynchronousReadCallback(CCallback* cb)
-{
-	if (m_pSocketThread)
-		m_pSocketThread->m_sync.lock();
-
-	m_synchronous_read_cb = cb;
-
-	if (m_pSocketThread)
-		m_pSocketThread->m_sync.unlock();
-}
-
-wxString CSocket::GetPeerHost() const
+fz::native_string CSocket::GetPeerHost() const
 {
 	return m_host;
 }
