@@ -79,32 +79,33 @@ static int GetAvailableUpdateMenuId()
 
 std::map<int, std::pair<std::function<void(wxTextEntry*)>, wxChar>> keyboardCommands;
 
+#ifdef __WXMAC__
 wxTextEntry* GetSpecialTextEntry(wxWindow* w, wxChar cmd)
 {
-#ifdef __WXMAC__
-	if( cmd == 'A' || cmd == 'V' ) {
+	if (cmd == 'A' || cmd == 'V') {
 		wxTextCtrl* text = dynamic_cast<wxTextCtrl*>(w);
-		if( text && text->GetWindowStyle() & wxTE_PASSWORD ) {
+		if (text && text->GetWindowStyle() & wxTE_PASSWORD) {
 			return text;
 		}
 	}
-	wxComboBox* combo = dynamic_cast<wxComboBox*>(w);
-	if( combo ) {
-		return combo;
-	}
-#endif
+	return dynamic_cast<wxComboBox*>(w);
+}
+#else
+wxTextEntry* GetSpecialTextEntry(wxWindow*, wxChar)
+{
 	return 0;
 }
+#endif
 
 bool HandleKeyboardCommand(wxCommandEvent& event, wxWindow& parent)
 {
 	auto const& it = keyboardCommands.find(event.GetId());
-	if( it == keyboardCommands.end() ) {
+	if (it == keyboardCommands.end()) {
 		return false;
 	}
 
 	wxTextEntry* e = GetSpecialTextEntry(parent.FindFocus(), it->second.second);
-	if( e ) {
+	if (e) {
 		it->second.first(e);
 	}
 	else {
@@ -197,8 +198,7 @@ public:
 protected:
 	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
 	{
-		if (notification == STATECHANGE_CHANGEDCONTEXT)
-		{
+		if (notification == STATECHANGE_CHANGEDCONTEXT) {
 			// Update window title
 			const CServer* pServer = pState ? pState->GetServer() : 0;
 			if (!pServer)
@@ -216,29 +216,25 @@ protected:
 		if (!controls)
 			return;
 
-		if (controls->tab_index == -1)
-		{
+		if (controls->tab_index == -1) {
 			if (notification == STATECHANGE_REMOTE_IDLE || notification == STATECHANGE_SERVER)
 				pState->Disconnect();
 
 			return;
 		}
 
-		if (notification == STATECHANGE_SERVER)
-		{
+		if (notification == STATECHANGE_SERVER) {
 			const CServer* pServer = pState->GetServer();
 
-			if (pState == CContextManager::Get()->GetCurrentContext())
-			{
-				if (!pServer)
+			if (pState == CContextManager::Get()->GetCurrentContext()) {
+				if (!pServer) {
 					m_pMainFrame->SetTitle(_T("FileZilla"));
-				else
-				{
+				}
+				else {
 					m_pMainFrame->SetTitle(pState->GetTitle() + _T(" - FileZilla"));
-					if (pServer->GetName().empty())
-					{
+					if (pServer->GetName().empty()) {
 						// Can only happen through quickconnect bar
-						CMenuBar* pMenuBar = wxDynamicCast(m_pMainFrame->GetMenuBar(), CMenuBar);
+						CMenuBar* pMenuBar = dynamic_cast<CMenuBar*>(m_pMainFrame->GetMenuBar());
 						if (pMenuBar)
 							pMenuBar->ClearBookmarks();
 					}
@@ -266,7 +262,7 @@ CMainFrame::CMainFrame()
 	initial_size.y = wxMin(950, screen_size.GetHeight() - 50);
 
 	Create(NULL, -1, _T("FileZilla"), wxDefaultPosition, initial_size);
-	SetSizeHints(250, 250);
+	SetSizeHints(350, 300);
 
 #ifdef __WXMSW__
 	// In order for the --close commandline argument to work,
@@ -331,7 +327,7 @@ CMainFrame::CMainFrame()
 	m_pTopSplitter->SetMinimumPaneSize(50);
 
 	m_pBottomSplitter = new CSplitterWindowEx(m_pTopSplitter, -1, wxDefaultPosition, wxDefaultSize, wxSP_NOBORDER | wxSP_LIVE_UPDATE);
-	m_pBottomSplitter->SetMinimumPaneSize(10, 60);
+	m_pBottomSplitter->SetMinimumPaneSize(20, 70);
 	m_pBottomSplitter->SetSashGravity(1.0);
 
 	const int message_log_position = COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION);
@@ -428,10 +424,19 @@ CMainFrame::CMainFrame()
 	CAutoAsciiFiles::SettingsChanged();
 
 	FixTabOrder();
+
+	RegisterOption(OPTION_LANGUAGE);
+	RegisterOption(OPTION_THEME);
+	RegisterOption(OPTION_THEME_ICONSIZE);
+	RegisterOption(OPTION_MESSAGELOG_POSITION);
+	RegisterOption(OPTION_FILEPANE_LAYOUT);
+	RegisterOption(OPTION_FILEPANE_SWAP);
 }
 
 CMainFrame::~CMainFrame()
 {
+	UnregisterAllOptions();
+
 	CPowerManagement::Destroy();
 
 	delete m_pStateEventHandler;
@@ -1056,20 +1061,16 @@ bool CMainFrame::CloseDialogsAndQuit(wxCloseEvent &event)
 
 	int size = wxTopLevelWindows.size();
 	static wxTopLevelWindow* pLast = 0;
-	if (wxTopLevelWindows.size())
-	{
+	if (wxTopLevelWindows.size()) {
 		wxWindowList::reverse_iterator iter = wxTopLevelWindows.rbegin();
 		wxTopLevelWindow* pTop = (wxTopLevelWindow*)(*iter);
-		while (pTop != this && (size != prev_size || pLast != pTop))
-		{
-			wxDialog* pDialog = wxDynamicCast(pTop, wxDialog);
+		while (pTop != this && (size != prev_size || pLast != pTop)) {
+			wxDialog* pDialog = dynamic_cast<wxDialog*>(pTop);
 			if (pDialog)
 				pDialog->EndModal(wxID_CANCEL);
-			else
-			{
+			else {
 				wxWindow* pParent = pTop->GetParent();
-				if (m_pQueuePane && pParent == m_pQueuePane)
-				{
+				if (m_pQueuePane && pParent == m_pQueuePane) {
 					// It's the AUI frame manager hint window. Ignore it
 					++iter;
 					if (iter == wxTopLevelWindows.rend())
@@ -1260,8 +1261,7 @@ void CMainFrame::OnReconnect(wxCommandEvent &)
 
 	CServer server = pState->GetLastServer();
 
-	if (server.GetLogonType() == ASK)
-	{
+	if (server.GetLogonType() == ASK) {
 		if (!CLoginManager::Get().GetPassword(server, false))
 			return;
 	}
@@ -1389,6 +1389,7 @@ void CMainFrame::OnSiteManager(wxCommandEvent& e)
 		return;
 	}
 #endif
+	(void)e;
 	OpenSiteManager();
 }
 
@@ -1412,42 +1413,17 @@ void CMainFrame::OnMenuEditSettings(wxCommandEvent&)
 
 	COptions* pOptions = COptions::Get();
 
-	wxString oldTheme = pOptions->GetOption(OPTION_THEME);
-	wxString oldThemeSize = pOptions->GetOption(OPTION_THEME_ICONSIZE);
 	wxString oldLang = pOptions->GetOption(OPTION_LANGUAGE);
 
 	int oldShowDebugMenu = pOptions->GetOptionVal(OPTION_DEBUG_MENU) != 0;
 
-	bool oldTimestamps = pOptions->GetOptionVal(OPTION_MESSAGELOG_TIMESTAMP) != 0;
-
 	int res = dlg.ShowModal();
-	if (res != wxID_OK)
-	{
-		UpdateLayout();
+	if (res != wxID_OK) {
 		return;
 	}
 
-	bool newTimestamps = pOptions->GetOptionVal(OPTION_MESSAGELOG_TIMESTAMP) != 0;
-
-	wxString newTheme = pOptions->GetOption(OPTION_THEME);
-	wxString newThemeSize = pOptions->GetOption(OPTION_THEME_ICONSIZE);
 	wxString newLang = pOptions->GetOption(OPTION_LANGUAGE);
 
-	if (oldTheme != newTheme ||
-		oldThemeSize != newThemeSize ||
-		oldLang != newLang)
-	{
-		CreateMainToolBar();
-		if (m_pToolBar)
-			m_pToolBar->UpdateToolbarState();
-	}
-
-	if (oldLang != newLang ||
-		oldTimestamps != newTimestamps)
-	{
-		m_pStatusView->InitDefAttr();
-	}
-
 	if (oldLang != newLang ||
 		oldShowDebugMenu != pOptions->GetOptionVal(OPTION_DEBUG_MENU))
 	{
@@ -1467,42 +1443,34 @@ void CMainFrame::OnToggleLogView(wxCommandEvent&)
 
 	bool shown;
 
-	if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) == 1)
-	{
+	if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) == 1) {
 		if (!m_pQueueLogSplitter)
 			return;
-		if (m_pQueueLogSplitter->IsSplit())
-		{
+		if (m_pQueueLogSplitter->IsSplit()) {
 			m_pQueueLogSplitter->Unsplit(m_pStatusView);
 			shown = false;
 		}
-		else if (m_pStatusView->IsShown())
-		{
+		else if (m_pStatusView->IsShown()) {
 			m_pStatusView->Hide();
 			m_pBottomSplitter->Unsplit(m_pQueueLogSplitter);
 			shown = false;
 		}
-		else if (!m_pQueueLogSplitter->IsShown())
-		{
+		else if (!m_pQueueLogSplitter->IsShown()) {
 			m_pQueueLogSplitter->Initialize(m_pStatusView);
 			m_pBottomSplitter->SplitHorizontally(m_pContextControl, m_pQueueLogSplitter);
 			shown = true;
 		}
-		else
-		{
+		else {
 			m_pQueueLogSplitter->SplitVertically(m_pQueuePane, m_pStatusView);
 			shown = true;
 		}
 	}
-	else
-	{
-		if (m_pTopSplitter->IsSplit())
-		{
+	else {
+		if (m_pTopSplitter->IsSplit()) {
 			m_pTopSplitter->Unsplit(m_pStatusView);
 			shown = false;
 		}
-		else
-		{
+		else {
 			m_pTopSplitter->SplitHorizontally(m_pStatusView, m_pBottomSplitter);
 			shown = true;
 		}
@@ -1562,7 +1530,7 @@ void CMainFrame::ShowDirectoryTree(bool local, bool show)
 	COptions::Get()->SetOption(local ? OPTION_SHOW_TREE_LOCAL : OPTION_SHOW_TREE_REMOTE, show);
 }
 
-void CMainFrame::OnToggleQueueView(wxCommandEvent& event)
+void CMainFrame::OnToggleQueueView(wxCommandEvent&)
 {
 	if (!m_pBottomSplitter)
 		return;
@@ -1639,19 +1607,19 @@ void CMainFrame::OnFilter(wxCommandEvent& event)
 }
 
 #if FZ_MANUALUPDATECHECK
-void CMainFrame::OnCheckForUpdates(wxCommandEvent& event)
+void CMainFrame::OnCheckForUpdates(wxCommandEvent&)
 {
-	if( !m_pUpdater ) {
+	if (!m_pUpdater) {
 		return;
 	}
 
 	update_dialog_timer_.Stop();
-	CUpdateDialog dlg( this, *m_pUpdater );
+	CUpdateDialog dlg(this, *m_pUpdater);
 	dlg.ShowModal();
 	update_dialog_timer_.Stop();
 }
 
-void CMainFrame::UpdaterStateChanged( UpdaterState s, build const& v )
+void CMainFrame::UpdaterStateChanged(UpdaterState s, build const& v)
 {
 #if FZ_AUTOUPDATECHECK
 	if( !m_pMenuBar ) {
@@ -1716,15 +1684,12 @@ void CMainFrame::TriggerUpdateDialog()
 }
 #endif
 
-void CMainFrame::UpdateLayout(int layout /*=-1*/, int swap /*=-1*/, int messagelog_position /*=-1*/)
+void CMainFrame::UpdateLayout()
 {
-	if (layout == -1)
-		layout = COptions::Get()->GetOptionVal(OPTION_FILEPANE_LAYOUT);
-	if (swap == -1)
-		swap = COptions::Get()->GetOptionVal(OPTION_FILEPANE_SWAP);
+	int const layout = COptions::Get()->GetOptionVal(OPTION_FILEPANE_LAYOUT);
+	int const swap = COptions::Get()->GetOptionVal(OPTION_FILEPANE_SWAP);
 
-	if (messagelog_position == -1)
-		messagelog_position = COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION);
+	int const messagelog_position = COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION);
 
 	// First handle changes in message log position as it can make size of the other panes change
 	{
@@ -1732,16 +1697,13 @@ void CMainFrame::UpdateLayout(int layout /*=-1*/, int swap /*=-1*/, int messagel
 		wxWindow* parent = m_pStatusView->GetParent();
 
 		bool changed;
-		if (parent == m_pTopSplitter && messagelog_position != 0)
-		{
+		if (parent == m_pTopSplitter && messagelog_position != 0) {
 			if (shown)
 				m_pTopSplitter->Unsplit(m_pStatusView);
 			changed = true;
 		}
-		else if (parent == m_pQueueLogSplitter && messagelog_position != 1)
-		{
-			if (shown)
-			{
+		else if (parent == m_pQueueLogSplitter && messagelog_position != 1) {
+			if (shown) {
 				if (m_pQueueLogSplitter->IsSplit())
 					m_pQueueLogSplitter->Unsplit(m_pStatusView);
 				else
@@ -1749,8 +1711,7 @@ void CMainFrame::UpdateLayout(int layout /*=-1*/, int swap /*=-1*/, int messagel
 			}
 			changed = true;
 		}
-		else if (parent != m_pTopSplitter && parent != m_pQueueLogSplitter && messagelog_position != 2)
-		{
+		else if (parent != m_pTopSplitter && parent != m_pQueueLogSplitter && messagelog_position != 2) {
 			m_pQueuePane->RemovePage(3);
 			changed = true;
 			shown = true;
@@ -1758,10 +1719,8 @@ void CMainFrame::UpdateLayout(int layout /*=-1*/, int swap /*=-1*/, int messagel
 		else
 			changed = false;
 
-		if (changed)
-		{
-			switch (messagelog_position)
-			{
+		if (changed) {
+			switch (messagelog_position) {
 			default:
 				m_pStatusView->Reparent(m_pTopSplitter);
 				if (shown)
@@ -1955,8 +1914,6 @@ bool CMainFrame::ConnectToSite(CSiteManagerItemData_Site & data, bool newTab)
 
 void CMainFrame::CheckChangedSettings()
 {
-	UpdateLayout();
-
 	m_pAsyncRequestQueue->RecheckDefaults();
 
 	CAutoAsciiFiles::SettingsChanged();
@@ -2176,7 +2133,7 @@ void CMainFrame::OnActivate(wxActivateEvent& event)
 		m_pAsyncRequestQueue->TriggerProcessing();
 }
 
-void CMainFrame::OnToolbarComparison(wxCommandEvent& event)
+void CMainFrame::OnToolbarComparison(wxCommandEvent&)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
 	if (!pState)
@@ -2285,7 +2242,7 @@ void CMainFrame::OnDropdownComparisonMode(wxCommandEvent& event)
 		pComparisonManager->CompareListings();
 }
 
-void CMainFrame::OnDropdownComparisonHide(wxCommandEvent& event)
+void CMainFrame::OnDropdownComparisonHide(wxCommandEvent&)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
 	if (!pState)
@@ -2377,7 +2334,7 @@ void CMainFrame::ProcessCommandLine()
 	}
 }
 
-void CMainFrame::OnFilterRightclicked(wxCommandEvent& event)
+void CMainFrame::OnFilterRightclicked(wxCommandEvent&)
 {
 	const bool active = CFilterManager::HasActiveFilters();
 
@@ -2435,7 +2392,7 @@ WXLRESULT CMainFrame::MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lPara
 }
 #endif
 
-void CMainFrame::OnSyncBrowse(wxCommandEvent& event)
+void CMainFrame::OnSyncBrowse(wxCommandEvent&)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
 	if (!pState)
@@ -2509,7 +2466,7 @@ void CMainFrame::OnTaskBarClick(wxTaskBarIconEvent&)
 }
 
 #ifdef __WXGTK__
-void CMainFrame::OnTaskBarClick_Delayed(wxCommandEvent& event)
+void CMainFrame::OnTaskBarClick_Delayed(wxCommandEvent&)
 {
 	m_taskbar_is_uniconizing = false;
 }
@@ -2517,12 +2474,17 @@ void CMainFrame::OnTaskBarClick_Delayed(wxCommandEvent& event)
 
 #endif
 
-void CMainFrame::OnSearch(wxCommandEvent& event)
+void CMainFrame::OnSearch(wxCommandEvent&)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
 	if (!pState)
 		return;
 
+	if (!pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
+		wxBell();
+		return;
+	}
+
 	CSearchDialog dlg(this, pState, m_pQueueView);
 	if (!dlg.Load())
 		return;
@@ -2553,7 +2515,7 @@ void CMainFrame::PostInitialize()
 	}
 }
 
-void CMainFrame::OnMenuNewTab(wxCommandEvent& event)
+void CMainFrame::OnMenuNewTab(wxCommandEvent&)
 {
 	if (m_pContextControl)
 		m_pContextControl->CreateTab();
@@ -2697,3 +2659,18 @@ void CMainFrame::SetupKeyboardAccelerators()
 	wxAcceleratorTable accel(entries.size(), &entries[0]);
 	SetAcceleratorTable(accel);
 }
+
+void CMainFrame::OnOptionsChanged(changed_options_t const& options)
+{
+	if (options.test(OPTION_FILEPANE_LAYOUT) || options.test(OPTION_FILEPANE_SWAP) || options.test(OPTION_MESSAGELOG_POSITION)) {
+		UpdateLayout();
+	}
+
+	bool const language_changed = options.test(OPTION_LANGUAGE);
+	if (options.test(OPTION_THEME) || options.test(OPTION_THEME_ICONSIZE) || language_changed) {
+		CreateMainToolBar();
+		if (m_pToolBar) {
+			m_pToolBar->UpdateToolbarState();
+		}
+	}
+}
