@@ -186,11 +186,12 @@ EVT_MENU(XRCID("ID_PROCESSQUEUE"), CQueueView::OnProcessQueue)
 EVT_MENU(XRCID("ID_REMOVEALL"), CQueueView::OnStopAndClear)
 EVT_MENU(XRCID("ID_REMOVE"), CQueueView::OnRemoveSelected)
 EVT_MENU(XRCID("ID_DEFAULT_FILEEXISTSACTION"), CQueueView::OnSetDefaultFileExistsAction)
-EVT_MENU(XRCID("ID_ACTIONAFTER_DISABLE"), CQueueView::OnActionAfter)
+EVT_MENU(XRCID("ID_ACTIONAFTER_NONE"), CQueueView::OnActionAfter)
+EVT_MENU(XRCID("ID_ACTIONAFTER_SHOW_NOTIFICATION_BUBBLE"), CQueueView::OnActionAfter)
+EVT_MENU(XRCID("ID_ACTIONAFTER_REQUEST_ATTENTION"), CQueueView::OnActionAfter)
 EVT_MENU(XRCID("ID_ACTIONAFTER_CLOSE"), CQueueView::OnActionAfter)
 EVT_MENU(XRCID("ID_ACTIONAFTER_DISCONNECT"), CQueueView::OnActionAfter)
 EVT_MENU(XRCID("ID_ACTIONAFTER_RUNCOMMAND"), CQueueView::OnActionAfter)
-EVT_MENU(XRCID("ID_ACTIONAFTER_SHOWMESSAGE"), CQueueView::OnActionAfter)
 EVT_MENU(XRCID("ID_ACTIONAFTER_PLAYSOUND"), CQueueView::OnActionAfter)
 EVT_MENU(XRCID("ID_ACTIONAFTER_REBOOT"), CQueueView::OnActionAfter)
 EVT_MENU(XRCID("ID_ACTIONAFTER_SHUTDOWN"), CQueueView::OnActionAfter)
@@ -438,24 +439,20 @@ protected:
 
 CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsyncRequestQueue *pAsyncRequestQueue)
 	: CQueueViewBase(parent, index, _("Queued files")),
-	  m_pMainFrame(pMainFrame),
-	  m_pAsyncRequestQueue(pAsyncRequestQueue)
+	m_pMainFrame(pMainFrame),
+	m_pAsyncRequestQueue(pAsyncRequestQueue)
 {
 	if (m_pAsyncRequestQueue)
 		m_pAsyncRequestQueue->SetQueue(this);
 
-	m_activeCount = 0;
-	m_activeCountDown = 0;
-	m_activeCountUp = 0;
-	m_activeMode = 0;
-	m_quit = 0;
-	m_waitStatusLineUpdate = false;
-	m_lastTopItem = -1;
-	
-	m_totalQueueSize = 0;
-	m_filesWithUnknownSize = 0;
-
-	m_actionAfterState = ActionAfterState_Disabled;
+	int action = COptions::Get()->GetOptionVal(OPTION_QUEUE_COMPLETION_ACTION);
+	if (action < 0 || action >= ActionAfterState::Count) {
+		action = 1;
+	}
+	else if (action == ActionAfterState::Reboot || action == ActionAfterState::Shutdown || action == ActionAfterState::Sleep) {
+		action = 1;
+	}
+	m_actionAfterState = static_cast<ActionAfterState::type>(action);
 
 	std::list<ColumnId> extraCols;
 	extraCols.push_back(colTransferStatus);
@@ -490,7 +487,7 @@ CQueueView::~CQueueView()
 bool CQueueView::QueueFile(const bool queueOnly, const bool download,
 						   const wxString& sourceFile, const wxString& targetFile,
 						   const CLocalPath& localPath, const CServerPath& remotePath,
-						   const CServer& server, int64_t size, enum CEditHandler::fileType edit,
+						   const CServer& server, int64_t size, CEditHandler::fileType edit,
 						   QueuePriority priority)
 {
 	CServerItem* pServerItem = CreateServerItem(server);
@@ -934,7 +931,7 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification
 	int replyCode = notification.nReplyCode;
 
 	if ((replyCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED) {
-		enum ResetReason reason;
+		ResetReason reason;
 		if (pEngineData->pItem) {
 			if (pEngineData->pItem->pending_remove())
 				reason = remove;
@@ -1100,7 +1097,7 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification
 	}
 
 	if (!m_activeMode) {
-		enum ResetReason reason;
+		ResetReason reason;
 		if (pEngineData->pItem && pEngineData->pItem->pending_remove())
 			reason = remove;
 		else
@@ -1112,7 +1109,7 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification
 	SendNextCommand(*pEngineData);
 }
 
-void CQueueView::ResetEngine(t_EngineData& data, const enum ResetReason reason)
+void CQueueView::ResetEngine(t_EngineData& data, const ResetReason reason)
 {
 	if (!data.active)
 		return;
@@ -1737,7 +1734,7 @@ void CQueueView::OnFolderThreadComplete(wxCommandEvent&)
 	ProcessUploadFolderItems();
 }
 
-int CQueueView::QueueFiles(const std::list<CFolderProcessingEntry*> &entryList, bool queueOnly, bool download, CServerItem* pServerItem, const enum CFileExistsNotification::OverwriteAction defaultFileExistsAction)
+int CQueueView::QueueFiles(const std::list<CFolderProcessingEntry*> &entryList, bool queueOnly, bool download, CServerItem* pServerItem, const CFileExistsNotification::OverwriteAction defaultFileExistsAction)
 {
 	wxASSERT(pServerItem);
 
@@ -2031,16 +2028,16 @@ void CQueueView::OnContextMenu(wxContextMenuEvent&)
 	bool has_selection = HasSelection();
 
 	pMenu->Check(XRCID("ID_PROCESSQUEUE"), IsActive() ? true : false);
-	pMenu->Check(XRCID("ID_ACTIONAFTER_DISABLE"), IsActionAfter(ActionAfterState_Disabled));
-	pMenu->Check(XRCID("ID_ACTIONAFTER_CLOSE"), IsActionAfter(ActionAfterState_Close));
-	pMenu->Check(XRCID("ID_ACTIONAFTER_DISCONNECT"), IsActionAfter(ActionAfterState_Disconnect));
-	pMenu->Check(XRCID("ID_ACTIONAFTER_RUNCOMMAND"), IsActionAfter(ActionAfterState_RunCommand));
-	pMenu->Check(XRCID("ID_ACTIONAFTER_SHOWMESSAGE"), IsActionAfter(ActionAfterState_ShowMessage));
-	pMenu->Check(XRCID("ID_ACTIONAFTER_PLAYSOUND"), IsActionAfter(ActionAfterState_PlaySound));
+	pMenu->Check(XRCID("ID_ACTIONAFTER_NONE"), IsActionAfter(ActionAfterState::None));
+	pMenu->Check(XRCID("ID_ACTIONAFTER_SHOW_NOTIFICATION_BUBBLE"), IsActionAfter(ActionAfterState::ShowNotification));
+	pMenu->Check(XRCID("ID_ACTIONAFTER_REQUEST_ATTENTION"), IsActionAfter(ActionAfterState::RequestAttention));
+	pMenu->Check(XRCID("ID_ACTIONAFTER_CLOSE"), IsActionAfter(ActionAfterState::Close));
+	pMenu->Check(XRCID("ID_ACTIONAFTER_RUNCOMMAND"), IsActionAfter(ActionAfterState::RunCommand));
+	pMenu->Check(XRCID("ID_ACTIONAFTER_PLAYSOUND"), IsActionAfter(ActionAfterState::PlaySound));
 #if defined(__WXMSW__) || defined(__WXMAC__)
-	pMenu->Check(XRCID("ID_ACTIONAFTER_REBOOT"), IsActionAfter(ActionAfterState_Reboot));
-	pMenu->Check(XRCID("ID_ACTIONAFTER_SHUTDOWN"), IsActionAfter(ActionAfterState_Shutdown));
-	pMenu->Check(XRCID("ID_ACTIONAFTER_SLEEP"), IsActionAfter(ActionAfterState_Sleep));
+	pMenu->Check(XRCID("ID_ACTIONAFTER_REBOOT"), IsActionAfter(ActionAfterState::Reboot));
+	pMenu->Check(XRCID("ID_ACTIONAFTER_SHUTDOWN"), IsActionAfter(ActionAfterState::Shutdown));
+	pMenu->Check(XRCID("ID_ACTIONAFTER_SLEEP"), IsActionAfter(ActionAfterState::Sleep));
 #endif
 	pMenu->Enable(XRCID("ID_REMOVE"), has_selection);
 
@@ -2067,14 +2064,11 @@ void CQueueView::OnStopAndClear(wxCommandEvent&)
 
 void CQueueView::OnActionAfter(wxCommandEvent& event)
 {
-	if (!event.IsChecked() || event.GetId() == XRCID("ID_ACTIONAFTER_DISABLE"))
-	{ // Goes from checked to non-checked or disable is pressed
-		m_actionAfterState = ActionAfterState_Disabled;
-		m_actionAfterRunCommand = _T("");
+	if (event.GetId() == XRCID("ID_ACTIONAFTER_NONE")) {
+		m_actionAfterState = ActionAfterState::None;
 
 #if defined(__WXMSW__) || defined(__WXMAC__)
-		if (m_actionAfterWarnDialog)
-		{
+		if (m_actionAfterWarnDialog) {
 			m_actionAfterWarnDialog->Destroy();
 			m_actionAfterWarnDialog = 0;
 		}
@@ -2082,51 +2076,51 @@ void CQueueView::OnActionAfter(wxCommandEvent& event)
 		m_actionAfterTimer = 0;
 #endif
 	}
-	else if (event.GetId() == XRCID("ID_ACTIONAFTER_DISABLE"))
-		m_actionAfterState = ActionAfterState_Disabled;
-
-	else if (event.GetId() == XRCID("ID_ACTIONAFTER_CLOSE"))
-		m_actionAfterState = ActionAfterState_Close;
-
-	else if (event.GetId() == XRCID("ID_ACTIONAFTER_DISCONNECT"))
-		m_actionAfterState = ActionAfterState_Disconnect;
-
-	else if (event.GetId() == XRCID("ID_ACTIONAFTER_SHOWMESSAGE"))
-		m_actionAfterState = ActionAfterState_ShowMessage;
-
-	else if (event.GetId() == XRCID("ID_ACTIONAFTER_PLAYSOUND"))
-		m_actionAfterState = ActionAfterState_PlaySound;
-
-	else if (event.GetId() == XRCID("ID_ACTIONAFTER_RUNCOMMAND"))
-	{
-		m_actionAfterState = ActionAfterState_RunCommand;
-		wxTextEntryDialog dlg(m_pMainFrame, _("Please enter a path and executable to run.\nE.g. c:\\somePath\\file.exe under MS Windows or /somePath/file under Unix.\nYou can also optionally specify program arguments."), _("Enter command"));
-
-		if (dlg.ShowModal() != wxID_OK)
-		{
-			m_actionAfterState = ActionAfterState_Disabled;
-			return;
-		}
-		const wxString &command = dlg.GetValue();
-
-		if (command.empty())
-		{
-			wxMessageBoxEx(_("No command given, aborting."), _("Empty command"), wxICON_ERROR, m_pMainFrame);
-			m_actionAfterState = ActionAfterState_Disabled;
-			return;
-		}
-		m_actionAfterRunCommand = command;
+	else if (event.GetId() == XRCID("ID_ACTIONAFTER_NONE")) {
+		m_actionAfterState = ActionAfterState::None;
 	}
+	else if (event.GetId() == XRCID("ID_ACTIONAFTER_SHOW_NOTIFICATION_BUBBLE")) {
+		m_actionAfterState = ActionAfterState::ShowNotification;
+	}
+	else if (event.GetId() == XRCID("ID_ACTIONAFTER_REQUEST_ATTENTION")) {
+		m_actionAfterState = ActionAfterState::RequestAttention;
+	}
+	else if (event.GetId() == XRCID("ID_ACTIONAFTER_CLOSE")) {
+		m_actionAfterState = ActionAfterState::Close;
+	}
+	else if (event.GetId() == XRCID("ID_ACTIONAFTER_CLOSE")) {
+		m_actionAfterState = ActionAfterState::Close;
+	}
+	else if (event.GetId() == XRCID("ID_ACTIONAFTER_PLAYSOUND")) {
+		m_actionAfterState = ActionAfterState::PlaySound;
+	}
+	else if (event.GetId() == XRCID("ID_ACTIONAFTER_RUNCOMMAND")) {
+		wxTextEntryDialog dlg(m_pMainFrame, _("Please enter the complete path of a program and its arguments. This command will be executed when the queue has finished processing.\nE.g. c:\\somePath\\file.exe under MS Windows or /somePath/file under Unix.\nYou need to properly quote commands and their arguments if they contain spaces."), _("Enter command"));
+		dlg.SetValue(COptions::Get()->GetOption(OPTION_QUEUE_COMPLETION_COMMAND));
 
+		if (dlg.ShowModal() == wxID_OK) {
+			const wxString &command = dlg.GetValue();
+			if (command.empty()) {
+				wxMessageBoxEx(_("No command given, aborting."), _("Empty command"), wxICON_ERROR, m_pMainFrame);
+			}
+			else {
+				m_actionAfterState = ActionAfterState::RunCommand;
+				COptions::Get()->SetOption(OPTION_QUEUE_COMPLETION_COMMAND, command);
+			}
+		}
+	}
 #if defined(__WXMSW__) || defined(__WXMAC__)
 	else if (event.GetId() == XRCID("ID_ACTIONAFTER_REBOOT"))
-		m_actionAfterState = ActionAfterState_Reboot;
+		m_actionAfterState = ActionAfterState::Reboot;
 	else if (event.GetId() == XRCID("ID_ACTIONAFTER_SHUTDOWN"))
-		m_actionAfterState = ActionAfterState_Shutdown;
+		m_actionAfterState = ActionAfterState::Shutdown;
 	else if (event.GetId() == XRCID("ID_ACTIONAFTER_SLEEP"))
-		m_actionAfterState = ActionAfterState_Sleep;
+		m_actionAfterState = ActionAfterState::Sleep;
 #endif
 
+	if (m_actionAfterState != ActionAfterState::Reboot && m_actionAfterState != ActionAfterState::Shutdown && m_actionAfterState != ActionAfterState::Sleep) {
+		COptions::Get()->SetOption(OPTION_QUEUE_COMPLETION_ACTION, m_actionAfterState);
+	}
 }
 
 void CQueueView::RemoveAll()
@@ -2147,20 +2141,17 @@ void CQueueView::RemoveAll()
 
 	std::vector<CServerItem*> newServerList;
 	m_itemCount = 0;
-	for (auto iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
-	{
+	for (auto iter = m_serverList.begin(); iter != m_serverList.end(); ++iter) {
 		if ((*iter)->TryRemoveAll())
 			delete *iter;
-		else
-		{
+		else {
 			newServerList.push_back(*iter);
 			m_itemCount += 1 + (*iter)->GetChildrenCount(true);
 		}
 	}
 
 	// Clear list of queued directories that aren't busy
-	for (unsigned int i = 0; i < 2; i++)
-	{
+	for (unsigned int i = 0; i < 2; ++i) {
 		auto begin = m_queuedFolders[i].begin();
 		auto end = m_queuedFolders[i].end();
 		if (begin != end && (*begin)->m_active)
@@ -2169,7 +2160,10 @@ void CQueueView::RemoveAll()
 	}
 
 	SaveSetItemCount(m_itemCount);
-	m_actionAfterState = ActionAfterState_Disabled;
+
+	if (newServerList.empty() && (m_actionAfterState == ActionAfterState::Reboot || m_actionAfterState == ActionAfterState::Shutdown || m_actionAfterState == ActionAfterState::Sleep)) {
+		m_actionAfterState = ActionAfterState::None;
+	}
 
 	m_serverList = newServerList;
 	UpdateStatusLinePositions();
@@ -2182,10 +2176,8 @@ void CQueueView::RemoveAll()
 
 void CQueueView::RemoveQueuedFolderItem(CFolderScanItem* pFolder)
 {
-	for (unsigned int i = 0; i < 2; i++)
-	{
-		for (auto iter = m_queuedFolders[i].begin(); iter != m_queuedFolders[i].end(); ++iter)
-		{
+	for (unsigned int i = 0; i < 2; ++i) {
+		for (auto iter = m_queuedFolders[i].begin(); iter != m_queuedFolders[i].end(); ++iter) {
 			if (*iter != pFolder)
 				continue;
 
@@ -2299,7 +2291,7 @@ bool CQueueView::StopItem(CFileItem* item)
 
 	if (item->m_pEngineData->state == t_EngineData::waitprimary)
 	{
-		enum ResetReason reason;
+		ResetReason reason;
 		if (item->m_pEngineData->pItem && item->m_pEngineData->pItem->pending_remove())
 			reason = remove;
 		else
@@ -2376,7 +2368,7 @@ void CQueueView::OnFolderThreadFiles(wxCommandEvent&)
 		m_folderscan_item_refresh_timer.Start(200, true);
 }
 
-void CQueueView::SetDefaultFileExistsAction(enum CFileExistsNotification::OverwriteAction action, const TransferDirection direction)
+void CQueueView::SetDefaultFileExistsAction(CFileExistsNotification::OverwriteAction action, const TransferDirection direction)
 {
 	for (auto iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
 		(*iter)->SetDefaultFileExistsAction(action, direction);
@@ -2392,8 +2384,8 @@ void CQueueView::OnSetDefaultFileExistsAction(wxCommandEvent &)
 		return;
 
 	// Get current default action for the item
-	enum CFileExistsNotification::OverwriteAction downloadAction = CFileExistsNotification::unknown;
-	enum CFileExistsNotification::OverwriteAction uploadAction = CFileExistsNotification::unknown;
+	CFileExistsNotification::OverwriteAction downloadAction = CFileExistsNotification::unknown;
+	CFileExistsNotification::OverwriteAction uploadAction = CFileExistsNotification::unknown;
 	bool has_upload = false;
 	bool has_download = false;
 	bool download_unknown = false;
@@ -2897,12 +2889,7 @@ void CQueueView::OnExclusiveEngineRequestGranted(wxCommandEvent& event)
 	SendNextCommand(*pEngineData);
 }
 
-enum ActionAfterState CQueueView::GetActionAfterState() const
-{
-	return m_actionAfterState;
-}
-
-bool CQueueView::IsActionAfter(enum ActionAfterState state)
+bool CQueueView::IsActionAfter(ActionAfterState::type state)
 {
 	return m_actionAfterState == state;
 }
@@ -2925,99 +2912,97 @@ void CQueueView::ActionAfter(bool warned /*=false*/)
 		}
 	}
 
-	if (!m_pMainFrame->IsActive()) {
-		wxString const title = _("Transfers finished");
-		wxString msg;
-		int const failed_count = m_pQueue->GetQueueView_Failed()->GetFileCount();
-		if (failed_count != 0) {
-			wxString fmt = wxPLURAL("All transfers have finished. %d file could not be transferred.", "All transfers have finished. %d files could not be transferred.", failed_count);
-			msg = wxString::Format(fmt, failed_count);
-		}
-		else
-			msg = _("All files have been successfully transferred");
-
-#if WITH_LIBDBUS
-		if (!m_desktop_notification)
-			m_desktop_notification = std::make_unique<CDesktopNotification>();
-		m_desktop_notification->Notify(title, msg, (failed_count > 0) ? _T("transfer.error") : _T("transfer.complete"));
-#elif defined(__WXGTK__) || defined(__WXMSW__)
-		m_desktop_notification = std::make_unique<wxNotificationMessage>();
-		m_desktop_notification->SetTitle(title);
-		m_desktop_notification->SetMessage(msg);
-		m_desktop_notification->Show(5);
-#endif
-}
-
 	switch (m_actionAfterState)
 	{
-		case ActionAfterState_Close:
+		case ActionAfterState::ShowNotification:
+		{
+			wxString const title = _("Transfers finished");
+			wxString msg;
+			int const failed_count = m_pQueue->GetQueueView_Failed()->GetFileCount();
+			if (failed_count != 0) {
+				wxString fmt = wxPLURAL("All transfers have finished. %d file could not be transferred.", "All transfers have finished. %d files could not be transferred.", failed_count);
+				msg = wxString::Format(fmt, failed_count);
+			}
+			else
+				msg = _("All files have been successfully transferred");
+
+#if WITH_LIBDBUS
+			if (!m_desktop_notification)
+				m_desktop_notification = std::make_unique<CDesktopNotification>();
+			m_desktop_notification->Notify(title, msg, (failed_count > 0) ? _T("transfer.error") : _T("transfer.complete"));
+#elif defined(__WXGTK__) || defined(__WXMSW__)
+			m_desktop_notification = std::make_unique<wxNotificationMessage>();
+			m_desktop_notification->SetTitle(title);
+			m_desktop_notification->SetMessage(msg);
+			m_desktop_notification->Show(5);
+#endif
+			break;
+		}
+		case ActionAfterState::RequestAttention:
+		{
+			int const failed_count = m_pQueue->GetQueueView_Failed()->GetFileCount();
+			m_pMainFrame->RequestUserAttention(failed_count ? wxUSER_ATTENTION_ERROR : wxUSER_ATTENTION_INFO);
+			break;
+		}
+		case ActionAfterState::Close:
 		{
 			m_pMainFrame->Close();
 			break;
 		}
-		case ActionAfterState_Disconnect:
+		case ActionAfterState::RunCommand:
 		{
-			for (auto & state : *pStates) {
-				if (state->IsRemoteConnected() && state->IsRemoteIdle()) {
-					state->Disconnect();
-				}
+			wxString cmd = COptions::Get()->GetOption(OPTION_QUEUE_COMPLETION_COMMAND);
+			if (!cmd.empty()) {
+				wxExecute(cmd);
 			}
 			break;
 		}
-		case ActionAfterState_RunCommand:
-		{
-			wxExecute(m_actionAfterRunCommand);
-			break;
-		}
-		case ActionAfterState_ShowMessage:
-		{
-			wxMessageDialog* dialog = new wxMessageDialog(m_pMainFrame, _("No more files in the queue!"), _T("Queue completion"), wxOK | wxICON_INFORMATION);
-			m_pMainFrame->RequestUserAttention(wxUSER_ATTENTION_ERROR);
-			dialog->ShowModal();
-			dialog->Destroy();
-			break;
-		}
-		case ActionAfterState_PlaySound:
+		case ActionAfterState::PlaySound:
 		{
 			wxSound sound(wxGetApp().GetResourceDir().GetPath() + _T("finished.wav"));
 			sound.Play(wxSOUND_ASYNC);
 			break;
 		}
 #ifdef __WXMSW__
-		case ActionAfterState_Reboot:
-		case ActionAfterState_Shutdown:
+		case ActionAfterState::Reboot:
+		case ActionAfterState::Shutdown:
 			if (!warned) {
 				ActionAfterWarnUser(m_actionAfterState);
 				return;
 			}
-			else
-				wxShutdown((m_actionAfterState == ActionAfterState_Reboot) ? wxSHUTDOWN_REBOOT : wxSHUTDOWN_POWEROFF);
+			else {
+				wxShutdown((m_actionAfterState == ActionAfterState::Reboot) ? wxSHUTDOWN_REBOOT : wxSHUTDOWN_POWEROFF);
+				m_actionAfterState = ActionAfterState::None;
+			}
 			break;
-		case ActionAfterState_Sleep:
+		case ActionAfterState::Sleep:
 			if (!warned) {
 				ActionAfterWarnUser(m_actionAfterState);
 				return;
 			}
-			else
+			else {
 				SetSuspendState(false, false, true);
+				m_actionAfterState = ActionAfterState::None;
+			}
 			break;
 #elif defined(__WXMAC__)
-		case ActionAfterState_Reboot:
-		case ActionAfterState_Shutdown:
-		case ActionAfterState_Sleep:
+		case ActionAfterState::Reboot:
+		case ActionAfterState::Shutdown:
+		case ActionAfterState::Sleep:
 			if (!warned) {
 				ActionAfterWarnUser(m_actionAfterState);
 				return;
 			}
 			else {
 				wxString action;
-				if( m_actionAfterState == ActionAfterState_Reboot )
+				if( m_actionAfterState == ActionAfterState::Reboot )
 					action = _T("restart");
-				else if( m_actionAfterState == ActionAfterState_Shutdown )
+				else if( m_actionAfterState == ActionAfterState::Shutdown )
 					action = _T("shut down");
 				else
 					action = _T("sleep");
 				wxExecute(_T("osascript -e 'tell application \"System Events\" to ") + action + _T("'"));
+				m_actionAfterState = ActionAfterState::None;
 			}
 			break;
 #else
@@ -3027,22 +3012,21 @@ void CQueueView::ActionAfter(bool warned /*=false*/)
 			break;
 
 	}
-	m_actionAfterState = ActionAfterState_Disabled; // Resetting the state.
 }
 
 #if defined(__WXMSW__) || defined(__WXMAC__)
-void CQueueView::ActionAfterWarnUser(ActionAfterState s)
+void CQueueView::ActionAfterWarnUser(ActionAfterState::type s)
 {
 	if (m_actionAfterWarnDialog != NULL)
 		return;
 
 	wxString message;
 	wxString label;
-	if(s == ActionAfterState_Shutdown) {
+	if(s == ActionAfterState::Shutdown) {
 		message = _("The system will soon shut down unless you click Cancel.");
 		label = _("Shutdown now");
 	}
-	else if(s == ActionAfterState_Reboot) {
+	else if(s == ActionAfterState::Reboot) {
 		message = _("The system will soon reboot unless you click Cancel.");
 		label = _("Reboot now");
 	}
@@ -3073,33 +3057,29 @@ void CQueueView::ActionAfterWarnUser(ActionAfterState s)
 
 void CQueueView::OnActionAfterTimerTick()
 {
-	if (!m_actionAfterWarnDialog)
-	{
+	if (!m_actionAfterWarnDialog) {
 		delete m_actionAfterTimer;
 		m_actionAfterTimer = 0;
 		return;
 	}
 
 	bool skipped = false;
-	if (m_actionAfterTimerCount > 150)
-	{
+	if (m_actionAfterTimerCount > 150) {
 		m_actionAfterWarnDialog->Destroy();
 		m_actionAfterWarnDialog = 0;
 		delete m_actionAfterTimer;
 		m_actionAfterTimer = 0;
 		ActionAfter(true);
 	}
-	else if (!m_actionAfterWarnDialog->Update(m_actionAfterTimerCount++, _T(""), &skipped))
-	{
+	else if (!m_actionAfterWarnDialog->Update(m_actionAfterTimerCount++, _T(""), &skipped)) {
 		// User has pressed cancel!
-		m_actionAfterState = ActionAfterState_Disabled; // resetting to disabled
+		m_actionAfterState = ActionAfterState::None; // resetting to disabled
 		m_actionAfterWarnDialog->Destroy();
 		m_actionAfterWarnDialog = 0;
 		delete m_actionAfterTimer;
 		m_actionAfterTimer = 0;
 	}
-	else if (skipped)
-	{
+	else if (skipped) {
 		m_actionAfterWarnDialog->Destroy();
 		m_actionAfterWarnDialog = 0;
 		delete m_actionAfterTimer;
