@@ -4,7 +4,7 @@
 #include "dndobjects.h"
 #include "queue.h"
 
-#include <libfilezilla.h>
+#include <libfilezilla_engine.h>
 #include <option_change_event_handler.h>
 
 #include <set>
@@ -20,9 +20,9 @@ public:
 		dir,
 		file
 	};
-	const enum t_type m_type;
+	const t_type m_type;
 
-	CFolderProcessingEntry(enum t_type type) : m_type(type) {}
+	CFolderProcessingEntry(t_type type) : m_type(type) {}
 	virtual ~CFolderProcessingEntry() {}
 };
 
@@ -35,27 +35,28 @@ public:
 
 	wxString name;
 	int64_t size{};
-	CDateTime time;
+	fz::datetime time;
 	int attributes{};
 	bool dir{};
 };
 
-enum ActionAfterState
-{
-	ActionAfterState_Disabled,
-	ActionAfterState_Close,
-	ActionAfterState_Disconnect,
-	ActionAfterState_RunCommand,
-	ActionAfterState_ShowMessage,
-	ActionAfterState_PlaySound
-// On Windows and OS X, wx can reboot or shutdown the system as well.
-#if defined(__WXMSW__) || defined(__WXMAC__)
-	,
-	ActionAfterState_Reboot,
-	ActionAfterState_Shutdown,
-	ActionAfterState_Sleep
-#endif
+namespace ActionAfterState {
+enum type {
+	None,
+	ShowNotification,
+	RequestAttention,
+	Close,
+	RunCommand,
+	PlaySound,
+
+	// On Windows and OS X, wx can reboot or shutdown the system as well.
+	Reboot,
+	Shutdown,
+	Sleep,
+
+	Count
 };
+}
 
 class CStatusLineCtrl;
 class CFileItem;
@@ -110,6 +111,8 @@ class CAsyncRequestQueue;
 class CQueue;
 #if WITH_LIBDBUS
 class CDesktopNotification;
+#elif defined(__WXGTK__) || defined(__WXMSW__)
+class wxNotificationMessage;
 #endif
 
 class CQueueView : public CQueueViewBase, public COptionChangeEventHandler
@@ -124,7 +127,7 @@ public:
 	bool QueueFile(const bool queueOnly, const bool download,
 		const wxString& localFile, const wxString& remoteFile,
 		const CLocalPath& localPath, const CServerPath& remotePath,
-		const CServer& server, int64_t size, enum CEditHandler::fileType edit = CEditHandler::none,
+		const CServer& server, int64_t size, CEditHandler::fileType edit = CEditHandler::none,
 		QueuePriority priority = QueuePriority::normal);
 
 	void QueueFile_Finish(const bool start); // Need to be called after QueueFile
@@ -197,17 +200,15 @@ protected:
 		remove
 	};
 
-	enum ActionAfterState GetActionAfterState() const;
-
-	void ResetEngine(t_EngineData& data, const enum ResetReason reason);
+	void ResetEngine(t_EngineData& data, const ResetReason reason);
 	void DeleteEngines();
 
-	virtual bool RemoveItem(CQueueItem* item, bool destroy, bool updateItemCount = true, bool updateSelections = true);
+	virtual bool RemoveItem(CQueueItem* item, bool destroy, bool updateItemCount = true, bool updateSelections = true, bool forward = true);
 
 	// Stops processing of given item
 	// Returns true on success, false if it would block
 	bool StopItem(CFileItem* item);
-	bool StopItem(CServerItem* pServerItem);
+	bool StopItem(CServerItem* pServerItem, bool updateSelections);
 
 	void CheckQueueState();
 	bool IncreaseErrorCount(t_EngineData& engineData);
@@ -216,10 +217,10 @@ protected:
 	void DisplayQueueSize();
 	void SaveQueue();
 
-	bool IsActionAfter(enum ActionAfterState);
+	bool IsActionAfter(ActionAfterState::type);
 	void ActionAfter(bool warned = false);
 #if defined(__WXMSW__) || defined(__WXMAC__)
-	void ActionAfterWarnUser(ActionAfterState s);
+	void ActionAfterWarnUser(ActionAfterState::type s);
 #endif
 
 	void ProcessNotification(t_EngineData* pEngineData, std::unique_ptr<CNotification> && pNotification);
@@ -230,7 +231,7 @@ protected:
 	void TryRefreshListings();
 	CServer m_last_refresh_server;
 	CServerPath m_last_refresh_path;
-	CMonotonicClock m_last_refresh_listing_time;
+	fz::monotonic_clock m_last_refresh_listing_time;
 
 	// Called from Process Reply.
 	// After a disconnect, check if there's another idle engine that
@@ -254,26 +255,25 @@ protected:
 
 	void RemoveQueuedFolderItem(CFolderScanItem* pFolder);
 
-	CFolderProcessingThread *m_pFolderProcessingThread;
+	CFolderProcessingThread *m_pFolderProcessingThread{};
 
 	/*
 	 * Don't update status line positions if m_waitStatusLineUpdate is true.
 	 * This assures we are updating the status line positions only once,
 	 * and not multiple times (for example inside a loop).
 	 */
-	bool m_waitStatusLineUpdate;
+	bool m_waitStatusLineUpdate{};
 
 	// Remember last top item in UpdateStatusLinePositions()
-	int m_lastTopItem;
+	int m_lastTopItem{-1};
 
-	int m_activeCount;
-	int m_activeCountDown;
-	int m_activeCountUp;
-	int m_activeMode; // 0 inactive, 1 only immediate transfers, 2 all
-	int m_quit;
+	int m_activeCount{};
+	int m_activeCountDown{};
+	int m_activeCountUp{};
+	int m_activeMode{}; // 0 inactive, 1 only immediate transfers, 2 all
+	int m_quit{};
 
-	enum ActionAfterState m_actionAfterState;
-	wxString m_actionAfterRunCommand;
+	ActionAfterState::type m_actionAfterState;
 #if defined(__WXMSW__) || defined(__WXMAC__)
 	wxTimer* m_actionAfterTimer{};
 	wxProgressDialog* m_actionAfterWarnDialog{};
@@ -281,8 +281,8 @@ protected:
 	int m_actionAfterTimerId{-1};
 #endif
 
-	int64_t m_totalQueueSize;
-	int m_filesWithUnknownSize;
+	int64_t m_totalQueueSize{};
+	int m_filesWithUnknownSize{};
 
 	CMainFrame* m_pMainFrame;
 	CAsyncRequestQueue* m_pAsyncRequestQueue;
@@ -304,7 +304,9 @@ protected:
 	void ReleaseExclusiveEngineLock(CFileZillaEngine* pEngine);
 
 #if WITH_LIBDBUS
-	CDesktopNotification* m_desktop_notification;
+	std::unique_ptr<CDesktopNotification> m_desktop_notification;
+#elif defined(__WXGTK__) || defined(__WXMSW__)
+	std::unique_ptr<wxNotificationMessage> m_desktop_notification;
 #endif
 
 	CQueueStorage m_queue_storage;
