@@ -4,12 +4,13 @@
 #include "edithandler.h"
 #include "filezillaapp.h"
 #include "file_utils.h"
-#include "local_filesys.h"
 #include "Options.h"
 #include "queue.h"
 #include "window_state_manager.h"
 #include "xrc_helper.h"
 
+#include <libfilezilla/local_filesys.hpp>
+
 class CChangedFileDialog : public wxDialogEx
 {
 	DECLARE_EVENT_TABLE()
@@ -22,12 +23,12 @@ EVT_BUTTON(wxID_YES, CChangedFileDialog::OnYes)
 EVT_BUTTON(wxID_NO, CChangedFileDialog::OnNo)
 END_EVENT_TABLE()
 
-void CChangedFileDialog::OnYes(wxCommandEvent& event)
+void CChangedFileDialog::OnYes(wxCommandEvent&)
 {
 	EndDialog(wxID_YES);
 }
 
-void CChangedFileDialog::OnNo(wxCommandEvent& event)
+void CChangedFileDialog::OnNo(wxCommandEvent&)
 {
 	EndDialog(wxID_NO);
 }
@@ -101,7 +102,7 @@ void CEditHandler::RemoveTemporaryFilesInSpecificDir(wxString const& temp)
 		if (fd >= 0)
 		{
 			// Try to lock 1 byte region in the lockfile. m_type specifies the byte to lock.
-			struct flock f = { 0 };
+			struct flock f = {};
 			f.l_type = F_WRLCK;
 			f.l_whence = SEEK_SET;
 			f.l_start = 0;
@@ -178,7 +179,7 @@ wxString CEditHandler::GetLocalDirectory()
 	RemoveTemporaryFiles(dir);
 
 #ifdef __WXMSW__
-	m_lockfile_handle = ::CreateFile(m_localDir + _T("fz3temp-lockfile"), GENERIC_WRITE, 0, 0, CREATE_NEW, FILE_ATTRIBUTE_TEMPORARY, 0);
+	m_lockfile_handle = ::CreateFile((m_localDir + _T("fz3temp-lockfile")).wc_str(), GENERIC_WRITE, 0, 0, CREATE_NEW, FILE_ATTRIBUTE_TEMPORARY, 0);
 	if (m_lockfile_handle == INVALID_HANDLE_VALUE)
 	{
 		wxRmdir(m_localDir);
@@ -190,7 +191,7 @@ wxString CEditHandler::GetLocalDirectory()
 	if (m_lockfile_descriptor >= 0)
 	{
 		// Lock 1 byte region in the lockfile.
-		struct flock f = {0};
+		struct flock f = {};
 		f.l_type = F_WRLCK;
 		f.l_whence = SEEK_SET;
 		f.l_start = 0;
@@ -487,7 +488,7 @@ std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const
 	return iter;
 }
 
-void CEditHandler::FinishTransfer(bool successful, const wxString& fileName)
+void CEditHandler::FinishTransfer(bool, const wxString& fileName)
 {
 	auto iter = GetFile(fileName);
 	if (iter == m_fileDataList[local].end())
@@ -524,15 +525,13 @@ void CEditHandler::FinishTransfer(bool successful, const wxString& fileName, con
 	switch (iter->state)
 	{
 	case upload_and_remove:
-		if (successful)
-		{
+		if (successful) {
 			if (wxFileName::FileExists(iter->file) && !wxRemoveFile(iter->file))
 				iter->state = removing;
 			else
 				m_fileDataList[remote].erase(iter);
 		}
-		else
-		{
+		else {
 			if (!wxFileName::FileExists(iter->file))
 				m_fileDataList[remote].erase(iter);
 			else
@@ -546,8 +545,7 @@ void CEditHandler::FinishTransfer(bool successful, const wxString& fileName, con
 			m_fileDataList[remote].erase(iter);
 		break;
 	case download:
-		if (wxFileName::FileExists(iter->file))
-		{
+		if (wxFileName::FileExists(iter->file)) {
 			iter->state = edit;
 			if (StartEditing(remote, *iter))
 				break;
@@ -588,7 +586,7 @@ bool CEditHandler::StartEditing(enum CEditHandler::fileType type, t_fileData& da
 	wxASSERT(data.state == edit);
 
 	bool is_link;
-	if (CLocalFileSystem::GetFileInfo(data.file, is_link, 0, &data.modificationTime, 0) != CLocalFileSystem::file)
+	if (fz::local_filesys::get_file_info(fz::to_native(data.file), is_link, 0, &data.modificationTime, 0) != fz::local_filesys::file)
 		return false;
 
 	bool program_exists = false;
@@ -615,40 +613,37 @@ void CEditHandler::CheckForModifications(bool emitEvent)
 
 	insideCheckForModifications = true;
 
-	for (int i = 0; i < 2; i++)
-	{
+	for (int i = 0; i < 2; ++i) {
 checkmodifications_loopbegin:
-		for (auto iter = m_fileDataList[i].begin(); iter != m_fileDataList[i].end(); ++iter)
-		{
+		for (auto iter = m_fileDataList[i].begin(); iter != m_fileDataList[i].end(); ++iter) {
 			if (iter->state != edit)
 				continue;
 
-			CDateTime mtime;
+			fz::datetime mtime;
 			bool is_link;
-			if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file) {
+			if (fz::local_filesys::get_file_info(fz::to_native(iter->file), is_link, 0, &mtime, 0) != fz::local_filesys::file) {
 				m_fileDataList[i].erase(iter);
 
 				// Evil goto. Imo the next C++ standard needs a comefrom keyword.
 				goto checkmodifications_loopbegin;
 			}
 
-			if (!mtime.IsValid())
+			if (!mtime.empty())
 				continue;
 
-			if (iter->modificationTime.IsValid() && !iter->modificationTime.Compare(mtime))
+			if (iter->modificationTime.empty() && !iter->modificationTime.compare(mtime))
 				continue;
 
 			// File has changed, ask user what to do
 
 			m_busyTimer.Stop();
-			if( !wxDialogEx::CanShowPopupDialog() ) {
+			if (!wxDialogEx::CanShowPopupDialog() ) {
 				m_busyTimer.Start(1000, true);
 				insideCheckForModifications = false;
 				return;
 			}
 			wxTopLevelWindow* pTopWindow = (wxTopLevelWindow*)wxTheApp->GetTopWindow();
-			if (pTopWindow && pTopWindow->IsIconized())
-			{
+			if (pTopWindow && pTopWindow->IsIconized()) {
 				pTopWindow->RequestUserAttention(wxUSER_ATTENTION_INFO);
 				insideCheckForModifications = false;
 				return;
@@ -659,30 +654,24 @@ checkmodifications_loopbegin:
 			if (res == -1)
 				continue;
 
-			if (res == wxID_YES)
-			{
+			if (res == wxID_YES) {
 				UploadFile(CEditHandler::fileType(i), iter, remove);
 				goto checkmodifications_loopbegin;
 			}
-			else if (remove)
-			{
-				if (i == static_cast<int>(remote))
-				{
-					if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file || wxRemoveFile(iter->file))
-					{
+			else if (remove) {
+				if (i == static_cast<int>(remote)) {
+					if (fz::local_filesys::get_file_info(fz::to_native(iter->file), is_link, 0, &mtime, 0) != fz::local_filesys::file || wxRemoveFile(iter->file)) {
 						m_fileDataList[i].erase(iter);
 						goto checkmodifications_loopbegin;
 					}
 					iter->state = removing;
 				}
-				else
-				{
+				else {
 					m_fileDataList[i].erase(iter);
 					goto checkmodifications_loopbegin;
 				}
 			}
-			else if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file)
-			{
+			else if (fz::local_filesys::get_file_info(fz::to_native(iter->file), is_link, 0, &mtime, 0) != fz::local_filesys::file) {
 				m_fileDataList[i].erase(iter);
 				goto checkmodifications_loopbegin;
 			}
@@ -708,20 +697,19 @@ int CEditHandler::DisplayChangeNotification(CEditHandler::fileType type, std::li
 
 	dlg.SetChildLabel(XRCID("ID_FILENAME"), iter->name);
 
-	if (type == local)
-	{
+	if (type == local) {
 		XRCCTRL(dlg, "ID_DESC_OPENEDAS", wxStaticText)->Hide();
 		XRCCTRL(dlg, "ID_OPENEDAS", wxStaticText)->Hide();
+
+		dlg.SetChildLabel("ID_DELETE", _T("&Finish editing"));
 	}
-	else
-	{
+	else {
 		wxString file = iter->file;
 		int pos = file.Find(wxFileName::GetPathSeparator(), true);
 		wxASSERT(pos != -1);
 		file = file.Mid(pos + 1);
 
-		if (file == iter->name)
-		{
+		if (file == iter->name) {
 			XRCCTRL(dlg, "ID_DESC_OPENEDAS", wxStaticText)->Hide();
 			XRCCTRL(dlg, "ID_OPENEDAS", wxStaticText)->Hide();
 		}
@@ -766,17 +754,16 @@ bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterato
 	iter->state = unedit ? upload_and_remove : upload;
 
 	int64_t size;
-	CDateTime mtime;
+	fz::datetime mtime;
 
 	bool is_link;
-	if (CLocalFileSystem::GetFileInfo(iter->file, is_link, &size, &mtime, 0) != CLocalFileSystem::file)
-	{
+	if (fz::local_filesys::get_file_info(fz::to_native(iter->file), is_link, &size, &mtime, 0) != fz::local_filesys::file) {
 		m_fileDataList[type].erase(iter);
 		return false;
 	}
 
-	if (!mtime.IsValid())
-		mtime = CDateTime::Now();
+	if (!mtime.empty())
+		mtime = fz::datetime::now();
 
 	iter->modificationTime = mtime;
 
@@ -796,7 +783,7 @@ bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterato
 	return true;
 }
 
-void CEditHandler::OnTimerEvent(wxTimerEvent& event)
+void CEditHandler::OnTimerEvent(wxTimerEvent&)
 {
 #ifdef __WXMSW__
 	// Don't check for changes if mouse is captured,
@@ -848,33 +835,30 @@ wxString CEditHandler::CanOpen(enum CEditHandler::fileType type, const wxString&
 
 wxString CEditHandler::GetOpenCommand(const wxString& file, bool& program_exists)
 {
-	if (!COptions::Get()->GetOptionVal(OPTION_EDIT_ALWAYSDEFAULT))
-	{
+	if (!COptions::Get()->GetOptionVal(OPTION_EDIT_ALWAYSDEFAULT)) {
 		const wxString command = GetCustomOpenCommand(file, program_exists);
 		if (!command.empty())
 			return command;
 
-		if (COptions::Get()->GetOptionVal(OPTION_EDIT_INHERITASSOCIATIONS))
-		{
-			const wxString command = GetSystemOpenCommand(file, program_exists);
-			if (!command.empty())
-				return command;
+		if (COptions::Get()->GetOptionVal(OPTION_EDIT_INHERITASSOCIATIONS)) {
+			const wxString sysCommand = GetSystemOpenCommand(file, program_exists);
+			if (!sysCommand.empty())
+				return sysCommand;
 		}
 	}
 
 	wxString command = COptions::Get()->GetOption(OPTION_EDIT_DEFAULTEDITOR);
 	if (command.empty() || command[0] == '0')
 		return wxString(); // None set
-	else if (command[0] == '1')
-	{
+	else if (command[0] == '1') {
 		// Text editor
 		const wxString random = _T("5AC2EE515D18406 space aB77C2C60F1F88952.txt"); // Chosen by fair dice roll. Guaranteed to be random.
-		wxString command = GetSystemOpenCommand(random, program_exists);
-		if (command.empty() || !program_exists)
-			return command;
+		wxString sysCommand = GetSystemOpenCommand(random, program_exists);
+		if (sysCommand.empty() || !program_exists)
+			return sysCommand;
 
-		command.Replace(random, file);
-		return command;
+		sysCommand.Replace(random, file);
+		return sysCommand;
 	}
 	else if (command[0] == '2')
 		command = command.Mid(1);
@@ -950,7 +934,7 @@ wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_
 	return wxString();
 }
 
-void CEditHandler::OnChangedFileEvent(wxCommandEvent& event)
+void CEditHandler::OnChangedFileEvent(wxCommandEvent&)
 {
 	CheckForModifications();
 }
@@ -1217,12 +1201,12 @@ void CEditHandlerStatusDialog::SetCtrlState()
 	XRCCTRL(*this, "ID_EDIT", wxWindow)->Enable(select);
 }
 
-void CEditHandlerStatusDialog::OnSelectionChanged(wxListEvent& event)
+void CEditHandlerStatusDialog::OnSelectionChanged(wxListEvent&)
 {
 	SetCtrlState();
 }
 
-void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent& event)
+void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent&)
 {
 	CEditHandler* const pEditHandler = CEditHandler::Get();
 	if (!pEditHandler)
@@ -1232,13 +1216,11 @@ void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent& event)
 
 	std::list<int> files;
 	int item = -1;
-	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-	{
+	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		pListCtrl->SetItemState(item, 0, wxLIST_STATE_SELECTED);
 		enum CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(item, type);
-		if (pData->state != CEditHandler::edit && pData->state != CEditHandler::upload_and_remove_failed)
-		{
+		if (pData->state != CEditHandler::edit && pData->state != CEditHandler::upload_and_remove_failed) {
 			wxBell();
 			return;
 		}
@@ -1246,23 +1228,19 @@ void CEditHandlerStatusDialog::OnUnedit(wxCommandEvent& event)
 		files.push_front(item);
 	}
 
-	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
-	{
+	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 		const int i = *iter;
 
 		enum CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(i, type);
 
-		if (type == CEditHandler::local)
-		{
+		if (type == CEditHandler::local) {
 			pEditHandler->Remove(pData->file);
 			delete pData;
 			pListCtrl->DeleteItem(i);
 		}
-		else
-		{
-			if (pEditHandler->Remove(pData->name, pData->remotePath, pData->server))
-			{
+		else {
+			if (pEditHandler->Remove(pData->name, pData->remotePath, pData->server)) {
 				delete pData;
 				pListCtrl->DeleteItem(i);
 			}
@@ -1284,23 +1262,20 @@ void CEditHandlerStatusDialog::OnUpload(wxCommandEvent& event)
 
 	std::list<int> files;
 	int item = -1;
-	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-	{
+	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		pListCtrl->SetItemState(item, 0, wxLIST_STATE_SELECTED);
 
 		enum CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(item, type);
 
-		if (pData->state != CEditHandler::edit && pData->state != CEditHandler::upload_and_remove_failed)
-		{
+		if (pData->state != CEditHandler::edit && pData->state != CEditHandler::upload_and_remove_failed) {
 			wxBell();
 			return;
 		}
 		files.push_front(item);
 	}
 
-	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
-	{
+	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 		const int i = *iter;
 
 		enum CEditHandler::fileType type;
@@ -1324,7 +1299,7 @@ void CEditHandlerStatusDialog::OnUpload(wxCommandEvent& event)
 	SetCtrlState();
 }
 
-void CEditHandlerStatusDialog::OnEdit(wxCommandEvent& event)
+void CEditHandlerStatusDialog::OnEdit(wxCommandEvent&)
 {
 	CEditHandler* const pEditHandler = CEditHandler::Get();
 	if (!pEditHandler)
@@ -1334,34 +1309,28 @@ void CEditHandlerStatusDialog::OnEdit(wxCommandEvent& event)
 
 	std::list<int> files;
 	int item = -1;
-	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-	{
+	while ((item = pListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		pListCtrl->SetItemState(item, 0, wxLIST_STATE_SELECTED);
 
 		enum CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(item, type);
 
-		if (pData->state != CEditHandler::edit)
-		{
+		if (pData->state != CEditHandler::edit) {
 			wxBell();
 			return;
 		}
 		files.push_front(item);
 	}
 
-	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
-	{
+	for (std::list<int>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 		const int i = *iter;
 
 		enum CEditHandler::fileType type;
 		CEditHandler::t_fileData* pData = GetDataFromItem(i, type);
 
-		if (type == CEditHandler::local)
-		{
-			if (!pEditHandler->StartEditing(pData->file))
-			{
-				if (pEditHandler->Remove(pData->file))
-				{
+		if (type == CEditHandler::local) {
+			if (!pEditHandler->StartEditing(pData->file)) {
+				if (pEditHandler->Remove(pData->file)) {
 					delete pData;
 					pListCtrl->DeleteItem(i);
 				}
@@ -1369,12 +1338,9 @@ void CEditHandlerStatusDialog::OnEdit(wxCommandEvent& event)
 					pListCtrl->SetItem(i, COLUMN_STATUS, _("Pending removal"));
 			}
 		}
-		else
-		{
-			if (!pEditHandler->StartEditing(pData->name, pData->remotePath, pData->server))
-			{
-				if (pEditHandler->Remove(pData->name, pData->remotePath, pData->server))
-				{
+		else {
+			if (!pEditHandler->StartEditing(pData->name, pData->remotePath, pData->server)) {
+				if (pEditHandler->Remove(pData->name, pData->remotePath, pData->server)) {
 					delete pData;
 					pListCtrl->DeleteItem(i);
 				}
@@ -1433,15 +1399,16 @@ bool CNewAssociationDialog::Run(const wxString &file)
 	else
 		m_ext.clear();
 
-	wxStaticText *pDesc = XRCCTRL(*this, "ID_DESC", wxStaticText);
-	pDesc->SetLabel(wxString::Format(pDesc->GetLabel(), m_ext));
+	wxStaticText *const pDesc = XRCCTRL(*this, "ID_DESC", wxStaticText);
+	if (pDesc) {
+		pDesc->SetLabel(wxString::Format(pDesc->GetLabel(), m_ext));
+	}
 
 	bool program_exists = false;
 	wxString cmd = GetSystemOpenCommand(_T("foo.txt"), program_exists);
 	if (!program_exists)
 		cmd.clear();
-	if (!cmd.empty())
-	{
+	if (!cmd.empty()) {
 		wxString args;
 		if (!UnquoteCommand(cmd, args))
 			cmd.clear();
@@ -1450,17 +1417,17 @@ bool CNewAssociationDialog::Run(const wxString &file)
 	if (!PathExpand(cmd))
 		cmd.clear();
 
-	pDesc = XRCCTRL(*this, "ID_EDITOR_DESC", wxStaticText);
-	if (cmd.empty())
-	{
-		XRCCTRL(*this, "ID_USE_CUSTOM", wxRadioButton)->SetValue(true);
-		XRCCTRL(*this, "ID_USE_EDITOR", wxRadioButton)->Enable(false);
-		pDesc->Hide();
+	if (cmd.empty()) {
+		xrc_call(*this, "ID_USE_CUSTOM", &wxRadioButton::SetValue, true);
+		xrc_call(*this, "ID_USE_EDITOR", &wxRadioButton::Enable, false);
+		xrc_call(*this, "ID_EDITOR_DESC", &wxStaticText::Hide);
 	}
-	else
-	{
-		XRCCTRL(*this, "ID_EDITOR_DESC_NONE", wxStaticText)->Hide();
-		pDesc->SetLabel(wxString::Format(pDesc->GetLabel(), cmd));
+	else {
+		xrc_call(*this, "ID_EDITOR_DESC_NONE", &wxStaticText::Hide);
+		wxStaticText* const pEditorDesc = XRCCTRL(*this, "ID_EDITOR_DESC", wxStaticText);
+		if (pEditorDesc) {
+			pEditorDesc->SetLabel(wxString::Format(pEditorDesc->GetLabel(), cmd));
+		}
 	}
 
 	SetCtrlState();
@@ -1475,25 +1442,24 @@ bool CNewAssociationDialog::Run(const wxString &file)
 
 void CNewAssociationDialog::SetCtrlState()
 {
-	wxRadioButton* pCustom = wxDynamicCast(FindWindow(XRCID("ID_USE_CUSTOM")), wxRadioButton);
-	if (!pCustom)
-	{
+	wxRadioButton* pCustom = dynamic_cast<wxRadioButton*>(FindWindow(XRCID("ID_USE_CUSTOM")));
+	if (!pCustom) {
 		// Return since it can get called before dialog got fully loaded
 		return;
 	}
 
-	const bool custom = pCustom->GetValue();
+	bool const custom = pCustom->GetValue();
 
-	XRCCTRL(*this, "ID_CUSTOM", wxTextCtrl)->Enable(custom);
-	XRCCTRL(*this, "ID_BROWSE", wxButton)->Enable(custom);
+	xrc_call(*this, "ID_CUSTOM", &wxTextCtrl::Enable, custom);
+	xrc_call(*this, "ID_BROWSE", &wxButton::Enable, custom);
 }
 
-void CNewAssociationDialog::OnRadioButton(wxCommandEvent& event)
+void CNewAssociationDialog::OnRadioButton(wxCommandEvent&)
 {
 	SetCtrlState();
 }
 
-void CNewAssociationDialog::OnOK(wxCommandEvent& event)
+void CNewAssociationDialog::OnOK(wxCommandEvent&)
 {
 	const bool custom = XRCCTRL(*this, "ID_USE_CUSTOM", wxRadioButton)->GetValue();
 	const bool always = XRCCTRL(*this, "ID_ALWAYS", wxCheckBox)->GetValue();
@@ -1560,7 +1526,7 @@ void CNewAssociationDialog::OnOK(wxCommandEvent& event)
 	EndModal(wxID_OK);
 }
 
-void CNewAssociationDialog::OnBrowseEditor(wxCommandEvent& event)
+void CNewAssociationDialog::OnBrowseEditor(wxCommandEvent&)
 {
 	wxFileDialog dlg(this, _("Select default editor"), _T(""), _T(""),
 #ifdef __WXMSW__
@@ -1579,8 +1545,7 @@ void CNewAssociationDialog::OnBrowseEditor(wxCommandEvent& event)
 	if (editor.empty())
 		return;
 
-	if (!ProgramExists(editor))
-	{
+	if (!ProgramExists(editor)) {
 		XRCCTRL(*this, "ID_EDITOR", wxWindow)->SetFocus();
 		wxMessageBoxEx(_("Selected editor does not exist."), _("File not found"), wxICON_EXCLAMATION, this);
 		return;
