@@ -34,7 +34,7 @@ CMenuBar::~CMenuBar()
 
 CMenuBar* CMenuBar::Load(CMainFrame* pMainFrame)
 {
-	CMenuBar* menubar = wxDynamicCast(wxXmlResource::Get()->LoadMenuBar(_T("ID_MENUBAR")), CMenuBar);
+	CMenuBar* menubar = dynamic_cast<CMenuBar*>(wxXmlResource::Get()->LoadMenuBar(_T("ID_MENUBAR")));
 	if (!menubar)
 		return 0;
 
@@ -114,13 +114,13 @@ CMenuBar* CMenuBar::Load(CMainFrame* pMainFrame)
 
 	pMainFrame->Connect(wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(CMenuBar::OnMenuEvent), 0, menubar);
 
-	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_REMOTE_IDLE, true, true);
-	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_SERVER, true, true);
-	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_SYNC_BROWSE, true, true);
-	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_COMPARISON, true, true);
+	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_REMOTE_IDLE, true);
+	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_SERVER, true);
+	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_SYNC_BROWSE, true);
+	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_COMPARISON, true);
 
-	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_QUEUEPROCESSING, false, false);
-	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_CHANGEDCONTEXT, false, false);
+	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_QUEUEPROCESSING, false);
+	CContextManager::Get()->RegisterHandler(menubar, STATECHANGE_CHANGEDCONTEXT, false);
 
 	menubar->RegisterOption(OPTION_ASCIIBINARY);
 	menubar->RegisterOption(OPTION_PRESERVE_TIMESTAMPS);
@@ -264,7 +264,7 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 			return;
 
 		pState->SetSyncBrowse(false);
-		if (!pData->m_remoteDir.empty() && pState->IsRemoteIdle()) {
+		if (!pData->m_remoteDir.empty() && pState->IsRemoteIdle(true)) {
 			const CServer* pServer = pState->GetServer();
 			if (!pServer || *pServer != pData->m_server) {
 				m_pMainFrame->ConnectToSite(*pData);
@@ -286,48 +286,47 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 	}
 
 	std::map<int, wxString>::const_iterator iter2 = m_bookmark_menu_id_map_global.find(event.GetId());
-	if (iter2 != m_bookmark_menu_id_map_global.end())
-	{
+	if (iter2 != m_bookmark_menu_id_map_global.end()) {
 		// We hit a global bookmark
 		wxString local_dir;
 		CServerPath remote_dir;
 		bool sync;
-		if (!CBookmarksDialog::GetBookmark(iter2->second, local_dir, remote_dir, sync))
+		bool comparison;
+		if (!CBookmarksDialog::GetBookmark(iter2->second, local_dir, remote_dir, sync, comparison))
 			return;
 
 		pState->SetSyncBrowse(false);
-		if (!remote_dir.empty() && pState->IsRemoteIdle())
-		{
+		if (!remote_dir.empty() && pState->IsRemoteIdle(true)) {
 			const CServer* pServer = pState->GetServer();
-			if (pServer)
-			{
+			if (pServer) {
 				CServerPath current_remote_path = pState->GetRemotePath();
-				if (!current_remote_path.empty() && current_remote_path.GetType() != remote_dir.GetType())
-				{
+				if (!current_remote_path.empty() && current_remote_path.GetType() != remote_dir.GetType()) {
 					wxMessageBoxEx(_("Selected global bookmark and current server use a different server type.\nUse site-specific bookmarks for this server."), _("Bookmark"), wxICON_EXCLAMATION, this);
 					return;
 				}
 				pState->ChangeRemoteDir(remote_dir);
 			}
 		}
-		if (!local_dir.empty())
-		{
+		if (!local_dir.empty()) {
 			bool set = pState->SetLocalDir(local_dir);
 
-			if (set && sync)
-			{
+			if (set && sync) {
 				wxASSERT(!remote_dir.empty());
 				pState->SetSyncBrowse(true, remote_dir);
 			}
 		}
 
+		if (comparison) {
+			pState->GetComparisonManager()->CompareListings();
+		}
+
 		return;
 	}
 
 	event.Skip();
 }
 
-void CMenuBar::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
+void CMenuBar::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
 {
 	switch (notification)
 	{
@@ -447,7 +446,6 @@ void CMenuBar::UpdateMenubarState()
 	Enable(XRCID("ID_CANCEL"), pServer && !idle);
 	Enable(XRCID("ID_MENU_SERVER_CMD"), pServer && idle);
 	Enable(XRCID("ID_MENU_FILE_COPYSITEMANAGER"), pServer != 0);
-	Enable(XRCID("ID_TOOLBAR_COMPARISON"), pServer != 0);
 	Enable(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), pServer != 0);
 	Enable(XRCID("ID_MENU_SERVER_SEARCH"), pServer && idle);
 
@@ -457,16 +455,14 @@ void CMenuBar::UpdateMenubarState()
 	bool canReconnect;
 	if (pServer || !idle)
 		canReconnect = false;
-	else
-	{
+	else {
 		CServer tmp;
 		canReconnect = !pState->GetLastServer().GetHost().empty();
 	}
 	Enable(XRCID("ID_MENU_SERVER_RECONNECT"), canReconnect);
 
 	wxMenuItem* pItem = FindItem(XRCID("ID_MENU_TRANSFER_TYPE"));
-	if (!pServer || CServer::ProtocolHasDataTypeConcept(pServer->GetProtocol()))
-	{
+	if (!pServer || CServer::ProtocolHasDataTypeConcept(pServer->GetProtocol())) {
 		pItem->Enable(true);
 	}
 	else
