@@ -26,82 +26,42 @@ void CQueueItem::SetPriority(QueuePriority priority)
 
 void CQueueItem::AddChild(CQueueItem* item)
 {
+	wxASSERT(GetType() != QueueItemType::Folder);
+	wxASSERT(GetType() != QueueItemType::Status);
+
 	if (m_removed_at_front) {
 		m_children.erase(m_children.begin(), m_children.begin() + m_removed_at_front);
 		m_removed_at_front = 0;
 	}
 	m_children.push_back(item);
 
-	m_visibleOffspring += 1 + item->m_visibleOffspring;
-	m_maxCachedIndex = -1;
 	CQueueItem* parent = GetParent();
-	while (parent)
-	{
-		parent->m_visibleOffspring += 1 + item->m_visibleOffspring;
-		parent->m_maxCachedIndex = -1;
+	while (parent) {
+		if (parent->GetType() == QueueItemType::Server) {
+			static_cast<CServerItem*>(parent)->m_visibleOffspring += 1 + item->GetChildrenCount(true);
+			static_cast<CServerItem*>(parent)->m_maxCachedIndex = -1;
+		}
 		parent = parent->GetParent();
 	}
 }
 
-CQueueItem* CQueueItem::GetChild(unsigned int item, bool recursive /*=true*/)
+
+CQueueItem* CQueueItem::GetChild(unsigned int item, bool recursive)
 {
-	std::vector<CQueueItem*>::iterator iter;
-	if (!recursive)
-	{
+	if (!recursive) {
 		if (item + m_removed_at_front >= m_children.size())
 			return 0;
-		iter = m_children.begin();
-		iter += item + m_removed_at_front;
-		return *iter;
+		return *(m_children.begin() + item + m_removed_at_front);
 	}
 
-	if ((int)item <= m_maxCachedIndex)
-	{
-		// Index is cached
-		iter = m_children.begin() + m_removed_at_front;
-		iter += m_lookupCache[item].child;
-		item -= m_lookupCache[item].index;
-		if (!item)
-			return *iter;
-		else
-			return (*iter)->GetChild(item - 1);
-	}
-
-	int index;
-	int child;
-	iter = m_children.begin() + m_removed_at_front;
-	if (m_maxCachedIndex == -1)
-	{
-		child = 0;
-		index = 0;
-	}
-	else
-	{
-		// Start with loop with the last cached item index
-		iter += m_lookupCache[m_maxCachedIndex].child + 1;
-		item -= m_maxCachedIndex + 1;
-		index = m_maxCachedIndex + 1;
-		child = m_lookupCache[m_maxCachedIndex].child + 1;
-	}
-
-	for (; iter != m_children.end(); ++iter, ++child)
-	{
+	std::vector<CQueueItem*>::iterator iter = m_children.begin() + m_removed_at_front;
+	for (; iter != m_children.end(); ++iter) {
 		if (!item)
 			return *iter;
 
 		unsigned int count = (*iter)->GetChildrenCount(true);
-		if (item > count)
-		{
-			if (m_maxCachedIndex == -1 && m_lookupCache.size() < (unsigned int)m_visibleOffspring)
-				m_lookupCache.resize(m_visibleOffspring);
-			for (unsigned int k = index; k <= index + count; k++)
-			{
-				m_lookupCache[k].child = child;
-				m_lookupCache[k].index = index;
-			}
-			m_maxCachedIndex = index + count;
+		if (item > count) {
 			item -= count + 1;
-			index += count + 1;
 			continue;
 		}
 
@@ -110,29 +70,36 @@ CQueueItem* CQueueItem::GetChild(unsigned int item, bool recursive /*=true*/)
 	return 0;
 }
 
-unsigned int CQueueItem::GetChildrenCount(bool recursive)
+unsigned int CQueueItem::GetChildrenCount(bool recursive) const
 {
-	if (!recursive)
-		return m_children.size() - m_removed_at_front;
+	unsigned int count{};
+	if (!recursive) {
+		count = m_children.size() - m_removed_at_front;
+	}
+	else {
+		for (auto iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter) {
+			count += 1 + (*iter)->GetChildrenCount(true);
+		}
+	}
 
-	return m_visibleOffspring;
+	return count;
 }
 
-bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
+bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy, bool forward)
 {
-	int oldVisibleOffspring = m_visibleOffspring;
+	int const oldVisibleOffspring = GetChildrenCount(true);
+	int visibleOffspring = oldVisibleOffspring;
+
 	bool deleted = false;
-	for (auto iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
-	{
-		if (*iter == pItem)
-		{
-			m_visibleOffspring -= 1;
-			m_visibleOffspring -= pItem->m_visibleOffspring;
+
+	auto doRemove = [&](std::vector<CQueueItem*>::iterator iter) {
+		if (*iter == pItem) {
+			visibleOffspring -= 1;
+			visibleOffspring -= pItem->GetChildrenCount(true);
 			if (destroy)
 				delete pItem;
 
-			if (iter - m_children.begin() - m_removed_at_front <= 10)
-			{
+			if (iter - m_children.begin() - m_removed_at_front <= 10) {
 				m_removed_at_front++;
 				unsigned int end = iter - m_children.begin();
 				int c = 0;
@@ -143,20 +110,17 @@ bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
 				m_children.erase(iter);
 
 			deleted = true;
-			break;
+			return;
 		}
 
-		int visibleOffspring = (*iter)->m_visibleOffspring;
-		if ((*iter)->RemoveChild(pItem, destroy))
-		{
-			m_visibleOffspring -= visibleOffspring - (*iter)->m_visibleOffspring;
-			if (!((*iter)->m_children.size() - (*iter)->m_removed_at_front))
-			{
-				m_visibleOffspring -= 1;
+		int childVisibleOffspring = (*iter)->GetChildrenCount(true);
+		if ((*iter)->RemoveChild(pItem, destroy)) {
+			visibleOffspring -= childVisibleOffspring - (*iter)->GetChildrenCount(true);
+			if (!((*iter)->m_children.size() - (*iter)->m_removed_at_front)) {
+				visibleOffspring -= 1;
 				delete *iter;
 
-				if (iter - m_children.begin() - m_removed_at_front <= 10)
-				{
+				if (iter - m_children.begin() - m_removed_at_front <= 10) {
 					m_removed_at_front++;
 					unsigned int end = iter - m_children.begin();
 					for (int i = end; i >= m_removed_at_front; i--)
@@ -167,60 +131,46 @@ bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
 			}
 
 			deleted = true;
-			break;
+		}
+	};
+
+	if (forward) {
+		for (auto iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter ) {
+			doRemove(iter);
+			if (deleted) {
+				break;
+			}
+		}
+	}
+	else {
+		for (auto iter = m_children.rbegin(); iter != m_children.rend() - m_removed_at_front; ++iter ) {
+			doRemove(iter.base() - 1);
+			if (deleted) {
+				break;
+			}
 		}
 	}
 
 	if (!deleted)
 		return false;
 
-	m_maxCachedIndex = -1;
+	if (GetType() == QueueItemType::Server) {
+		static_cast<CServerItem*>(this)->m_visibleOffspring = visibleOffspring;
+	}
 
 	// Propagate new children count to parent
 	CQueueItem* parent = GetParent();
-	while (parent)
-	{
-		parent->m_maxCachedIndex = -1;
-		parent->m_visibleOffspring -= oldVisibleOffspring - m_visibleOffspring;
+	while (parent) {
+		if (parent->GetType() == QueueItemType::Server) {
+			static_cast<CServerItem*>(parent)->m_maxCachedIndex = -1;
+			static_cast<CServerItem*>(parent)->m_visibleOffspring -= oldVisibleOffspring - visibleOffspring;
+		}
 		parent = parent->GetParent();
 	}
 
 	return true;
 }
 
-bool CQueueItem::TryRemoveAll()
-{
-	const int oldVisibleOffspring = m_visibleOffspring;
-	std::vector<CQueueItem*>::iterator iter;
-	std::vector<CQueueItem*> keepChildren;
-	m_visibleOffspring = 0;
-	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
-	{
-		CQueueItem* pItem = *iter;
-		if (pItem->TryRemoveAll())
-			delete pItem;
-		else
-		{
-			keepChildren.push_back(pItem);
-			m_visibleOffspring++;
-			m_visibleOffspring += pItem->GetChildrenCount(true);
-		}
-	}
-	m_children = keepChildren;
-	m_maxCachedIndex = -1;
-	m_removed_at_front = 0;
-
-	CQueueItem* parent = GetParent();
-	while (parent)
-	{
-		parent->m_maxCachedIndex = -1;
-		parent->m_visibleOffspring -= oldVisibleOffspring - m_visibleOffspring;
-		parent = parent->GetParent();
-	}
-
-	return m_children.empty();
-}
-
 CQueueItem* CQueueItem::GetTopLevelItem()
 {
 	if (!m_parent)
@@ -228,8 +178,7 @@ CQueueItem* CQueueItem::GetTopLevelItem()
 
 	CQueueItem* newParent = m_parent;
 	CQueueItem* parent = 0;
-	while (newParent)
-	{
+	while (newParent) {
 		parent = newParent;
 		newParent = newParent->GetParent();
 	}
@@ -276,7 +225,7 @@ CFileItem::CFileItem(CServerItem* parent, bool queued, bool download,
 					 const CLocalPath& localPath, const CServerPath& remotePath, int64_t size)
 	: CQueueItem(parent)
 	, m_sourceFile(sourceFile)
-	, m_targetFile(targetFile.empty() ? CSparseOptional<fzstring>() : CSparseOptional<fzstring>(to_fzstring(targetFile)))
+	, m_targetFile(targetFile.empty() ? fz::sparse_optional<std::wstring>() : fz::sparse_optional<std::wstring>(to_wstring(targetFile)))
 	, m_localPath(localPath)
 	, m_remotePath(remotePath)
 	, m_size(size)
@@ -317,6 +266,7 @@ QueuePriority CFileItem::GetPriority() const
 void CFileItem::SetActive(const bool active)
 {
 	if (active && !IsActive()) {
+		wxASSERT(!GetChildrenCount(false));
 		AddChild(new CStatusItem);
 		flags |= flag_active;
 	}
@@ -361,7 +311,7 @@ bool CFileItem::TryRemoveAll()
 void CFileItem::SetTargetFile(wxString const& file)
 {
 	if (!file.empty() && file != m_sourceFile)
-		m_targetFile = CSparseOptional<fzstring>(to_fzstring(file));
+		m_targetFile = fz::sparse_optional<std::wstring>(to_wstring(file));
 	else
 		m_targetFile.clear();
 }
@@ -447,9 +397,22 @@ wxString CServerItem::GetName() const
 void CServerItem::AddChild(CQueueItem* pItem)
 {
 	CQueueItem::AddChild(pItem);
+	m_maxCachedIndex = -1;
+	m_visibleOffspring += 1 + pItem->GetChildrenCount(true);
 	if (pItem->GetType() == QueueItemType::File ||
 		pItem->GetType() == QueueItemType::Folder)
 		AddFileItemToList((CFileItem*)pItem);
+
+	wxASSERT(m_visibleOffspring >= static_cast<int>(m_children.size()) - m_removed_at_front);
+	wxASSERT(((m_children.size() - m_removed_at_front) != 0) == (m_visibleOffspring != 0));
+}
+
+unsigned int CServerItem::GetChildrenCount(bool recursive) const
+{
+	if (!recursive)
+		return m_children.size() - m_removed_at_front;
+
+	return m_visibleOffspring;
 }
 
 void CServerItem::AddFileItemToList(CFileItem* pItem)
@@ -460,13 +423,23 @@ void CServerItem::AddFileItemToList(CFileItem* pItem)
 	m_fileList[pItem->queued() ? 0 : 1][static_cast<int>(pItem->GetPriority())].push_back(pItem);
 }
 
-void CServerItem::RemoveFileItemFromList(CFileItem* pItem)
+void CServerItem::RemoveFileItemFromList(CFileItem* pItem, bool forward)
 {
-	std::list<CFileItem*>& fileList = m_fileList[pItem->queued() ? 0 : 1][static_cast<int>(pItem->GetPriority())];
-	for (auto iter = fileList.begin(); iter != fileList.end(); ++iter) {
-		if (*iter == pItem) {
-			fileList.erase(iter);
-			return;
+	std::deque<CFileItem*>& fileList = m_fileList[pItem->queued() ? 0 : 1][static_cast<int>(pItem->GetPriority())];
+	if (forward) {
+		for (auto iter = fileList.begin(); iter != fileList.end(); ++iter) {
+			if (*iter == pItem) {
+				fileList.erase(iter);
+				return;
+			}
+		}
+	}
+	else {
+		for (auto iter = fileList.rbegin(); iter != fileList.rend(); ++iter) {
+			if (*iter == pItem) {
+				fileList.erase(iter.base() - 1);
+				return;
+			}
 		}
 	}
 	wxFAIL_MSG(_T("File item not deleted from m_fileList"));
@@ -492,8 +465,68 @@ void CServerItem::SetDefaultFileExistsAction(CFileExistsNotification::OverwriteA
 	}
 }
 
+CQueueItem* CServerItem::GetChild(unsigned int item, bool recursive)
+{
+	std::vector<CQueueItem*>::iterator iter;
+	if (!recursive) {
+		if (item + m_removed_at_front >= m_children.size())
+			return 0;
+		iter = m_children.begin();
+		iter += item + m_removed_at_front;
+		return *iter;
+	}
+
+	if ((int)item <= m_maxCachedIndex) {
+		// Index is cached
+		iter = m_children.begin() + m_removed_at_front;
+		iter += m_lookupCache[item].child;
+		item -= m_lookupCache[item].index;
+		if (!item)
+			return *iter;
+		else
+			return (*iter)->GetChild(item - 1);
+	}
+
+	int index;
+	int child;
+	iter = m_children.begin() + m_removed_at_front;
+	if (m_maxCachedIndex == -1) {
+		child = 0;
+		index = 0;
+	}
+	else {
+		// Start with loop with the last cached item index
+		iter += m_lookupCache[m_maxCachedIndex].child + 1;
+		item -= m_maxCachedIndex + 1;
+		index = m_maxCachedIndex + 1;
+		child = m_lookupCache[m_maxCachedIndex].child + 1;
+	}
+
+	for (; iter != m_children.end(); ++iter, ++child) {
+		if (!item)
+			return *iter;
+
+		unsigned int count = (*iter)->GetChildrenCount(true);
+		if (item > count) {
+			if (m_maxCachedIndex == -1 && m_lookupCache.size() < (unsigned int)m_visibleOffspring)
+				m_lookupCache.resize(m_visibleOffspring);
+			for (unsigned int k = index; k <= index + count; ++k) {
+				m_lookupCache[k].child = child;
+				m_lookupCache[k].index = index;
+			}
+			m_maxCachedIndex = index + count;
+			item -= count + 1;
+			index += count + 1;
+			continue;
+		}
+
+		return (*iter)->GetChild(item - 1);
+	}
+	return 0;
+}
+
 namespace {
-CFileItem* DoGetIdleChild(std::list<CFileItem*> const* fileList, TransferDirection direction)
+CFileItem* DoGetIdleChild(std::deque<CFileItem*> const* fileList, TransferDirection direction)
 {
 	int i = 0;
 	for (i = static_cast<int>(QueuePriority::count) - 1; i >= 0; --i) {
@@ -526,37 +559,43 @@ CFileItem* CServerItem::GetIdleChild(bool immediateOnly, TransferDirection direc
 	return item;
 }
 
-bool CServerItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
+bool CServerItem::RemoveChild(CQueueItem* pItem, bool destroy, bool forward)
 {
 	if (!pItem)
 		return false;
 
-	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder)
-	{
+	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder) {
 		CFileItem* pFileItem = static_cast<CFileItem*>(pItem);
-		RemoveFileItemFromList(pFileItem);
+		RemoveFileItemFromList(pFileItem, forward);
 	}
 
-	return CQueueItem::RemoveChild(pItem, destroy);
+	bool removed = CQueueItem::RemoveChild(pItem, destroy, forward);
+	if (removed) {
+		m_maxCachedIndex = -1;
+	}
+	
+	wxASSERT(m_visibleOffspring >= static_cast<int>(m_children.size()) - m_removed_at_front);
+	wxASSERT(((m_children.size() - m_removed_at_front) != 0) == (m_visibleOffspring != 0));
+
+	return removed;
 }
 
 void CServerItem::QueueImmediateFiles()
 {
 	for (int i = 0; i < static_cast<int>(QueuePriority::count); ++i) {
-		std::list<CFileItem*> activeList;
-		std::list<CFileItem*>& fileList = m_fileList[1][i];
-		for (std::list<CFileItem*>::reverse_iterator iter = fileList.rbegin(); iter != fileList.rend(); ++iter) {
+		std::deque<CFileItem*> activeList;
+		std::deque<CFileItem*>& fileList = m_fileList[1][i];
+		for (auto iter = fileList.rbegin(); iter != fileList.rend(); ++iter) {
 			CFileItem* item = *iter;
 			wxASSERT(!item->queued());
 			if (item->IsActive())
 				activeList.push_front(item);
-			else
-			{
+			else {
 				item->set_queued(true);
 				m_fileList[0][i].push_front(item);
 			}
 		}
-		fileList = activeList;
+		std::swap(fileList, activeList);
 	}
 }
 
@@ -565,7 +604,7 @@ void CServerItem::QueueImmediateFile(CFileItem* pItem)
 	if (pItem->queued())
 		return;
 
-	std::list<CFileItem*>& fileList = m_fileList[1][static_cast<int>(pItem->GetPriority())];
+	std::deque<CFileItem*>& fileList = m_fileList[1][static_cast<int>(pItem->GetPriority())];
 	for (auto iter = fileList.begin(); iter != fileList.end(); ++iter) {
 		if (*iter != pItem)
 			continue;
@@ -592,9 +631,8 @@ int64_t CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, i
 	int64_t totalSize = 0;
 	for (int i = 0; i < static_cast<int>(QueuePriority::count); ++i) {
 		for (int j = 0; j < 2; ++j) {
-			const std::list<CFileItem*>& fileList = m_fileList[j][i];
-			for (std::list<CFileItem*>::const_iterator iter = fileList.begin(); iter != fileList.end(); ++iter) {
-				const CFileItem* item = *iter;
+			const std::deque<CFileItem*>& fileList = m_fileList[j][i];
+			for (auto const& item : fileList) {
 				int64_t size = item->GetSize();
 				if (size >= 0)
 					totalSize += size;
@@ -617,6 +655,8 @@ int64_t CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, i
 
 bool CServerItem::TryRemoveAll()
 {
+	wxASSERT(!GetParent());
+
 	const int oldVisibleOffspring = m_visibleOffspring;
 	std::vector<CQueueItem*>::iterator iter;
 	std::vector<CQueueItem*> keepChildren;
@@ -626,7 +666,7 @@ bool CServerItem::TryRemoveAll()
 		if (pItem->TryRemoveAll()) {
 			if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder) {
 				CFileItem* pFileItem = static_cast<CFileItem*>(pItem);
-				RemoveFileItemFromList(pFileItem);
+				RemoveFileItemFromList(pFileItem, true);
 			}
 			delete pItem;
 		}
@@ -636,14 +676,14 @@ bool CServerItem::TryRemoveAll()
 			m_visibleOffspring += pItem->GetChildrenCount(true);
 		}
 	}
-	m_children = keepChildren;
+	std::swap(m_children, keepChildren);
 	m_removed_at_front = 0;
 
-	CQueueItem* parent = GetParent();
-	while (parent) {
-		parent->m_visibleOffspring -= oldVisibleOffspring - m_visibleOffspring;
-		parent = parent->GetParent();
-	}
+	m_maxCachedIndex = -1;
+
+	wxASSERT(oldVisibleOffspring >= m_visibleOffspring);
+	wxASSERT(m_visibleOffspring >= static_cast<int>(m_children.size()));
+	(void)oldVisibleOffspring;
 
 	return m_children.empty();
 }
@@ -674,8 +714,10 @@ void CServerItem::SetPriority(QueuePriority priority)
 
 	for (int i = 0; i < 2; ++i)
 		for (int j = 0; j < static_cast<int>(QueuePriority::count); ++j) {
-			if (j != static_cast<int>(priority))
-				m_fileList[i][static_cast<int>(priority)].splice(m_fileList[i][static_cast<int>(priority)].end(), m_fileList[i][j]);
+			if (j != static_cast<int>(priority)) {
+				std::move(m_fileList[i][j].begin(), m_fileList[i][j].end(), std::back_inserter(m_fileList[i][static_cast<int>(priority)]));
+				m_fileList[i][j].clear();
+			}
 		}
 }
 
@@ -683,8 +725,7 @@ void CServerItem::SetChildPriority(CFileItem* pItem, QueuePriority oldPriority,
 {
 	int i = pItem->queued() ? 0 : 1;
 
-	for (auto iter = m_fileList[i][static_cast<int>(oldPriority)].begin(); iter != m_fileList[i][static_cast<int>(oldPriority)].end(); ++iter)
-	{
+	for (auto iter = m_fileList[i][static_cast<int>(oldPriority)].begin(); iter != m_fileList[i][static_cast<int>(oldPriority)].end(); ++iter) {
 		if (*iter != pItem)
 			continue;
 
@@ -1285,7 +1326,7 @@ void CQueueViewBase::InsertItem(CServerItem* pServerItem, CQueueItem* pItem)
 	}
 }
 
-bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItemCount /*=true*/, bool updateSelections /*=true*/)
+bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItemCount, bool updateSelections, bool forward)
 {
 	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder) {
 		wxASSERT(m_fileCount > 0);
@@ -1306,7 +1347,7 @@ bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItem
 	CQueueItem* topLevelItem = pItem->GetTopLevelItem();
 
 	int count = topLevelItem->GetChildrenCount(true);
-	topLevelItem->RemoveChild(pItem, destroy);
+	topLevelItem->RemoveChild(pItem, destroy, forward);
 
 	bool didRemoveParent;
 
@@ -1397,13 +1438,12 @@ void CQueueViewBase::OnChar(wxKeyEvent& event)
 	m_pQueue->SetSelection(selection);
 }
 
-void CQueueViewBase::OnEndColumnDrag(wxListEvent& event)
+void CQueueViewBase::OnEndColumnDrag(wxListEvent&)
 {
-	for (unsigned int i = 0; i < m_pQueue->GetPageCount(); i++)
-	{
+	for (unsigned int i = 0; i < m_pQueue->GetPageCount(); ++i) {
 		wxWindow *page = m_pQueue->GetPage(i);
 
-		wxListCtrl* queue_page = wxDynamicCast(page, wxListCtrl);
+		wxListCtrl* queue_page = dynamic_cast<wxListCtrl*>(page);
 		if (!queue_page || queue_page == this)
 			continue;
 
