@@ -39,7 +39,7 @@ bool CQuickconnectBar::Create(CMainFrame* pParent)
 
 #ifdef __WXMAC__
 	// Under OS X default buttons are toplevel window wide, where under Windows / GTK they stop at the parent panel.
-	wxTopLevelWindow *tlw = wxDynamicCast(wxGetTopLevelParent(pParent), wxTopLevelWindow);
+	wxTopLevelWindow *tlw = dynamic_cast<wxTopLevelWindow*>(wxGetTopLevelParent(pParent));
 	if (tlw)
 		tlw->SetDefaultItem(0);
 #endif
@@ -117,10 +117,9 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 	else
 		m_pPass->SetValue(_T(""));
 
-	if (protocol == HTTP || protocol == HTTPS)
-	{
-		wxString error = _("Invalid protocol specified. Valid protocols are:\nftp:// for normal FTP with optional encryption,\nsftp:// for SSH file transfer protocol,\nftps:// for FTP over TLS (implicit) and\nftpes:// for FTP over TLS (explicit).");
-		wxMessageBoxEx(error, _("Syntax error"), wxICON_EXCLAMATION);
+	if (protocol == HTTP || protocol == HTTPS) {
+		wxString protocolError = _("Invalid protocol specified. Valid protocols are:\nftp:// for normal FTP with optional encryption,\nsftp:// for SSH file transfer protocol,\nftps:// for FTP over TLS (implicit) and\nftpes:// for FTP over TLS (explicit).");
+		wxMessageBoxEx(protocolError, _("Syntax error"), wxICON_EXCLAMATION);
 		return;
 	}
 
@@ -130,13 +129,17 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 	if (server.GetLogonType() != ANONYMOUS && !CAskSavePasswordDialog::Run(this))
 		return;
 
+	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && server.GetLogonType() == NORMAL) {
+		server.SetLogonType(ASK);
+		CLoginManager::Get().RememberPassword(server);
+	}
 	if (!m_pMainFrame->ConnectToServer(server, path))
 		return;
 
 	CRecentServerList::SetMostRecentServer(server);
 }
 
-void CQuickconnectBar::OnQuickconnectDropdown(wxCommandEvent& event)
+void CQuickconnectBar::OnQuickconnectDropdown(wxCommandEvent&)
 {
 	wxMenu* pMenu = new wxMenu;
 
@@ -190,8 +193,7 @@ void CQuickconnectBar::OnMenu(wxCommandEvent& event)
 	std::advance(iter, index);
 
 	CServer server = *iter;
-	if (server.GetLogonType() == ASK)
-	{
+	if (server.GetLogonType() == ASK) {
 		if (!CLoginManager::Get().GetPassword(server, false))
 			return;
 	}
