@@ -30,20 +30,20 @@ CToolBar* CToolBar::Load(CMainFrame* pMainFrame)
 		wxToolBarXmlHandlerEx::SetIconSize(iconSize);
 	}
 
-	CToolBar* toolbar = wxDynamicCast(wxXmlResource::Get()->LoadToolBar(pMainFrame, _T("ID_TOOLBAR")), CToolBar);
+	CToolBar* toolbar = dynamic_cast<CToolBar*>(wxXmlResource::Get()->LoadToolBar(pMainFrame, _T("ID_TOOLBAR")));
 	if (!toolbar)
 		return 0;
 
 	toolbar->m_pMainFrame = pMainFrame;
 
-	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_REMOTE_IDLE, true, true);
-	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_SERVER, true, true);
-	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_SYNC_BROWSE, true, true);
-	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_COMPARISON, true, true);
-	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_APPLYFILTER, true, false);
+	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_REMOTE_IDLE, true);
+	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_SERVER, true);
+	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_SYNC_BROWSE, true);
+	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_COMPARISON, true);
+	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_APPLYFILTER, true);
 
-	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_QUEUEPROCESSING, false, false);
-	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_CHANGEDCONTEXT, false, false);
+	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_QUEUEPROCESSING, false);
+	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_CHANGEDCONTEXT, false);
 
 	toolbar->RegisterOption(OPTION_SHOW_MESSAGELOG);
 	toolbar->RegisterOption(OPTION_SHOW_QUEUE);
@@ -122,7 +122,6 @@ void CToolBar::UpdateToolbarState()
 
 	EnableTool(XRCID("ID_TOOLBAR_DISCONNECT"), pServer && idle);
 	EnableTool(XRCID("ID_TOOLBAR_CANCEL"), pServer && !idle);
-	EnableTool(XRCID("ID_TOOLBAR_COMPARISON"), pServer != 0);
 	EnableTool(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), pServer != 0);
 	EnableTool(XRCID("ID_TOOLBAR_FIND"), pServer && idle);
 
